@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    $Id: blockmean.c,v 1.19 2004/04/25 09:10:45 pwessel Exp $
+ *    $Id: blockmean.c,v 1.28 2006/02/18 02:51:06 pwessel Exp $
  *
- *	Copyright (c) 1991-2004 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2006 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -35,6 +35,7 @@
    Modified:	3.3.5: 10 Jul 2000 by PW to add -L
    Version:	3.4: 01-MAR-2001 by PW, Use -F instead of -N, and add -C
    Version:	4: 01-AUG-2001 by PW, Added -f
+   Version:	4.1: 14-SEP-2005 by PW, Added enhanced -I
 */
 
 #include "gmt.h"
@@ -44,7 +45,7 @@ struct	WXYZ {
 	double	x;
 	double	y;
 	double	z;
-}	*wxyz;	
+}	*wxyz;
 
 struct LIST {
 	int		key;
@@ -54,30 +55,31 @@ struct LIST {
 }	**hash, *this_box;
 
 int	set_up_arrays(int n_x, int n_y, int *method);
-void	write_output_and_free_space (int n_x, int n_y, int method, int report_weight, BOOLEAN mean_xy, double west, double east, double delta_x, double south, double north, double delta_y, double xy_off, int *n_cells_filled);
+void	write_output_and_free_space (struct GRD_HEADER *h, int method, int report_weight, BOOLEAN mean_xy, int *n_cells_filled);
 struct	LIST *find_box(int i, int j, int method);
 int sum_only = 0;
 
-main (int argc, char **argv)
+int main (int argc, char **argv)
 {
 
-	BOOLEAN	error, weighted, offset, report_weight, nofile = TRUE, done = FALSE, first = TRUE, mean_xy = TRUE;
-	
+	BOOLEAN	error, weighted, report_weight, nofile = TRUE, done = FALSE, first = TRUE, mean_xy = TRUE;
+
 	FILE *fp = NULL;
-	
-	double	west, east, south, north, delta_x, delta_y, del_off, xy_off, *in;
-	double	idx, idy, weight;
-	
-	int	i, j, method, n_x, n_y, ij, n_expected_fields, n_fields, n_req;
+
+	double	weight, *in, wesn[4];
+
+	int	i, j, method, ij, n_expected_fields, n_fields, n_req;
 	int	n_lost, n_read, n_pitched, n_cells_filled, n_files = 0, fno, n_args;
-	
+
 	char	modifier, line[BUFSIZ], format[BUFSIZ];
+	
+	struct GRD_HEADER h;
 
 	argc = GMT_begin (argc, argv);
 
-	west = east = south = north = delta_x = delta_y = 0.0;
-	del_off = 0.5;
-	error = weighted = offset = report_weight = FALSE;
+	GMT_grd_init (&h, argc, argv, FALSE);
+	
+	error = weighted = report_weight = FALSE;
 
 	for (i = 1; i < argc; i++) {
 		if (argv[i][0] == '-') {
@@ -92,8 +94,8 @@ main (int argc, char **argv)
 				case 'b':
 				case 'f':
 				case '\0':
-                                      error += GMT_get_common_args (argv[i], &west, &east, &south, &north);
-                                      break;
+					error += GMT_get_common_args (argv[i], &h.x_min, &h.x_max, &h.y_min, &h.y_max);
+					break;
                               
 				/* Supplemental parameters */
                               
@@ -101,7 +103,10 @@ main (int argc, char **argv)
 					mean_xy = FALSE;	/* Report center of block instead */
 					break;
 				case 'I':
-					GMT_getinc (&argv[i][2], &delta_x, &delta_y);
+					if (GMT_getinc (&argv[i][2], &h.x_inc, &h.y_inc)) {
+						GMT_inc_syntax ('I', 1);
+						error = TRUE;
+					}
 					break;
 				case 'L':	/* Obsolete, but backward compatibility prevails [use -f instead] */
 					GMT_io.in_col_type[0] = GMT_io.out_col_type[0] = GMT_IS_LON;
@@ -110,7 +115,7 @@ main (int argc, char **argv)
 					break;
 				case 'N':	/* Backward compatible with 3.3.6 */
 				case 'F':
-					offset = TRUE;
+					h.node_offset = 1;
 					break;
 				case 'S':
 					switch (argv[i][2]) {
@@ -134,7 +139,7 @@ main (int argc, char **argv)
 					else
 						weighted = report_weight = TRUE;
 					break;
-					
+
 				default:
 					error = TRUE;
 					GMT_default_error (argv[i][1]);
@@ -147,14 +152,12 @@ main (int argc, char **argv)
 
 	if (argc == 1 || GMT_quick) {
 		fprintf (stderr, "blockmean %s - Block averaging by L2 norm\n\n", GMT_VERSION);
-		fprintf (stderr, "usage: blockmean [infile(s)] -I<xinc[m]>[/<yinc>[m]] -R<west/east/south/north>\n");
+		fprintf (stderr, "usage: blockmean [infile(s)] -I<xinc[u][!|+]>[/<yinc>[u][!|+]] -R<west/east/south/north>\n");
 		fprintf (stderr, "\t[-C] [-F] [-H[<nrec>]] [-S[w|z]] [-V] [-W[i][o]] [-:] [-bi[s][<n>]]\n\t[-bo[s][<n>]] [-f[i|o]<colinfo>]\n\n");
-		
+
 		if (GMT_quick) exit (EXIT_FAILURE);
-		
-		fprintf (stderr, "\t-I sets Increment of the grid; enter xinc, optionally xinc/yinc.\n");
-		fprintf (stderr, "\t   Default is yinc = xinc.  Append an m [or c] to xinc or yinc to indicate minutes [or seconds],\n");
-		fprintf (stderr, "\t   e.g., -I10m/5m grids longitude every 10 minutes, latitude every 5 minutes.\n");
+
+		GMT_inc_syntax ('I', 0);
 		GMT_explain_option ('R');
 		fprintf (stderr, "\n\tOPTIONS:\n");
 		fprintf (stderr, "\t-C Output center of block and mean z-value.  [Default outputs mean x-y location]\n");
@@ -176,16 +179,17 @@ main (int argc, char **argv)
 		GMT_explain_option ('.');
 		exit (EXIT_FAILURE);
 	}
-	
+
 	if (!project_info.region_supplied) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR:  Must specify -R option\n", GMT_program);
 		error++;
 	}
-	if (delta_x <= 0.0 || delta_y <= 0.0) {
+	GMT_RI_prepare (&h);	/* Ensure -R -I consistency and set nx, ny */
+	if (h.x_inc <= 0.0 || h.y_inc <= 0.0) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -I option.  Must specify positive increment(s)\n", GMT_program);
 		error++;
 	}
-	
+
 	if (GMT_io.binary[GMT_IN] && gmtdefs.io_header[GMT_IN]) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data cannot have header -H\n", GMT_program);
 		error++;
@@ -199,7 +203,7 @@ main (int argc, char **argv)
 	if (error) exit (EXIT_FAILURE);
 
 	GMT_put_history (argc, argv);	/* Update .gmtcommands4 */
-	
+
 	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
 		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
@@ -208,47 +212,40 @@ main (int argc, char **argv)
 #ifdef SET_IO_MODE
 	GMT_setmode (GMT_OUT);
 #endif
-	idx = 1.0 / delta_x;
-	idy = 1.0 / delta_y;
-	n_x = irint ((east - west) * idx) + 1;
-	n_y = irint ((north - south) * idy) + 1;
-	
-	if (offset) {
-		n_x--;
-		n_y--;
-		del_off = 0.0;
-	}
-	xy_off = 0.5 - del_off;	/* Use to calculate mean location of block */
-	
-	if ( (set_up_arrays(n_x, n_y, &method) ) ) exit (EXIT_FAILURE);
 	
+	h.xy_off = 0.5 * h.node_offset;	/* Use to calculate mean location of block */
+
+	if ( (set_up_arrays(h.nx, h.ny, &method) ) ) exit (EXIT_FAILURE);
+
 	if (gmtdefs.verbose) {
 		sprintf (format, "%%s: W: %s E: %s S: %s N: %s nx: %%d ny: %%d\n", gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format);
-		fprintf (stderr, format, GMT_program, west, east, south, north, n_x, n_y);
+		fprintf (stderr, format, GMT_program, h.x_min, h.x_max, h.y_min, h.y_max, h.nx, h.ny);
 		if (method) {
-			fprintf (stderr, "%s: Direct method requires %lu bytes for array.\n", GMT_program, (unsigned long)n_x*n_y*sizeof(struct WXYZ));
+			fprintf (stderr, "%s: Direct method requires %lu bytes for array.\n", GMT_program, (unsigned long)h.nx*h.ny*sizeof(struct WXYZ));
 			fprintf (stderr, "%s: Constructing hash table and linked lists.\n", GMT_program);
 		}
 		else {
 			fprintf (stderr, "%s: Region fits inside core memory.\n", GMT_program);
-			fprintf (stderr, "%s: Using %lu bytes for array.\n", GMT_program, (unsigned long)n_x*n_y*sizeof(struct WXYZ));
+			fprintf (stderr, "%s: Using %lu bytes for array.\n", GMT_program, (unsigned long)h.nx*h.ny*sizeof(struct WXYZ));
 		}
 	}
 
+	GMT_set_xy_domain (wesn, &h);	/* May include some padding if gridline-registered */
+
 	n_read = n_pitched = 0;
 
 	n_expected_fields = (GMT_io.binary[GMT_IN]) ? GMT_io.ncol[GMT_IN] : 3 + weighted;
-	
+
 	if (n_files > 0)
 		nofile = FALSE;
 	else
 		n_files = 1;
 
 	n_args = (argc > 1) ? argc : 2;
-	
+
 	for (fno = 1; !done && fno < n_args; fno++) {	/* Loop over input files, if any */
 		if (!nofile && argv[fno][0] == '-') continue;
-		
+
 		if (nofile) {	/* Just read standard input */
 			fp = GMT_stdin;
 			done = TRUE;
@@ -262,7 +259,7 @@ main (int argc, char **argv)
 		}
 
 		if (!nofile && gmtdefs.verbose) fprintf (stderr, "%s: Working on file %s\n", GMT_program, argv[fno]);
-		
+
 		if (gmtdefs.io_header[GMT_IN]) {
 			for (i = 0; i < gmtdefs.n_header_recs; i++) {
 				GMT_fgets (line, BUFSIZ, fp);
@@ -271,7 +268,7 @@ main (int argc, char **argv)
 			}
 			first = FALSE;
 		}
-				
+
 		while ((n_fields = GMT_input (fp, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
 
 			if (GMT_io.status & GMT_IO_MISMATCH) {
@@ -280,30 +277,33 @@ main (int argc, char **argv)
 			}
 
 			if (GMT_is_dnan (in[2])) continue;	/* Skip when z = NaN */
-			
+
 			n_read++;
-			
-			i = GMT_x_to_i (in[0], west, idx, del_off, n_x);
-			if ( i < 0 || i >= n_x ) continue;
-			j = GMT_y_to_j (in[1], south, idy, del_off, n_y);
-			if ( j < 0 || j >= n_y ) continue;
+
+			if (GMT_y_is_outside (in[1],  wesn[2], wesn[3])) continue;	/* Outside y-range */
+			if (GMT_x_is_outside (&in[0], wesn[0], wesn[1])) continue;	/* Outside x-range */
+
+			i = GMT_x_to_i (in[0], h.x_min, h.x_inc, h.xy_off, h.nx);
+			if ( i < 0 || i >= h.nx ) continue;
+			j = GMT_y_to_j (in[1], h.y_min, h.y_inc, h.xy_off, h.ny);
+			if ( j < 0 || j >= h.ny ) continue;
 			weight = (weighted) ? in[3] : 1.0;
 			if (method) {
 				this_box = find_box(i, j, method);
-				this_box->key = (n_x > n_y) ? j : i;
+				this_box->key = (h.nx > h.ny) ? j : i;
 				if (mean_xy) {
 					this_box->sum.x += (in[0]*weight);
 					this_box->sum.y += (in[1]*weight);
 				}
 				else if (this_box->sum.w == 0.0) {	/* So we only do this once */
-					this_box->sum.x = GMT_i_to_x (i,  west,  east, delta_x, xy_off, n_x);
-					this_box->sum.y = GMT_j_to_y (j, south, north, delta_y, xy_off, n_y);
+					this_box->sum.x = GMT_i_to_x (i, h.x_min, h.x_max, h.x_inc, h.xy_off, h.nx);
+					this_box->sum.y = GMT_j_to_y (j, h.y_min, h.y_max, h.y_inc, h.xy_off, h.ny);
 				}
 				this_box->sum.w += weight;
 				this_box->sum.z += (in[2]*weight);
 			}
 			else {
-				ij = i * n_y + j;
+				ij = i * h.ny + j;
 				wxyz[ij].w += weight;
 				if (mean_xy) {
 					wxyz[ij].x += (in[0]*weight);
@@ -314,10 +314,10 @@ main (int argc, char **argv)
 			n_pitched++;
 		}
 		if (fp != GMT_stdin) GMT_fclose(fp);
-		
+
 	}
 
-	write_output_and_free_space (n_x, n_y, method, report_weight, mean_xy, west, east, delta_x, south, north, delta_y, xy_off, &n_cells_filled);
+	write_output_and_free_space (&h, method, report_weight, mean_xy, &n_cells_filled);
 
 	n_lost = n_read - n_pitched;
 	if (gmtdefs.verbose) {
@@ -326,6 +326,8 @@ main (int argc, char **argv)
 	}
 
         GMT_end (argc, argv);
+
+	exit (EXIT_SUCCESS);
 }
 
 int	set_up_arrays(int n_x, int n_y, int *method)
@@ -351,39 +353,39 @@ int	set_up_arrays(int n_x, int n_y, int *method)
 	return(0);
 }
 
-void	write_output_and_free_space (int n_x, int n_y, int method, int report_weight, BOOLEAN mean_xy, double west, double east, double delta_x, double south, double north, double delta_y, double xy_off, int *n_cells_filled)
+void	write_output_and_free_space (struct GRD_HEADER *h, int method, int report_weight, BOOLEAN mean_xy, int *n_cells_filled)
 {
 	int	loop, limit, n_out, i, j;
 	double	out[4], iw;
-	
+
 	n_out = (report_weight) ? 4 : 3;
 
 	*n_cells_filled = 0;
-	
+
 	if (gmtdefs.verbose) fprintf (stderr, "%s: Calculating block means\n", GMT_program);
-	
+
 	if (method) {	/* We have to loop over the linked lists  */
-	
+
 		if (method == 1)
-			limit = n_x;
+			limit = h->nx;
 		else
-			limit = n_y;
-		
+			limit = h->ny;
+
 		for (loop = 0; loop < limit; loop++) {
 
 			/* If this list is empty, continue:  */
-			
+
 			if (hash[loop] == NULL) continue;
-			
+
 			/* Go to the leftmost box in the list  */
-			
+
 			this_box = hash[loop];
 			while (this_box->p_left) this_box = this_box->p_left;
-			
+
 			/* While we are at a box, write it, and move right  */
-			
+
 			while (this_box) {
-			
+
 				(*n_cells_filled)++;
 				out[3] = this_box->sum.w;
 				iw = 1.0 / out[3];
@@ -402,7 +404,7 @@ void	write_output_and_free_space (int n_x, int n_y, int method, int report_weigh
 					out[2] = this_box->sum.z * iw;
 
 				GMT_output (GMT_stdout, n_out, out);
-				
+
 				if (this_box->p_right) {
 					this_box = this_box->p_right;
 					GMT_free ((void *)this_box->p_left);
@@ -415,15 +417,15 @@ void	write_output_and_free_space (int n_x, int n_y, int method, int report_weigh
 		}
 		GMT_free ((void *)hash);
 	}
-	
+
 	else {	/* We have a simple array in memory  */
-	
-		limit = n_x * n_y;
-		
+
+		limit = h->nx * h->ny;
+
 		for (loop = 0; loop < limit; loop++) {
-		
+
 			if (wxyz[loop].w == 0.0) continue;
-			
+
 			(*n_cells_filled)++;
 			out[3] = wxyz[loop].w;
 			iw = 1.0 / out[3];
@@ -432,10 +434,10 @@ void	write_output_and_free_space (int n_x, int n_y, int method, int report_weigh
 				out[1] = wxyz[loop].y * iw;
 			}
 			else {	/* Use block center */
-				j = loop % n_y;
-				i = loop / n_y;
-				out[0] = GMT_i_to_x (i,  west,  east, delta_x, xy_off, n_x);
-				out[1] = GMT_j_to_y (j, south, north, delta_y, xy_off, n_y);
+				j = loop % h->ny;
+				i = loop / h->ny;
+				out[0] = GMT_i_to_x (i, h->x_min, h->x_max, h->x_inc, h->xy_off, h->nx);
+				out[1] = GMT_j_to_y (j, h->y_min, h->y_max, h->y_inc, h->xy_off, h->ny);
 			}
 			if (sum_only) {
 				out[2] = (sum_only == 2) ? wxyz[loop].w : wxyz[loop].z;
@@ -445,11 +447,11 @@ void	write_output_and_free_space (int n_x, int n_y, int method, int report_weigh
 
 			GMT_output (GMT_stdout, n_out, out);
 		}
-		
+
 		free ((void *)wxyz);	/* Free because allocated with calloc instead of GMT_memory */
 	}
 }
-		
+
 struct LIST *find_box(int i, int j, int method)
 {
 	int	hash_key, list_key;
@@ -458,11 +460,11 @@ struct LIST *find_box(int i, int j, int method)
 	static int last_i = -1;
 	static int last_j = -1;
 	static struct LIST *last_box = NULL;
-	
+
 	if ( (i == last_i) && (j == last_j) ) return (last_box);
-	
+
 	/* Get here when we have to search  */
-	
+
 	if (method > 0) {
 		hash_key = i;
 		list_key = j;
@@ -471,29 +473,29 @@ struct LIST *find_box(int i, int j, int method)
 		hash_key = j;
 		list_key = i;
 	}
-	
+
 	current_box = hash[hash_key];
-	
+
 	if (current_box) {
-	
+
 		/* Hash table has an entry into a linked list;
 			if it doesn't match the list_key, search the list */
-		
+
 		if (current_box->key < list_key) {
-		
+
 			/* Current's key too low; move right while necessary  */
-		
+
 			while ( (current_box->p_right) && (current_box->p_right->key <= list_key) )
 				current_box = current_box->p_right;
-			
+
 			if (current_box->key < list_key) {
-			
+
 				/* Current's key still too low  */
-			
+
 				if (current_box->p_right) {
-				
+
 					/* Next's key too high; insert a box in between  */
-					
+
 					temp_box = (struct LIST *) GMT_memory (VNULL, (size_t)1, sizeof (struct LIST), GMT_program);
 					temp_box->p_right = current_box->p_right;
 					current_box->p_right->p_left = temp_box;
@@ -503,7 +505,7 @@ struct LIST *find_box(int i, int j, int method)
 				}
 				else {
 					/* There is no next; make new box at end of list  */
-					
+
 					temp_box = (struct LIST *) GMT_memory (VNULL, (size_t)1, sizeof (struct LIST), GMT_program);
 					temp_box->p_left = current_box;
 					current_box->p_right = temp_box;
@@ -511,22 +513,22 @@ struct LIST *find_box(int i, int j, int method)
 				}
 			}
 		}
-		
+
 		if (current_box->key > list_key) {
-		
+
 			/* Current's key too high; move left while necessary  */
-		
+
 			while ( (current_box->p_left) && (current_box->p_left->key >= list_key) )
 				current_box = current_box->p_left;
-				
+
 			if (current_box->key > list_key) {
-			
+
 				/* Current's key still too high  */
-			
+
 				if (current_box->p_left) {
-				
+
 					/* Next's key too low; insert a box in between  */
-					
+
 					temp_box = (struct LIST *) GMT_memory (VNULL, (size_t)1, sizeof (struct LIST), GMT_program);
 					temp_box->p_left = current_box->p_left;
 					current_box->p_left->p_right = temp_box;
@@ -536,7 +538,7 @@ struct LIST *find_box(int i, int j, int method)
 				}
 				else {
 					/* There is no next; make new box at end of list  */
-					
+
 					temp_box = (struct LIST *) GMT_memory (VNULL, (size_t)1, sizeof (struct LIST), GMT_program);
 					temp_box->p_right = current_box;
 					current_box->p_left = temp_box;
@@ -544,19 +546,19 @@ struct LIST *find_box(int i, int j, int method)
 				}
 			}
 		}
-			
 
-			
-		
-		
+
+
+
+
 	}
 	else  {	/* Hash table is NULL; create first box in linked list  */
-	
+
 		current_box = (struct LIST *) GMT_memory (VNULL, (size_t)1, sizeof (struct LIST), GMT_program);
 	}
-		
+
 	/* Now we set the state of the static variables and return  */
-	
+
 	last_i = i;
 	last_j = j;
 	last_box = current_box;
