@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    $Id: filter1d.c,v 1.17 2004/08/14 00:38:19 pwessel Exp $
+ *    $Id: filter1d.c,v 1.23 2005/12/17 05:59:21 pwessel Exp $
  *
- *	Copyright (c) 1991-2004 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2006 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -76,13 +76,13 @@ double	*min_scl;
 double	*max_scl;
 double	*last_scl;
 double	*this_scl;
-double	**work;			/* Pointer to array of pointers to doubles for work  */	
-double	**data;			/* Pointer to array of pointers to doubles for data  */	
+double	**work;			/* Pointer to array of pointers to doubles for work  */
+double	**data;			/* Pointer to array of pointers to doubles for data  */
 double	dt;			/* Delta time resolution for filter computation  */
 double	q_factor;		/* Quality level for mean weights or n in median  */
 double filter_width;		/* Full width of filter in user's units */
-double half_width;		
-double ignore_val;		/* Value to skip over in input stream  */	
+double half_width;
+double ignore_val;		/* Value to skip over in input stream  */
 double t_start;			/* x-value of first output point */
 double t_stop;			/* x-value of last output point */
 double t_int;			/* Output interval */
@@ -126,20 +126,20 @@ FILE	*fp_wt = NULL;		/* File pointer to custom weight coefficients (optional) */
 
 int GMT_mode_selection = 0, GMT_n_multiples = 0;
 
-main (int argc, char **argv)
+int main (int argc, char **argv)
 {
 
 	int	i;
 	char	c;
-	
+
 	argc = GMT_begin (argc, argv);
 
 	for (i = 1; i < argc; i++) {
 		if (argv[i][0] == '-') {
 			switch (argv[i][1]) {
-		
+
 				/* Common parameters */
-			
+
 				case 'H':
 				case 'V':
 				case ':':
@@ -148,9 +148,9 @@ main (int argc, char **argv)
 				case '\0':
 					error += GMT_get_common_args (argv[i], 0, 0, 0, 0);
 					break;
-				
+
 				/* Supplemental parameters */
-				
+
 				case 'F':	/* Filter selection  */
 					switch (argv[i][2]) {
 						case 'B':
@@ -223,7 +223,7 @@ main (int argc, char **argv)
 							break;
 					}
 					break;
-					
+
 				case 'D':
 					dt = atof (&argv[i][2]);
 					equidist = FALSE;
@@ -275,9 +275,9 @@ main (int argc, char **argv)
 		fprintf (stderr, "usage: filter1d [infile] -F<type><width>[<mode>] [-D<increment>] [-E] [-H[<nrec>]]\n");
 		fprintf (stderr, "\t[-I<ignore_val>] [-L<lack_width>] [-N<n_cols>/<t_col>] -Q<q_factor>]\n");
 		fprintf (stderr, "\t[[-S<symmetry_factor>] [-T<start>/<stop>/<int>] [-V] [-bi[s][<n>]]\n\t[-bo[s][<n>]] [-f[i|o]<colinfo>]\n\n");
-		
+
 		if (GMT_quick) exit (EXIT_FAILURE);
-		
+
 		fprintf (stderr, "\t-F sets Filtertype.  Choose from convolution and non-convolution filters.\n");
 		fprintf (stderr, "\t   and append full filter <width> in same units as time column,\n");
 		fprintf (stderr, "\t   Convolution filters:\n");
@@ -297,7 +297,7 @@ main (int argc, char **argv)
 		fprintf (stderr, "\t   Upper case type B, C, G, M, P, F will use robust filter versions:\n");
 		fprintf (stderr, "\t   i.e., replace outliers (2.5 L1 scale off median) with median during filtering.\n");
 		fprintf (stderr, "\n\tOPTIONS:\n");
-		
+
 		fprintf (stderr, "\t-D used when series is NOT equidistantly sampled.\n");
 		fprintf (stderr, "\t   Then <increment> will be the abscissae resolution, i.e. all abscissae\n");
 		fprintf (stderr, "\t   will be rounded off to a multiple of <increment>.\n");
@@ -366,18 +366,21 @@ main (int argc, char **argv)
 		error++;
 	}
 
-	
+
 	if (error) exit (EXIT_FAILURE);
 
 	GMT_put_history (argc, argv);	/* Update .gmtcommands4 */
 
-	
+
 	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
 		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 
 
+	GMT_io.skip_if_NaN[0] = GMT_io.skip_if_NaN[1] = FALSE;	/* Turn off default GMT NaN-handling */
+	GMT_io.skip_if_NaN[t_col] = TRUE;			/* ... But disallow NaN in "time" column */
+
 #ifdef SET_IO_MODE
 	GMT_setmode (GMT_OUT);
 #endif
@@ -408,14 +411,16 @@ main (int argc, char **argv)
 	}
 
 	free_space ();
-	
+
 	if (GMT_n_multiples > 0 && gmtdefs.verbose) fprintf (stderr, "%s: WARNING: %d multiple modes found\n", GMT_program, GMT_n_multiples);
-	
+
 	GMT_end (argc, argv);
+
+	exit (EXIT_SUCCESS);
 }
-	
+
 int load_data_and_check_extrema (void)
-{	
+{
 
 	int	i, n_fields, n_expected_fields;
 	double	last_time, new_time, *in;
@@ -426,7 +431,7 @@ int load_data_and_check_extrema (void)
 		GMT_setmode (GMT_IN);
 #endif
 	}
-	
+
 	if (gmtdefs.io_header[GMT_IN]) {
 		for (i = 0; i < gmtdefs.n_header_recs; i++) {
 			GMT_fgets (buffer, BUFSIZ, fp_in);
@@ -447,9 +452,9 @@ int load_data_and_check_extrema (void)
 		n_expected_fields = GMT_io.ncol[GMT_IN];
 	else
 		n_expected_fields = n_cols;
-	
+
 	while ((n_fields = GMT_input (fp_in, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
-	
+
 		if (GMT_io.status & GMT_IO_MISMATCH) {
 			fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields,  n_expected_fields, n_rows);
 			exit (EXIT_FAILURE);
@@ -466,10 +471,10 @@ int load_data_and_check_extrema (void)
 				}
 			}
 		}
-		
+
 		if (!GMT_is_dnan (data[t_col][n_rows]) )
 		{	/* Don't n_rows++ if time is bad this row  */
-		
+
 			new_time = data[t_col][n_rows];
 			n_rows++;
 			if (new_time < last_time) {
@@ -479,20 +484,20 @@ int load_data_and_check_extrema (void)
 			}
 			last_time = new_time;
 		}
-		
+
 		if (n_rows == n_row_alloc) {	/* Need more memory */
 			n_row_alloc += GMT_CHUNK;
 			allocate_more_data_space ();
 		}
 	}
-	
+
 	if (fp_in != GMT_stdin) GMT_fclose (fp_in);
-	
+
 	for (i = 0; i < n_cols; i++) data[i] = (double *) GMT_memory ((void *)data[i], (size_t)n_rows, sizeof (double), GMT_program);
 	if (gmtdefs.verbose) fprintf (stderr, "%s: Read %d data lines\n", GMT_program, n_rows);
 
 	/* Initialize scale parameters and last_loc based on min and max of data  */
-	
+
 	if (robust || (filter_type == MEDIAN) ) {
 		for (i = 0; i < n_cols; i++) {
 			min_scl[i] = 0.0;
@@ -510,13 +515,13 @@ int	set_up_filter(void)
 	int	i, i1, i2;
 	double	t_0, t_1, time, w_sum;
 	BOOLEAN	normalize = FALSE;
-	
-	
-	
+
+
+
 	t_0 = data[t_col][0];
 	t_1 = data[t_col][n_rows-1];
 	if (equidist) dt = (t_1 - t_0) / (n_rows - 1);
-	
+
 
 	if (fp_wt != NULL) {
 		f_wt = (double *) GMT_memory (VNULL, (size_t)n_work_alloc, sizeof(double), GMT_program);
@@ -534,6 +539,7 @@ int	set_up_filter(void)
 		f_operator = (w_sum == 0.0);	/* If weights sum to zero it is an operator like {-1 1] or [1 -2 1] */
 		half_n_f_wts = n_f_wts / 2;
 		half_width = half_n_f_wts * dt;
+		filter_width = 2.0 * half_width;
 		f_wt = (double *) GMT_memory ((void *)f_wt, (size_t)n_f_wts, sizeof (double), GMT_program);
 		if (gmtdefs.verbose) fprintf (stderr, "%s: Read %d filter weights from file.\n", GMT_program, n_f_wts);
 	}
@@ -545,7 +551,7 @@ int	set_up_filter(void)
 		half_width = 0.5 * filter_width;
 		half_n_f_wts = (int)floor (half_width / dt);
 		n_f_wts = 2 * half_n_f_wts + 1;
-		
+
 		f_wt = (double *) GMT_memory (VNULL, (size_t)n_f_wts, sizeof(double), GMT_program);
 		for (i = 0; i <= half_n_f_wts; i++) {
 			time = i * dt;
@@ -566,7 +572,7 @@ int	set_up_filter(void)
 	/* Initialize start/stop time */
 
 	if (out_at_time) {
-		
+
 		if (use_ends) {
 			while (t_start < t_0) t_start += t_int;
 			while (t_stop > t_1) t_stop -= t_int;
@@ -591,26 +597,25 @@ int	set_up_filter(void)
 
 	if (gmtdefs.verbose) fprintf (stderr, "F width: %g Resolution: %g Start: %g Stop: %g\n",
 		filter_width, dt, t_start, t_stop);
-	
+
 	return(0);
 }
 
 int	do_the_filter (void)
 {
-
 	int	i_row, i_col, left, right, n_l, n_r, iq, i_f_wt;
-	int	i_t_output, n_in_filter, n_for_call, n_good_ones;
+	int	i_t_output = 0, n_in_filter, n_for_call, n_good_ones;
 	double	time, delta_time, *outval, wt, val, med, scl, small;
 
 	outval = (double *)GMT_memory (VNULL, (size_t)n_cols, sizeof (double), GMT_program);
-	
+
 	if(!out_at_time) {	/* Position i_t_output at first output time  */
 		for(i_t_output = 0; data[t_col][i_t_output] < t_start; i_t_output++);
 		small = (data[t_col][1] - data[t_col][0]);
 	}
 	else
 		small = t_int;
-	
+
 	small *= GMT_CONV_LIMIT;
 	time = t_start;
 	left = right = 0;		/* Left/right end of filter window */
@@ -619,8 +624,8 @@ int	do_the_filter (void)
 
 	/* while (time <= t_stop) { */
 	while (time <= (t_stop + small)) {
-		while ((time - data[t_col][left]) > half_width) left++;
-		while (right < n_rows && (data[t_col][right] - time) <= half_width) right++;
+		while ((time - data[t_col][left] - small) > half_width) left++;
+		while (right < n_rows && (data[t_col][right] - time - small) <= half_width) right++;
 		n_in_filter = right - left;
 		if ( (!(n_in_filter)) || (check_lack && ( (filter_width / n_in_filter) > lack_width) ) ) {
 			if (out_at_time)
@@ -631,7 +636,7 @@ int	do_the_filter (void)
 			}
 			continue;
 		}
-			
+
 		for (i_col = 0; i_col < n_cols; i_col++) {
 			n_this_col[i_col] = 0;
 			wt_sum[i_col] = 0.0;
@@ -650,7 +655,7 @@ int	do_the_filter (void)
 				n_right[i_col] = 0;
 			}
 		}
-			
+
 		if (robust || filter_type > CONVOLVE) {
 			if (n_in_filter > n_work_alloc) {
 				n_work_alloc = n_in_filter;
@@ -682,20 +687,20 @@ int	do_the_filter (void)
 					if (n_this_col[i_col] < iq) good_one[i_col] = FALSE;
 				}
 			}
-				
+
 			for (i_col = 0; i_col < n_cols; i_col++) {
 				if (good_one[i_col]) {
 					n_for_call = n_this_col[i_col];
 					get_robust_estimates (i_col, n_for_call, robust);
 				}
 			}
-				
+
 		}	/* That's it for the robust work  */
 
 		if (filter_type > CONVOLVE) {
-			
+
 			/* Need to count how many good ones; use data_sum area  */
-				
+
 			n_good_ones = 0;
 			for (i_col = 0; i_col < n_cols; i_col++) {
 				if (i_col == t_col) {
@@ -711,16 +716,16 @@ int	do_the_filter (void)
 			}
 			if (n_good_ones) GMT_output (GMT_stdout, n_cols, data_sum);
 		}
-			
+
 		else {
-			
+
 			if (robust) for (i_col = 0; i_col < n_cols; i_col++) n_this_col[i_col] = 0;
-						
+
 			for (i_row = left; i_row < right; i_row++) {
 				delta_time = time - data[t_col][i_row];
 				i_f_wt = half_n_f_wts + (int)floor(0.5 + delta_time/dt);
 				if ( (i_f_wt < 0) || (i_f_wt >= n_f_wts) ) continue;
-					
+
 				for(i_col = 0; i_col < n_cols; i_col++) {
 					if ( !(good_one[i_col]) ) continue;
 					if (!GMT_is_dnan (data[i_col][i_row])) {
@@ -784,9 +789,9 @@ int	do_the_filter (void)
 			time = (i_t_output < n_rows) ? data[t_col][i_t_output] : t_stop + 1.0;
 		}
 	}
-	
+
 	GMT_free ((void *)outval);
-	
+
 	return(0);
 }
 
@@ -797,7 +802,7 @@ int	allocate_space (void)
 
 	data = (double **) GMT_memory (VNULL, (size_t)n_cols, sizeof(double *), GMT_program);
 	for (i = 0; i < n_cols; i++) data[i] = (double *) GMT_memory (VNULL, (size_t)n_row_alloc, sizeof(double), GMT_program);
-	
+
 	wt_sum = (double *) GMT_memory (VNULL, (size_t)n_cols, sizeof(double), GMT_program);
 	data_sum = (double *) GMT_memory (VNULL, (size_t)n_cols, sizeof(double), GMT_program);
 	n_this_col = (int *) GMT_memory (VNULL, (size_t)n_cols, sizeof(int), GMT_program);
@@ -805,7 +810,7 @@ int	allocate_space (void)
 
 	if (check_asym) n_left = (int *) GMT_memory (VNULL, (size_t)n_cols, sizeof(int), GMT_program);
 	if (check_asym) n_right = (int *) GMT_memory (VNULL, (size_t)n_cols, sizeof(int), GMT_program);
-	
+
 	if (robust || (filter_type > CONVOLVE) ) {	/* Then we need workspace  */
 
 		work = (double **) GMT_memory (VNULL, (size_t)n_cols, sizeof(double *), GMT_program);
@@ -858,20 +863,20 @@ void	free_space (void)
 }
 
 BOOLEAN	lack_check (int i_col, int left, int right)
-{			
+{
 
 	int	last_row, this_row;
 	BOOLEAN	lacking = FALSE;
 	double	last_t;
-	
+
 	last_row = left;
 	while (!(GMT_is_dnan (data[i_col][last_row])) && last_row < (right - 1)) last_row++;
-	
+
 	last_t = data[t_col][last_row];
 	this_row = last_row + 1;
 	while ( !(lacking) && this_row < (right - 1)) {
 		while (!(GMT_is_dnan (data[i_col][this_row])) && this_row < (right - 1)) this_row++;
-		
+
 		if ( (data[t_col][this_row] - last_t) > lack_width)
 			lacking = TRUE;
 		else {
@@ -887,11 +892,11 @@ void	get_robust_estimates (int j, int n, int both)
 {
 	int i, n_smooth, sort_me = TRUE;
 	double	low, high, last, temp;
-	
+
 	if (filter_type == MODE) {
 
 		n_smooth = n / 2;
-	
+
 		GMT_mode (work[j], n, n_smooth, sort_me, GMT_mode_selection, &GMT_n_multiples, &temp);
 	}
 	else if (filter_type == MEDIAN) {
@@ -899,7 +904,7 @@ void	get_robust_estimates (int j, int n, int both)
 		low = min_loc[j];
 		high = max_loc[j];
 		last = last_loc[j];
-	
+
 		GMT_median (work[j], n, low, high, last, &temp);
 	}
 	else {	/* EXTREME filter **/
@@ -907,7 +912,7 @@ void	get_robust_estimates (int j, int n, int both)
 	}
 
 	last_loc[j] = this_loc[j] = temp;
-	
+
 	if (both) {
 		for (i = 0; i < n; i++)
 			work[j][i] = fabs(work[j][i] - this_loc[j]);
@@ -918,16 +923,16 @@ void	get_robust_estimates (int j, int n, int both)
 		last_scl[j] = this_scl[j] = temp;
 	} 
 }
-	
+
 double	boxcar_weight (double radius, double half_width)
 {
 	double weight;
-	
+
 	if (radius > half_width)
 		weight = 0.0;
 	else
 		weight = 1.0;
-		
+
 	return (weight);
 }
 
@@ -939,7 +944,7 @@ double	cosine_weight (double radius, double half_width)
 		weight = 0.0;
 	else
 		weight = 1.0 + cos (radius * cosine_constant);
-		
+
 	return (weight);
 }
 
@@ -951,6 +956,6 @@ double	gaussian_weight (double radius, double half_width)
 		weight = 0.0;
 	else
 		weight = exp (radius * radius * gauss_constant);
-		
+
 	return (weight);
 }
