@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    $Id: fitcircle.c,v 1.12 2004/04/25 09:10:45 pwessel Exp $
+ *    $Id: fitcircle.c,v 1.16 2005/12/17 05:59:21 pwessel Exp $
  *
- *	Copyright (c) 1991-2004 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2006 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -64,7 +64,7 @@ struct	DATA {
 	double	x[3];
 }	*data;
 
-main (int argc, char **argv)
+int main (int argc, char **argv)
 {
 	int	i, j, k, imin, imax, n_alloc, n_data, n, np, nrots, norm = -1, n_fields, n_expected_fields;
 	BOOLEAN	error = FALSE, greenwich = FALSE, find_small_circle = FALSE;
@@ -72,7 +72,8 @@ main (int argc, char **argv)
 	double	lonsum, latsum, *in;
 	double	meanv[3], cross[3], cross_sum[3], gcpole[3], scpole[3];		/* Extra vectors  */
 	double	*a, *lambda, *v, *b, *z;	/* Matrix stuff */
-	double	get_small_circle(struct DATA *data, int ndata, double *center, double *gcpole, double *scpole, int norm, double *work), rad, *work;
+	double	get_small_circle(struct DATA *data, int ndata, double *center, double *gcpole, double *scpole, int norm, double *work);
+	double	rad, *work = VNULL;
 
 	char	buffer[BUFSIZ], format[BUFSIZ];
 
@@ -83,10 +84,10 @@ main (int argc, char **argv)
         for (i = 1; i < argc; i++) {
                 if (argv[i][0] == '-') {
                         switch (argv[i][1]) {
-			
-		
+
+
 				/* Common parameters */
-			
+
 				case 'H':
 				case 'V':
 				case ':':
@@ -95,9 +96,9 @@ main (int argc, char **argv)
 				case '\0':
 					error += GMT_get_common_args (argv[i], 0, 0, 0, 0);
 					break;
-				
+
 				/* Supplemental parameters */
-				
+
 				case 'L':
 					norm = 3;
                                         if (argv[i][2]) norm = atoi(&argv[i][2]);
@@ -123,7 +124,7 @@ main (int argc, char **argv)
 		fprintf (stderr, "fitcircle %s - find best-fitting great circle to points on sphere\n\n", GMT_VERSION);
 		fprintf(stderr,"usage:  fitcircle [<input_file>] -L[<n>] [-H[<nrec>]] [-S] [-V] [-:]\n");
 		fprintf(stderr,"\t[-bi[s][<n>]] [-f[i|o]<colinfo>]\n\n");
-		
+
 		if (GMT_quick) exit (EXIT_FAILURE);
 
 		fprintf(stderr,"\tReads from input_file or standard input\n");
@@ -173,11 +174,11 @@ main (int argc, char **argv)
 	n_data = 0;
 	lonsum = latsum = 0.0;
 	sprintf (format, "%s\t%s", gmtdefs.d_format, gmtdefs.d_format);
-	
+
 	data = (struct DATA *) GMT_memory (VNULL, (size_t)n_alloc, sizeof(struct DATA), GMT_program);
 
 	if (gmtdefs.io_header[GMT_IN]) for (i = 0; i < gmtdefs.n_header_recs; i++) GMT_fgets (buffer, BUFSIZ, fp);
-		
+
 	n_expected_fields = (GMT_io.ncol[GMT_IN]) ? GMT_io.ncol[GMT_IN] : 2;
 
 	while ((n_fields = GMT_input (fp, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
@@ -243,7 +244,7 @@ main (int argc, char **argv)
 		}
 		GMT_normalize3v (cross_sum);
 		if (find_small_circle) for (i = 0; i < 3; i++) gcpole[i] = cross_sum[i];
-		
+
 		GMT_cart_to_geo (&latsum, &lonsum, cross_sum, TRUE);
 		if (greenwich && lonsum < 0.0) lonsum += 360.0;
 		fprintf (GMT_stdout, format, lonsum, latsum);
@@ -297,13 +298,13 @@ main (int argc, char **argv)
 		if (greenwich && lonsum < 0.0) lonsum += 360.0;
 		fprintf (GMT_stdout, format, lonsum, latsum);
 		fprintf (GMT_stdout, "\tL2 Average Position (Eigenval Method)\n");
-		
+
 		if (v[imin*np+2] < 0.0)	/* Eigvec is in S Hemisphere  */
 			for (i = 0; i < 3; i++) gcpole[i] = -v[imin*np+i];
 		else
 			for (i = 0; i < 3; i++) gcpole[i] = v[imin*np+i];
-		
-		GMT_cart_to_geo (&latsum, &lonsum, gcpole, TRUE);		
+
+		GMT_cart_to_geo (&latsum, &lonsum, gcpole, TRUE);
 		if (greenwich && lonsum < 0.0) lonsum += 360.0;
 		fprintf (GMT_stdout, format, lonsum, latsum);
 		fprintf (GMT_stdout, "\tL2 N Hemisphere Great Circle Pole (Eigenval Method)\n");
@@ -335,6 +336,7 @@ main (int argc, char **argv)
 	if (find_small_circle && (norm%2) ) GMT_free ((void *)work);
 	GMT_free ((void *)data);
 	GMT_end (argc, argv);
+	exit (EXIT_SUCCESS);
 }
 
 double	get_small_circle (struct DATA *data, int ndata, double *center, double *gcpole, double *scpole, int norm, double *work)
