@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: gmt_calclock.c,v 1.30 2004/04/24 01:30:00 pwessel Exp $
+ *	$Id: gmt_calclock.c,v 1.39 2006/02/23 05:48:49 pwessel Exp $
  *
- *	Copyright (c) 1991-2004 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2006 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -32,6 +32,7 @@
 
 */
 
+#define GMT_WITH_NO_PS
 #include "gmt.h"
 
 BOOLEAN want_iso = FALSE;	/* Temporary to test compilation.  Delete this.  */
@@ -48,101 +49,29 @@ GMT_dtime GMT_rdc2dt (GMT_cal_rd rd, double secs) {
 	return ((GMT_dtime)(GMT_DAY2SEC_F*(rd-GMT_GCAL_EPOCH) + secs));
 }
 
+int	splitinteger(double value, int epsilon, double *doublepart) {
+/*	split value into integer and and floating part for date usage. */
+	int i;
+
+	i = (int) (value/(double)epsilon);
+	*doublepart = value - ((double) i)*((double)epsilon);
+	return i;
+}
+
 void	GMT_dt2rdc (GMT_dtime t, GMT_cal_rd *rd, double *s) {
-/*	Given GMT_dtime value t, load rata die day of that t
+/*	Given GMT_dtime value t, load calendar data of that day
 	in rd and the seconds since the start of that day in
 	s.  */
 
-	double	x;
-	
-	x = floor(t * GMT_SEC2DAY);
-	*s = t - GMT_DAY2SEC_F * x;
-	*rd = (GMT_cal_rd)(x+GMT_GCAL_EPOCH);
+	int i;
+
+	i = splitinteger(t, 86400, s);
+	*rd = (GMT_cal_rd)(i+GMT_GCAL_EPOCH);
 }
 
 
 /* String reading functions for parsing ISO 8601 cal and clock  */
 
-int	GMT_atoft (char *s, double *t) {
-/*	Given s, which may be a string representation of
-	a floating point number, or a calendar[and clock]
-	value, try to decode s.  Return -1 on failure.
-	Return 0 if s is a float, and store it in t.
-	Return 1 if s is a calendar[and clock]value, and
-	store the GMT_cctime value in t.
-	
-	Note that s could be a floating point representation
-	of a time, such as a double precision Julian Day number.
-	In that case, there is a GMT_cctime value which may be
-	associated with it via TIME_UNIT and TIME_EPOCH.  But
-	that association is not done in this routine.
-	
-	WHF Smith 20 April 2000.
-*/
-
-	double	secs = 0.0;
-	int	j, k, err = -1, is_float = 0, is_time = 1;
-	GMT_cal_rd	rd;
-	char	*cp;
-	
-	if ( cp = strpbrk (s, "T") ) {
-		k = strlen(s);
-		j = strlen(cp);
-		if (j > 1) {
-			/* A clock string lies beyond the 'T' of s[]; read it.  */
-			if (GMT_read_clock (&cp[1], &secs)) return (err);
-		}
-		/* Terminate s before the 'T' to make calendar string  */
-		
-		if (GMT_read_cal (s, &rd) ) return (err);
-		
-		*t = GMT_rdc2dt (rd, secs);
-		return (is_time);
-	}
-	else if (strpbrk (s, "W") ) {
-	
-		if (GMT_read_cal (s, &rd) ) return (err);
-		
-		*t = GMT_rdc2dt (rd, secs);
-		return (is_time);
-	}
-	else if ( (strpbrk (s, "E")) || (strpbrk (s, "e")) ) {
-		
-		if ( (sscanf (s, "%lf", t)) != 1) return (err);
-		return (is_float);
-	}
-	
-	/* Get here when string is ambiguous.  Examine last occurrence
-	of a hyphen.  If this is preceded by a digit, then it is an
-	internal hyphen, and the string is a time string.  */
-	
-	if (cp = strrchr (s, '-') ) {
-		/* There is a hyphen.  If more than one, this is the last.  */
-		k = strlen(s);
-		j = strlen(cp);
-		if (j < k) {
-			/* A character precedes this hyphen.  */
-			if (isdigit ((int)s[k-j-1]) ) {
-				
-				if (GMT_read_cal (s, &rd) ) return (err);
-		
-				*t = GMT_rdc2dt (rd, secs);
-				return (is_time);
-			}
-			else if (s[k-j-1] != ' ') {
-				/* It isn't space either; this is an error.  */
-				return (err);
-			}
-		}
-	}
-	
-	/* Get here when string has no calendar-like internal hyphens.
-		Presume it is a float.  */
-	
-	if ( (sscanf (s, "%lf", t)) != 1) return (err);
-	return (is_float);
-}
-
 int	GMT_read_clock (char *s, double *t) {
 /* 	Given a clock string s, try to read the form
 	hh[:mm[:ss[.xxx]]] with 0 <= hh <= 24 (yes, 24; 
@@ -157,7 +86,7 @@ int	GMT_read_clock (char *s, double *t) {
 	W H F Smith, 20 april 2000
 */
 
-	double	tau, dsec;
+	double	dsec;
 	int	j, k;
 	char	*cm, *cs;
 	
@@ -171,7 +100,7 @@ int	GMT_read_clock (char *s, double *t) {
 	}
 	if ( (sscanf(s, "%d", &k)) != 1) return (-1);
 	if (k < 0 || k > 24) return (-1);
-	tau = GMT_HR2SEC_I * k;
+	*t = GMT_HR2SEC_I * k;		/* t now has hours (in secs) */
 	
 	if (!(cm)) return (0);	/* This allows colon-terminated
 		strings to be treated as OK.  */
@@ -184,12 +113,12 @@ int	GMT_read_clock (char *s, double *t) {
 	}
 	if ( (sscanf(cm, "%d", &k)) != 1) return (-1);
 	if (k < 0 || k > 59) return (-1);
-	tau += GMT_MIN2SEC_I * k;
+	(*t) += GMT_MIN2SEC_I * k;	/* Now add in minutes (in secs) */
 	
 	if (!(cs)) return (0);
 	if ( (sscanf(cs, "%lf", &dsec)) != 1) return (-1);
 	if (dsec < 0.0 || dsec >= 60.0) return (-1);
-	*t = tau + dsec;
+	*t += dsec;			/* Finally add in seconds */
 	return (0);
 }
 
@@ -230,7 +159,7 @@ int	GMT_read_cal (char *s, GMT_cal_rd *rd) {
 	if ( (sscanf (s, "%d", &itemp1) ) != 1) return (-1);
 	
 	/* A null value of cj is not an error.  We might have
-		had "1985T" passed to GMT_atoft () and it might
+		had "1985T" passed to a function and it might
 		have stripped off the 'T' and passed the remaining
 		string to this routine, in which case cj is null,
 		and we should interpret this as gregorian, with
@@ -615,13 +544,14 @@ void	GMT_gcal_from_dt (GMT_dtime t, struct GMT_gcal *cal) {
 	
 	GMT_cal_rd rd;
 	double	x;
-	
+	int i;
+
 	GMT_dt2rdc (t, &rd, &x);
 	GMT_gcal_from_rd (rd, cal);
-	cal->hour = (int) floor (x * GMT_SEC2HR);
-	x -= GMT_HR2SEC_F * cal->hour;
-	cal->min  = (int) floor (x * GMT_SEC2MIN);
-	cal->sec  = x - GMT_MIN2SEC_F * cal->min;
+	/* split double seconds and integer time */
+	i = splitinteger(x, 60, &cal->sec);
+	cal->hour = i/60;
+	cal->min  = i%60;
 	return;
 }
 	
@@ -633,7 +563,7 @@ int	GMT_verify_time_step (int step, char unit) {
 	
 	int	retval = 0;
 	
-	if (step <= 0) {
+	if (step < 0) {
 		fprintf (stderr, "GMT SYNTAX ERROR:  time steps must be positive.\n");
 		return (-1);
 	}
@@ -1163,8 +1093,10 @@ void GMT_format_calendar (char *date, char *clock, struct GMT_DATE_IO *D, struct
 		i_sec = (int) floor (calendar.sec);
 		m_sec = irint (C->f_sec_to_int * (calendar.sec - i_sec));
 	}
-	else
+	else {
 		i_sec = irint (calendar.sec);
+		m_sec = 0;
+	}
 	
 	if (C->twelve_hr_clock) {		/* Must deal with am/pm formatting */
 		if (calendar.hour < 12)
@@ -1192,3 +1124,72 @@ void GMT_format_calendar (char *date, char *clock, struct GMT_DATE_IO *D, struct
 	}
 	return;
 }
+
+void GMT_get_time_label (char *string, struct GMT_PLOT_CALCLOCK *P, struct PLOT_AXIS_ITEM *T, GMT_dtime t)
+{	/* Assemble the annotation label given the formatting options presented */
+	struct GMT_gcal calendar;
+
+	GMT_gcal_from_dt (t, &calendar);			/* Convert t to a complete calendar structure */
+
+	switch (T->unit) {
+		case 'Y':	/* 4-digit integer year */
+			(P->date.compact) ? sprintf (string, "%d", calendar.year) : sprintf (string, "%4.4d", calendar.year);
+			break;
+		case 'y':	/* 2-digit integer year */
+			/* (P->date.compact) ? sprintf (string, "%d", calendar.year % 100) : sprintf (string, "%2.2d", calendar.year % 100); */
+			sprintf (string, "%2.2d", calendar.year % 100);
+			break;
+		case 'O':	/* Plot via date format */
+			GMT_format_calendar (string, CNULL, &P->date, &P->clock, T->upper_case, T->flavor, t);
+			break;
+		case 'o':	/* 2-digit month */
+			(P->date.compact) ? sprintf (string, "%d", calendar.month) : sprintf (string, "%2.2d", calendar.month);
+			break;
+		case 'U':	/* ISO year, week, day via date format */
+			GMT_format_calendar (string, CNULL, &P->date, &P->clock, T->upper_case, T->flavor, t);
+			break;
+		case 'u':	/* 2-digit ISO week */		
+			(P->date.compact) ? sprintf (string, "%d", calendar.iso_w) : sprintf (string, "%2.2d", calendar.iso_w);
+			break;
+		case 'K':	/*  ISO Weekday name */
+			if (T->upper_case) GMT_str_toupper (GMT_time_language.day_name[T->flavor][calendar.iso_d%7]);
+			sprintf (string, "%s", GMT_time_language.day_name[T->flavor][calendar.iso_d%7]);
+			break;
+		case 'k':	/* Day of the week 1-7 */
+			sprintf (string, "%d", (calendar.day_w - gmtdefs.time_week_start + 7) % 7 + 1);
+			break;
+		case 'D':	/* Day, via date format */
+			GMT_format_calendar (string, CNULL, &P->date, &P->clock, T->upper_case, T->flavor, t);
+			break;
+		case 'd':	/* 2-digit day or 3-digit day of year */
+		case 'R':	/* Gregorian month-days are the same thing - only they start at beginning of weeks and not months */
+			if (P->date.day_of_year)
+				(P->date.compact) ? sprintf (string, "%d", calendar.day_y) : sprintf (string, "%3.3d", calendar.day_y);
+			else
+				(P->date.compact) ? sprintf (string, "%d", calendar.day_m) : sprintf (string, "%2.2d", calendar.day_m);
+			break;
+		case 'H':	/* Hours via clock format */
+			GMT_format_calendar (CNULL, string, &P->date, &P->clock, T->upper_case, T->flavor, t);
+			break;
+		case 'h':	/* 2-digit hour */
+			(P->date.compact) ? sprintf (string, "%d", calendar.hour) : sprintf (string, "%2.2d", calendar.hour);
+			break;
+		case 'M':	/* Minutes via clock format */
+			GMT_format_calendar (CNULL, string, &P->date, &P->clock, T->upper_case, T->flavor, t);
+			break;
+		case 'm':	/* 2-digit minutes */
+			(P->date.compact) ? sprintf (string, "%d", calendar.min) : sprintf (string, "%2.2d", calendar.min);
+			break;
+		case 'C':	/* Seconds via clock format */
+			GMT_format_calendar (CNULL, string, &P->date, &P->clock, T->upper_case, T->flavor, t);
+			break;
+		case 'c':	/* 2-digit seconds */
+			(P->date.compact) ? sprintf (string, "%d", (int)calendar.sec) : sprintf (string, "%2.2d", (int)calendar.sec);
+			break;
+		default:
+			fprintf (stderr, "ERROR: wrong unit passed to GMT_get_time_label\n");
+			exit (EXIT_FAILURE);
+			break;
+	}
+}
+		
