@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: gmtselect.c,v 1.25 2004/06/25 22:39:01 pwessel Exp $
+ *	$Id: gmtselect.c,v 1.45 2006/02/06 14:00:37 remko Exp $
  *
- *	Copyright (c) 1991-2004 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2006 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -63,26 +63,27 @@ struct DATA {
 
 int compare_x(const void *point_1, const void *point_2);
 
-main (int argc, char **argv)
+int main (int argc, char **argv)
 {
-	int i, j, k, fno, n_read = 0, n_pass = 0, n_files = 0, n_args, n_points, n_lines, n_pols;
+	int i, j, k, fno, n_read = 0, n_pass = 0, n_files = 0, n_args, n_points = 0, n_lines = 0, n_pols = 0;
 	int n_fields, n_expected_fields, out_edge_in[5], ind, bin, last_bin = -1;
 	int min_level = 0, max_level = MAX_LEVEL, np[2], base = 3, wd[2], id, this_node;
-	int side, is_inside = 1, row, col, n_output = -1;
+	int side, is_inside = 1, row, col, pos, n_output = -1;
+
 
-	
 	BOOLEAN error = FALSE, limit_line_distance = FALSE, limit_polygon = FALSE, limit_point_distance = FALSE, limit_point_distance_cart = FALSE;
-	BOOLEAN want_inside[6] = {TRUE,TRUE,TRUE,TRUE,TRUE,TRUE}, nofile = TRUE, done = FALSE, first = TRUE, inside, greenwich = FALSE;
-	BOOLEAN flat_earth = FALSE, do_project = FALSE, binary = FALSE, limit_coast = FALSE, dry_wet_only, long_verbose = FALSE, limit_z = FALSE;
-	
+	BOOLEAN want_inside[6] = {TRUE,TRUE,TRUE,TRUE,TRUE,TRUE}, nofile = TRUE, done = FALSE, first = TRUE, inside, greenwich = FALSE, output_header = FALSE;
+	BOOLEAN flat_earth = FALSE, do_project = FALSE, binary = FALSE, limit_coast = FALSE, dry_wet_only = FALSE, long_verbose = FALSE, limit_z = FALSE;
+	BOOLEAN need_header;
+
 	double west = 0.0, east = 0.0, south = 0.0, north = 0.0, xx, yy, min_area = 0.0;
-	double *xp, *yp, *dp, pt_dist = 0.0, ln_dist = 0.0, *in, west_border, east_border;
-	double xmin, xmax, ymin, ymax, lon, r, sa, ca, zmin = -DBL_MAX, zmax = DBL_MAX;
-	
-	char buffer[BUFSIZ], *pt_file, *ln_file, *pol_file, *ptr, res = 'l', za[16], zb[16];
-	
+	double *xp, *yp, *dp, pt_dist = 0.0, ln_dist = 0.0, *in, west_border = 0.0, east_border = 0.0;
+	double xmin, xmax, ymin, ymax, lon, zmin = -DBL_MAX, zmax = DBL_MAX, step = 0.0;
+
+	char buffer[BUFSIZ], ptr[BUFSIZ], *pt_file, *ln_file, *pol_file, res = 'l', za[16], zb[16];
+
 	FILE *fp = NULL;
-	
+
 	struct GMT_LINES *pol, *line;
 	struct POL *p[2];
 
@@ -93,13 +94,13 @@ main (int argc, char **argv)
 	out_edge_in[1] = out_edge_in[3] = 1;			/* Default for "dry" areas = 1 (inside) */
 
 	argc = GMT_begin (argc, argv);
-	
+
 	for (i = 1; i < argc; i++) {
 		if (argv[i][0] == '-') {
 			switch (argv[i][1]) {
-		
+
 				/* Common parameters */
-			
+
 				case 'V':
 					if (argv[i][2] == 'l') long_verbose = TRUE;
 				case 'H':
@@ -111,9 +112,9 @@ main (int argc, char **argv)
 				case '\0':
 					error += GMT_get_common_args (argv[i], &west, &east, &south, &north);
 					break;
-				
+
 				/* Supplemental parameters */
-				
+
 				case 'A':
 					j = sscanf (&argv[i][2], "%lf/%d/%d", &min_area, &min_level, &max_level);
 					if (j == 1) min_level = 0, max_level = MAX_LEVEL;
@@ -196,9 +197,8 @@ main (int argc, char **argv)
 						is_inside = 2;
 						buffer[strlen(buffer)-1] = 0;
 					}
-					ptr = strtok (buffer, "/");
-					j = 0;
-					while (j < 5 && ptr) {
+					j = pos = 0;
+					while (j < 5 && (GMT_strtok (buffer, "/", &pos, ptr))) {
 						switch (ptr[0]) {
 							case 's':	/* Skip points in this level */
 								out_edge_in[j] = 0;
@@ -210,7 +210,6 @@ main (int argc, char **argv)
 								fprintf (stderr, "%s: GMT SYNTAX ERROR -N option:  Bad modifier (use s or k)\n", GMT_program);
 								error++;
 						}
-						ptr = strtok (CNULL, "/");
 						j++;
 					}
 					if (!(j == 2 || j == 5)) {
@@ -229,6 +228,9 @@ main (int argc, char **argv)
 					if (!(za[0] == '-' && za[1] == '\0')) zmin = atof (za);
 					if (!(zb[0] == '-' && zb[1] == '\0')) zmax = atof (zb);
 					break;
+				case '+':	/* Undocumented option to increase path-fix resolution */
+					step = atof (&argv[i][2]);
+					break;
 				default:
 					error = TRUE;
 					GMT_default_error (argv[i][1]);
@@ -238,15 +240,15 @@ main (int argc, char **argv)
 		else
 			n_files++;
 	}
-	
+
 	if (argc == 1 || GMT_quick) {
 		fprintf (stderr, "gmtselect %s - Select data subsets based on multiple spatial criteria\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: gmtselect <infiles> [-A<min_area>[/<min_level>/<max_level>]]\n\t[-C[f]<dist>/<ptfile>] [-D<resolution>] [-F<polygon>]\n");
 		fprintf (stderr, "\t[-J<parameters>] [-L[<dist>/<lfile>]] [-H[<nrec>]] [-I[cflrsz] [-M[<flag>]] [-N<maskvalues>[o]]\n");
 		fprintf (stderr, "\t[-R<west/east/south/north>] [-V[l]]\n\t[-:] [-Z<min>/<max>] [-bi[s][<n>]] [-bo[s][<n>]] [-f[i|o]<colinfo>]\n\n");
-		
+
 		if (GMT_quick) exit (EXIT_FAILURE);
-		
+
 		fprintf (stderr, "\tinfiles (in ASCII or binary) have 2 or more columns with (x,y) or (y,x) in first columns.\n");
 		fprintf (stderr, "\t  If no file(s) is given, standard input is read.\n");
 		fprintf (stderr, "\n\tOPTIONS:\n");
@@ -255,7 +257,8 @@ main (int argc, char **argv)
 		fprintf (stderr, "\t-C pass locations that are within <dist> of any point in ASCII <ptfile>\n");
 		fprintf (stderr, "\t   Give 0 distance if 3rd column of ptfile has individual distances.\n");
 		fprintf (stderr, "\t   Distances are in users units [or km if -R -J are used].\n");
-		fprintf (stderr, "\t   Prepend f to use approximate (flat earth) rather than exact great circle distances.\n");
+		fprintf (stderr, "\t   Prepend f to use approximate (flat earth) rather than exact geodesic distances.\n");
+		fprintf (stderr, "\t   If the current ELLIPSOID is Sphere then spherical great circle distances are used.\n");
 		fprintf (stderr, "\t-D Choose one of the following resolutions: (Ignored unless -N is set)\n");
 		fprintf (stderr, "\t   f - full resolution (may be very slow for large regions)\n");
 		fprintf (stderr, "\t   h - high resolution (may be slow for large regions)\n");
@@ -298,7 +301,7 @@ main (int argc, char **argv)
 		GMT_explain_option ('.');
 		exit (EXIT_FAILURE);
 	}
-	
+
 	if (limit_z && zmax <= zmin) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR:  -Z must have zmax > zmin!\n", GMT_program);
 		error++;
@@ -316,7 +319,7 @@ main (int argc, char **argv)
 
 	GMT_put_history (argc, argv);	/* Update .gmtcommands4 */
 
-	if (!MAPPING && limit_point_distance) limit_point_distance_cart = TRUE;
+	if (limit_point_distance && !MAPPING && !(GMT_io.in_col_type[0] == GMT_IS_LON && GMT_io.in_col_type[1] == GMT_IS_LAT)) limit_point_distance_cart = TRUE;
 
 	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
@@ -327,7 +330,7 @@ main (int argc, char **argv)
 	GMT_setmode (GMT_OUT);
 #endif
 
-	if (!project_info.region_supplied && (limit_coast || (GMT_io.in_col_type[0] == GMT_IS_LON && GMT_io.in_col_type[1] == GMT_IS_LAT))) {	/* If we use coastline data or used -fg but didnt give -R we imply -Rg */
+	if (!project_info.region_supplied && limit_coast) {	/* If we use coastline data or used -fg but didnt give -R we imply -Rg */
 		project_info.region_supplied = TRUE;
 		west = 0.0;	east = 360.0;	south = -90.0;	north = +90.0;
 		GMT_io.in_col_type[0] = GMT_IS_LON;
@@ -343,7 +346,8 @@ main (int argc, char **argv)
 			project_info.pars[0] = project_info.pars[1] = 1.0;
 		}
 		do_project = TRUE;
-		if (MAPPING) {
+		step = 0.01;
+		if (project_info.degree[0]) {
 			if (west < 0.0 && east < 0.0) {
 				west += 360.0;
 				east += 360.0;
@@ -371,7 +375,10 @@ main (int argc, char **argv)
 	binary = (GMT_io.binary[GMT_IN] || GMT_io.binary[GMT_OUT]);
 
 	if (GMT_io.in_col_type[0] & GMT_IS_GEO) {	/* Geographic data */
-		GMT_distance_func = (PFD) ((MAPPING) ? ((flat_earth) ? GMT_flatearth_dist : GMT_km_dist) : GMT_km_dist);
+		if (flat_earth)
+			GMT_distance_func = GMT_flatearth_dist_km;
+		else
+			GMT_distance_func = (PFD) ((SPHERICAL) ? GMT_geodesic_dist_km : GMT_great_circle_dist_km);
 		near_a_line = (PFI) GMT_near_a_line_spherical;
 	}
 	else {
@@ -380,10 +387,10 @@ main (int argc, char **argv)
 	}
 
 	if (limit_point_distance) { 
-		n_points = GMT_points_init (pt_file, &xp, &yp, &dp, pt_dist, greenwich);
-		
+		n_points = GMT_points_init (pt_file, &xp, &yp, &dp, pt_dist, greenwich, FALSE);
+
 		/* Copy xp into struct data, sort, and copy back */
-		
+
 		data = (struct DATA *) GMT_memory (VNULL, (size_t)n_points, sizeof(struct DATA), GMT_program);
 
 		for (k = 0; k < n_points; k++) {
@@ -402,41 +409,33 @@ main (int argc, char **argv)
 		GMT_free ((void *)data);
 	}
 
-	if (limit_line_distance)  n_lines  = GMT_lines_init (ln_file, &line, ln_dist, greenwich);
+	if (limit_line_distance) n_lines  = GMT_lines_init (ln_file, &line, ln_dist, greenwich, FALSE, FALSE);
 	if (limit_polygon) {
-		n_pols = GMT_lines_init (pol_file, &pol, -9999.0, greenwich);
-		if (do_project) {	/* Convert all the polygons */
+		n_pols = GMT_lines_init (pol_file, &pol, 0.0, greenwich, TRUE, FALSE);
+		if (do_project) {	/* Convert all the polygons points using the map projection */
 			for (i = 0; i < n_pols; i++) {
-				if (pol[i].polar) {	/* Polygon is a polar cap, use simple polar (r-theta) transformation */
-					for (j = 0; j < pol[i].np; j++) {
-						r = 90.0 - fabs (pol[i].lat[j]);
-						sincos (pol[i].lon[j] * D2R, &sa, &ca);
-						pol[i].lon[j] = r * ca;
-						pol[i].lat[j] = r * sa;
-					}
-				}
-				else {
-					for (j = 0; j < pol[i].np; j++) {
-						GMT_geo_to_xy (pol[i].lon[j], pol[i].lat[j], &xx, &yy);
-						pol[i].lon[j] = xx;
-						pol[i].lat[j] = yy;
-					}
+				for (j = 0; j < pol[i].np; j++) {
+					GMT_geo_to_xy (pol[i].lon[j], pol[i].lat[j], &xx, &yy);
+					pol[i].lon[j] = xx;
+					pol[i].lat[j] = yy;
 				}
 			}
 		}
 	}
 
+	need_header = GMT_io.multi_segments;	/* Only need t obreak up segments */
+
 	/* Now we are ready to take on some input values */
-	
+
 	if (n_files > 0)
 		nofile = FALSE;
 	else
 		n_files = 1;
 	n_args = (argc > 1) ? argc : 2;
-		
+
 	for (fno = 1; !done && fno < n_args; fno++) {	/* Loop over input files, if any */
 		if (!nofile && argv[fno][0] == '-') continue;
-		
+
 		if (nofile) {	/* Just read standard input */
 			fp = GMT_stdin;
 			done = TRUE;
@@ -451,7 +450,7 @@ main (int argc, char **argv)
 		}
 
 		if (!nofile && gmtdefs.verbose) fprintf (stderr, "%s: Working on file %s\n", GMT_program, argv[fno]);
-		
+
 		if (gmtdefs.io_header[GMT_IN]) {
 			for (i = 0; i < gmtdefs.n_header_recs; i++) {
 				GMT_fgets (buffer, BUFSIZ, fp);
@@ -460,11 +459,11 @@ main (int argc, char **argv)
 			first = FALSE;
 		}
 		n_expected_fields = (GMT_io.ncol[GMT_IN]) ? GMT_io.ncol[GMT_IN] : BUFSIZ;
-	
+
 		while ((n_fields = GMT_input (fp, &n_expected_fields, &in)) >= 0 && ! (GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
 
 			while (GMT_io.status & GMT_IO_SEGMENT_HEADER && !(GMT_io.status & GMT_IO_EOF)) {
-				GMT_write_segmentheader (GMT_stdout, n_expected_fields);
+				output_header = TRUE;
 				n_fields = GMT_input (fp, &n_expected_fields, &in);
 			}
 			if ((GMT_io.status & GMT_IO_EOF)) continue;	/* At EOF */
@@ -482,29 +481,29 @@ main (int argc, char **argv)
 			}
 
 			if (limit_z) {
-				if (GMT_is_dnan (in[2])) continue;	/* cannot keep when no z */
+				if (GMT_is_dnan (in[2])) { output_header = need_header; continue;}	/* cannot keep when no z */
 				inside = (in[2] >= zmin && in[2] <= zmax); 
-				if (inside != want_inside[5]) continue;
+				if (inside != want_inside[5]) { output_header = need_header; continue;}
 			}
 
 			lon = in[0];
 			if (project_info.region_supplied) {
 				inside = !GMT_map_outside (lon, in[1]);
-				if (inside != want_inside[0]) continue;
+				if (inside != want_inside[0]) { output_header = need_header; continue;}
 			}
 
 			if (limit_point_distance_cart) {
 				inside = GMT_near_a_point_cart (lon, in[1], xp, yp, dp, n_points); 
-				if (inside != want_inside[1]) continue;
+				if (inside != want_inside[1]) { output_header = need_header; continue;}
 			}
 			else if (limit_point_distance) {
 				inside = GMT_near_a_point (lon, in[1], xp, yp, dp, n_points);
-				if (inside != want_inside[1]) continue;
+				if (inside != want_inside[1]) { output_header = need_header; continue;}
 			}
 
 			if (limit_line_distance) {
-				inside = near_a_line (lon, in[1], line, n_lines, FALSE, NULL);
-				if (inside != want_inside[2]) continue;
+				inside = near_a_line (lon, in[1], line, n_lines, FALSE, NULL, NULL, NULL);
+				if (inside != want_inside[2]) { output_header = need_header; continue;}
 			}
 
 			if (limit_polygon) {
@@ -517,21 +516,13 @@ main (int argc, char **argv)
 				i = 0;
 				inside = FALSE;
 				while (i < n_pols && !inside) {
-					if (pol[i].polar == -1 && in[1] <= 0.0) {	/* Point on S hemisphere, check */
-						r = 90.0 + in[1];
-						sincos (lon * D2R, &sa, &ca);
-						inside = (GMT_non_zero_winding (r * ca, r * sa, pol[i].lon, pol[i].lat, pol[i].np) >= is_inside);
-					}
-					else if (pol[i].polar == +1 && in[1] >= 0.0) {	/* Point on N hemisphere, check */
-						r = 90.0 - in[1];
-						sincos (lon * D2R, &sa, &ca);
-						inside = (GMT_non_zero_winding (r * ca, r * sa, pol[i].lon, pol[i].lat, pol[i].np) >= is_inside);
-					}
-					else
+					if (do_project)
 						inside = (GMT_non_zero_winding (xx, yy, pol[i].lon, pol[i].lat, pol[i].np) >= is_inside);
+					else
+						inside = (GMT_inonout_sphpol (xx, yy, &pol[i]) >= is_inside);
 					i++;
 				}
-				if (inside != want_inside[3]) continue;
+				if (inside != want_inside[3]) { output_header = need_header; continue;}
 			}
 
 			if (limit_coast) {
@@ -553,7 +544,7 @@ main (int argc, char **argv)
 						GMT_free_polygons (p[id], np[id]);
 						if (np[id]) GMT_free ((void *)p[id]);
 						np[id] = GMT_assemble_shore (&c, wd[id],  min_level, TRUE, greenwich, west_border, east_border, &p[id]);
-						np[id] = GMT_prep_polygons (&p[id], np[id], greenwich, FALSE, 0.0, -1);
+						np[id] = GMT_prep_polygons (&p[id], np[id], greenwich, do_project, step, -1);
 					}
 				}
 
@@ -564,13 +555,13 @@ main (int argc, char **argv)
 					this_node = 0;
 					GMT_geo_to_xy (lon, in[1], &xx, &yy);
 					for (id = 0; id < 2; id++) {
-		
+
 						for (k = 0; k < np[id]; k++) {
-		
+
 							if (p[id][k].n == 0) continue;
 
 							/* Find min/max of polygon */
-			
+
 							xmin = xmax = p[id][k].lon[0];
 							ymin = ymax = p[id][k].lat[0];
 
@@ -580,26 +571,31 @@ main (int argc, char **argv)
 								if (p[id][k].lat[i] < ymin) ymin = p[id][k].lat[i];
 								if (p[id][k].lat[i] > ymax) ymax = p[id][k].lat[i];
 							}
-							
+
 							if (yy < ymin || yy > ymax) continue;
 							if (xx < xmin || xx > xmax) continue;
 
 							/* Must compare with polygon */
 
 							if ((side = GMT_non_zero_winding (xx, yy, p[id][k].lon, p[id][k].lat, p[id][k].n)) < is_inside) continue;	/* Outside polygon */
-				
+
 							/* Here, point is inside, we must assign value */
-				
+
 							if (p[id][k].level > this_node) this_node = p[id][k].level;
 						}
 					}
 				}
 				inside = out_edge_in[this_node];
-				if (inside != want_inside[4]) continue;
+				if (inside != want_inside[4]) { output_header = need_header; continue;}
 			}
 
 			/* Here, we have passed all test and the point is output */
 
+			if (output_header) {	/* First output point for this segment - write the header */
+				GMT_write_segmentheader (GMT_stdout, n_expected_fields);
+				output_header = FALSE;
+			}
+
 			if (binary) {
 				if (n_output < 0) n_output = n_expected_fields;
 				GMT_output (GMT_stdout, n_output, in);
@@ -607,27 +603,27 @@ main (int argc, char **argv)
 			else
 				fprintf (GMT_stdout, "%s", GMT_io.current_record);
 			n_pass++;
-			
+
 			if (long_verbose && n_read%1000 == 0) fprintf (stderr, "%s: Read %d records\r", GMT_program, n_read);
 		}
 		if (fp != GMT_stdin) GMT_fclose (fp);
 	}
-	
+
 	if (gmtdefs.verbose) fprintf (stderr, "%s: Read %d records, passed %d records\n", GMT_program, n_read, n_pass);
 
-	if (limit_point_distance) {
-		GMT_free ((void *)xp);
-		GMT_free ((void *)yp);
-		GMT_free ((void *)dp);
-	}
+	if (limit_point_distance) GMT_points_delete (xp, yp, dp);
+	if (limit_line_distance) GMT_lines_delete (line, n_lines);
+	if (limit_polygon) GMT_lines_delete (pol, n_pols);
 	
 	GMT_end (argc, argv);
+
+	exit (EXIT_SUCCESS);
 }
 
 int compare_x (const void *point_1, const void *point_2)
 {
 	struct DATA *p1, *p2;
-	
+
 	p1 = (struct DATA *)point_1;
 	p2 = (struct DATA *)point_2;
 
