@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    $Id: grdblend.c,v 1.12 2004/01/13 02:33:04 pwessel Exp $
+ *    $Id: grdblend.c,v 1.26 2006/02/07 14:03:39 remko Exp $
  *
- *	Copyright (c) 1991-2004 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2006 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +13,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU General Public License for more details.
  *
- *	Contact info: www.soest.hawaii.edu/gmt
+ *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
  * grdblend reads any number of gridfiles that may partly overlap and
@@ -52,48 +52,53 @@ struct BLEND {	/* Structure with info about each input grid file */
 
 void sync_input_rows (int row, struct BLEND *blend, int n_blend);
 void decode_R (char *string, double *w, double *e, double *s, double *n);
-int init_blend_job (FILE *fp, double xmin, double xmax, double ymin, double ymax, double dx, double dy, BOOLEAN *pixel, struct BLEND **blend);
+int init_blend_job (FILE *fp, struct GRD_HEADER *h, struct BLEND **blend);
 
 double half;	/* 0 for gridline grids, 0.5 for pixel grids */
 
-main (int argc, char **argv)
+int main (int argc, char **argv)
 {
-	int i, col, row, k, kk, m, n_blend, pixel, one_or_zero, error = 0, n_fill, n_tot;
+	int i, col, row, k, kk, m, n_blend, one_or_zero, error = 0;
+	size_t n_fill, n_tot;
 	BOOLEAN write_header = TRUE, do_blend = TRUE;
 	BOOLEAN do_z_scale = FALSE;
-	double xmin, xmax, ymin, ymax, dx, dy, wt_x, w, wt, no_data_d, z_scale = 1.0;
+	double wt_x, w, wt, no_data_d, z_scale = 1.0;
 	float *z, no_data_f;
-	char *grdfile, mode[2] = {'W', 'w'};
+	char *grdfile, mode[2] = {'W', 'w'}, type;
 	FILE *fp = NULL;
 	struct BLEND *blend;
 	struct GMT_GRDFILE S;
-	
+
 	argc = GMT_begin (argc, argv);
 
+	GMT_grd_init (&S.header, argc, argv, FALSE);
+
 	grdfile = CNULL;
-	xmin = xmax = ymin = ymax = dx = dy = 0.0;
 	n_fill = n_tot = 0;
 	no_data_f = GMT_f_NaN;
 	no_data_d = GMT_d_NaN;
-	
+
 	for (i = 1; i < argc; i++) {
 		if (argv[i][0] == '-') {
 			switch (argv[i][1]) {
-			
+
 				/* Common parameters */
-			
+
 				case 'R':
 				case 'V':
 				case 'f':
 				case ':':
 				case '\0':
-					error += GMT_get_common_args (argv[i], &xmin, &xmax, &ymin, &ymax);
+					error += GMT_get_common_args (argv[i], &S.header.x_min, &S.header.x_max, &S.header.y_min, &S.header.y_max);
 					break;
 
 				/* Supplemental parameters */
-				
+
 				case 'I':
-					GMT_getinc (&argv[i][2], &dx, &dy);
+					if (GMT_getinc (&argv[i][2], &S.header.x_inc, &S.header.y_inc)) {
+						GMT_inc_syntax ('I', 1);
+						error = TRUE;
+					}
 					break;
 				case 'G':
 					grdfile = &argv[i][2];
@@ -132,21 +137,19 @@ main (int argc, char **argv)
 			exit (EXIT_FAILURE);
 		}
 	}
-	
+
 	if (argc == 1 || GMT_quick) {
 		fprintf (stderr,"grdblend %s - Blend several partially over-lapping grdfiles onto one grid\n\n", GMT_VERSION);
-		fprintf (stderr, "usage: grdblend [<blendfile>] -G<grdfile> -I<dx>[m|c][/<dy>[m|c]] -Rw/s/e/n[r]\n");
+		fprintf (stderr, "usage: grdblend [<blendfile>] -G<grdfile> -I<xinc[u][!|+]>[/<yinc>[u][!|+]] -Rw/s/e/n[r]\n");
 		fprintf (stderr, "\t[-N<nodata>] [-Q] [-Z<scale>] [-V] [-W] [-f[i|o]<colinfo>]\n");
-		
+
 		if (GMT_quick) exit (EXIT_FAILURE);
-		
+
 		fprintf (stderr, "\t<blendfile> is an ASCII file (or stdin) with blending parameters for each input grid\n");
 		fprintf (stderr, "\t   Each record has:  grdfilename -Rw/e/s/n weight\n");
 		fprintf (stderr, "\t   Relative weights are <weight> inside the given -R and cosine taper to 0 at actual -R.\n");
 		fprintf (stderr, "\t-G <grdfile> is the name of the final 2-D binary data set\n");
-		fprintf (stderr, "\t-I Sets Increment of the grid; enter xinc, optionally xinc/yinc.\n");
-		fprintf (stderr, "\t   Default is yinc = xinc.  Append an m [or c] to xinc or yinc to indicate minutes [or seconds],\n");
-		fprintf (stderr, "\t   e.g., -I10m/5m grids longitude every 10 minutes, latitude every 5 minutes.\n");
+		GMT_inc_syntax ('I', 0);
 		GMT_explain_option ('R');
 		fprintf (stderr, "\n\tOPTIONS:\n");
 		fprintf (stderr, "\t-N Set value for nodes without constraints [Default is NaN]\n");
@@ -159,12 +162,14 @@ main (int argc, char **argv)
 		GMT_explain_option ('.');
 		exit (EXIT_FAILURE);
 	}
-	
+
 	if (!project_info.region_supplied) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -R:  Must specify region\n", GMT_program);
 		error++;
 	}
-	if (dx <= 0.0 || dy <= 0.0) {
+	GMT_RI_prepare (&S.header);	/* Ensure -R -I consistency and set nx, ny */
+
+	if (S.header.x_inc <= 0.0 || S.header.y_inc <= 0.0) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -I:  Must specify positive dx, dy\n", GMT_program);
 		error++;
 	}
@@ -172,11 +177,12 @@ main (int argc, char **argv)
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -G:  Must specify output file\n", GMT_program);
 		error++;
 	}
-	if (!write_header && GRD_IS_CDF (GMT_grd_get_o_format (grdfile, S.name, &S.scale, &S.offset))) {
+	type = GMT_grdformats[GMT_grd_get_format (grdfile, &S.header)][0];
+	if (!write_header && (type == 'c' || type == 'n')) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -Q:  Output grdfile cannot be netCDF format\n", GMT_program);
 		error++;
 	}
-	
+
 	if (error) exit (EXIT_FAILURE);
 
 	GMT_put_history (argc, argv);	/* Update .gmtcommands4 */
@@ -187,56 +193,51 @@ main (int argc, char **argv)
 	}
 
 	/* Process blend parameters and populate blend structure and open input files and seek to first row inside the output grid */
-	
-	n_blend = init_blend_job (fp, xmin, xmax, ymin, ymax, dx, dy, &pixel, &blend);
-	
+
+	n_blend = init_blend_job (fp, &S.header, &blend);
+
 	if (fp != GMT_stdin) fclose (fp);
-	
+
 	if (!do_blend && n_blend > 1) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -W:  Only applies when there is a single input grdfile\n", GMT_program);
 		exit (EXIT_FAILURE);
 	}
 
 	/* Initialize header structure for output blend grid */
-	
-	GMT_grd_init (&S.header, argc, argv, FALSE);
-	one_or_zero = !pixel;
-	half = (pixel) ? 0.5 : 0.0;
-	
-	S.header.nx = irint ((xmax - xmin)/dx) + one_or_zero;
-	S.header.ny = irint ((ymax - ymin)/dy) + one_or_zero;
-	S.header.x_min = xmin;	S.header.x_max = xmax;
-	S.header.y_min = ymin;	S.header.y_max = ymax;
-	S.header.x_inc = dx;	S.header.y_inc = dy;
-	S.header.node_offset = pixel;
-	S.header.z_min = DBL_MAX;
-	S.header.z_max = -DBL_MAX;
+
+	one_or_zero = !S.header.node_offset;
+	half = (S.header.node_offset) ? 0.5 : 0.0;
+
+	S.header.z_min = S.header.z_max = 0.0;
 
 	GMT_grd_RI_verify (&S.header, 1);
 
-	n_tot = S.header.nx * S.header.ny;
+	n_tot = (size_t)S.header.nx * (size_t)S.header.ny;
 
 	z = (float *) GMT_memory (VNULL, (size_t)S.header.nx, sizeof (float), GMT_program);	/* Memory for one output row */
-	
+
 	if (write_header && GMT_write_grd_info (grdfile, &S.header)) {	/* Write the output header structure to file */
 		fprintf (stderr, "%s: Error opening file %s\n", GMT_program, grdfile);
 		exit (EXIT_FAILURE);
 	}
-	
+
 	GMT_open_grd (grdfile, &S, mode[write_header]);	/* Open the gridfile for further incremental writing */
-	
+
 	if (gmtdefs.verbose && do_z_scale) {
 		fprintf (stderr, "%s: Output data will be scaled by %g\n", GMT_program, z_scale);
 	}
-	
+
+	S.header.z_min = DBL_MAX;	/* These will be updated in the loop below */
+	S.header.z_max = -DBL_MAX;
+
 	for (row = 0; row < S.header.ny; row++) {	/* For every output row */
-	
+
 		memset ((void *)z, 0, (size_t)(S.header.nx * sizeof (float)));	/* Start from scratch */
-		
+
 		sync_input_rows (row, blend, n_blend);	/* Wind each input file to current record and read each of the overlapping rows */
-		
+
 		for (col = 0; col < S.header.nx; col++) {	/* For each output node on the current row */
-		
+
 			w = 0.0;
 			for (k = m = 0; k < n_blend; k++) {	/* Loop over every input grid */
 				if (blend[k].outside) continue;					/* This grid is currently outside the s/n range */
@@ -254,7 +255,7 @@ main (int argc, char **argv)
 				w += wt;							/* Add up the weight sum */
 				m++;								/* Add up the number of contributing grids */
 			}
-			
+
 			if (m) {		/* OK, at least one grid contributed to an output value */
 				if (do_blend) {		/* Want output z blend */
 					z[col] = (float)((w == 0.0) ? 0.0 : z[col] / w);	/* Get weighted average z */
@@ -270,7 +271,7 @@ main (int argc, char **argv)
 				z[col] = no_data_f;
 		}
 		GMT_write_grd_row (&S, 0, z);	/* Write out this weighted average row */
-		
+
 		if (gmtdefs.verbose && row%10 == 0)  fprintf (stderr, "%s: Processed row %7d of %d\r", GMT_program, row, S.header.ny);
 
 	}
@@ -282,11 +283,11 @@ main (int argc, char **argv)
 		exit (EXIT_FAILURE);
 	}
 	GMT_free ((void *)z);
-				
+
 	for (k = 0; k < n_blend; k++) GMT_close_grd (&blend[k].G);	/* Close all the input grd files */
 
 	if (gmtdefs.verbose) {
-		char empty[32];
+		char empty[GMT_TEXT_LEN];
 		fprintf (stderr, "%s: Blended grid size of %s is %d x %d\n", GMT_program, grdfile, S.header.nx, S.header.ny);
 		if (n_fill == n_tot)
 			fprintf (stderr, "%s: All nodes assigned values\n", GMT_program);
@@ -295,26 +296,28 @@ main (int argc, char **argv)
 				strcpy (empty, "NaN");
 			else
 				sprintf (empty, "%g", no_data_f);
-			fprintf (stderr, "%s: %d nodes assigned values, %d set to %s\n", GMT_program, n_fill, (n_tot - n_fill), empty);
+			fprintf (stderr, "%s: %ld nodes assigned values, %lu set to %s\n", GMT_program, n_fill, (n_tot - n_fill), empty);
 		}
 	}
-	
+
 	GMT_free ((void *)blend);
-	
+
 	GMT_end (argc, argv);
+
+	exit (EXIT_SUCCESS);
 }
 
-int init_blend_job (FILE *fp, double xmin, double xmax, double ymin, double ymax, double dx, double dy, BOOLEAN *pixel, struct BLEND **blend) {
-	int n = 0, nr, one_or_zero;
+int init_blend_job (FILE *fp, struct GRD_HEADER *h, struct BLEND **blend) {
+	int n = 0, nr, one_or_zero = 0;
 	size_t n_alloc = GMT_SMALL_CHUNK;
 	struct BLEND *B;
-	char line[BUFSIZ], r_in[256], file[256];
-	
+	char line[BUFSIZ], r_in[GMT_LONG_TEXT], file[GMT_LONG_TEXT], type;
+
 	B = (struct BLEND *) GMT_memory (VNULL, n_alloc, sizeof (struct BLEND), GMT_program);
-	
+
 	while (fgets (line, BUFSIZ, fp)) {	/* Read each input file for grid information */
 		if (line[0] == '#' || line[0] == '\n' || line[0] == '\r') continue;	/* Skip comment lines or blank lines */
-		
+
 		nr = sscanf (line, "%s %s %lf", file, r_in, &B[n].weight);
 		if (nr != 3) {
 			fprintf (stderr, "%s: Read error for blending parameters near row %d\n", GMT_program, n);
@@ -325,76 +328,77 @@ int init_blend_job (FILE *fp, double xmin, double xmax, double ymin, double ymax
 			fprintf (stderr, "%s: Error opening file %s\n", GMT_program, file);
 			exit (EXIT_FAILURE);
 		}
-		
+
 		/* Various sanity checking - e.g., all grid of same registration type and grid spacing */
-		
+
 		if (n == 0) {
-			*pixel = B[n].G.header.node_offset;
-			one_or_zero = !(*pixel);
+			h->node_offset = B[n].G.header.node_offset;
+			one_or_zero = !h->node_offset;
 		}
-		if ((*pixel) != B[n].G.header.node_offset){
+		if (h->node_offset != B[n].G.header.node_offset){
 			fprintf (stderr, "%s: File %s has different registration that the first file given\n", GMT_program, file);
 			exit (EXIT_FAILURE);
 		}
-		if (fabs (B[n].G.header.x_inc - dx) > GMT_CONV_LIMIT){
-			fprintf (stderr, "%s: File %s has different x-increment (%g) that the chosen output increment (%g)\n", GMT_program, file, B[n].G.header.x_inc, dx);
+		if (fabs (B[n].G.header.x_inc - h->x_inc) > GMT_CONV_LIMIT){
+			fprintf (stderr, "%s: File %s has different x-increment (%g) that the chosen output increment (%g)\n", GMT_program, file, B[n].G.header.x_inc, h->x_inc);
 			exit (EXIT_FAILURE);
 		}
-		if (fabs (B[n].G.header.y_inc - dy) > GMT_CONV_LIMIT){
-			fprintf (stderr, "%s: File %s has different y-increment (%g) that the chosen output increment (%g)\n", GMT_program, file, B[n].G.header.y_inc, dy);
+		if (fabs (B[n].G.header.y_inc - h->y_inc) > GMT_CONV_LIMIT){
+			fprintf (stderr, "%s: File %s has different y-increment (%g) that the chosen output increment (%g)\n", GMT_program, file, B[n].G.header.y_inc, h->y_inc);
 			exit (EXIT_FAILURE);
 		}
 		GMT_open_grd (file, &B[n].G, 'r');	/* Open the grid for incremental row reading */
 
 		/* Here, i0, j0 is the very first col, row to read, while i1, j1 is the very last col, row to read .
 		 * Weights at the outside i,j should be 0, and reach 1 at the edge of the inside block */
-		
+
 		/* The following works for both pixel and grid-registered grids since we are here using the i,j to measure the width of the
 		 * taper zone in units of dx, dy. */
 		 
-		B[n].out_i0 = irint ((B[n].G.header.x_min - xmin) / dx);
-		B[n].in_i0  = irint ((B[n].w_in - xmin) / dx) - 1;
-		B[n].in_i1  = irint ((B[n].e_in - xmin) / dx) + one_or_zero;
-		B[n].out_i1 = irint ((B[n].G.header.x_max - xmin) / dx) - B[n].G.header.node_offset;
-		B[n].out_j0 = irint ((ymax - B[n].G.header.y_max) / dy);
-		B[n].in_j0  = irint ((ymax - B[n].n_in) / dy) - 1;
-		B[n].in_j1  = irint ((ymax - B[n].s_in) / dy) + one_or_zero;
-		B[n].out_j1 = irint ((ymax - B[n].G.header.y_min) / dy) - B[n].G.header.node_offset;
-		
-		B[n].wxl = M_PI * dx / (B[n].w_in - B[n].G.header.x_min);
-		B[n].wxr = M_PI * dx / (B[n].G.header.x_max - B[n].e_in);
-		B[n].wyu = M_PI * dy / (B[n].G.header.y_max - B[n].n_in);
-		B[n].wyd = M_PI * dy / (B[n].s_in - B[n].G.header.y_min);
-		
+		B[n].out_i0 = irint ((B[n].G.header.x_min - h->x_min) / h->x_inc);
+		B[n].in_i0  = irint ((B[n].w_in - h->x_min) / h->x_inc) - 1;
+		B[n].in_i1  = irint ((B[n].e_in - h->x_min) / h->x_inc) + one_or_zero;
+		B[n].out_i1 = irint ((B[n].G.header.x_max - h->x_min) / h->x_inc) - B[n].G.header.node_offset;
+		B[n].out_j0 = irint ((h->y_max - B[n].G.header.y_max) / h->y_inc);
+		B[n].in_j0  = irint ((h->y_max - B[n].n_in) / h->y_inc) - 1;
+		B[n].in_j1  = irint ((h->y_max - B[n].s_in) / h->y_inc) + one_or_zero;
+		B[n].out_j1 = irint ((h->y_max - B[n].G.header.y_min) / h->y_inc) - B[n].G.header.node_offset;
+
+		B[n].wxl = M_PI * h->x_inc / (B[n].w_in - B[n].G.header.x_min);
+		B[n].wxr = M_PI * h->x_inc / (B[n].G.header.x_max - B[n].e_in);
+		B[n].wyu = M_PI * h->y_inc / (B[n].G.header.y_max - B[n].n_in);
+		B[n].wyd = M_PI * h->y_inc / (B[n].s_in - B[n].G.header.y_min);
+
 		if (B[n].out_j0 < 0) {	/* Must skip to first row inside the present -R */
-			if (GRD_IS_CDF (GMT_grd_i_format))
+			type = GMT_grdformats[B[n].G.header.type][0];
+			if (type == 'c' || type == 'n')
 				B[n].G.start[0] += B[n].G.header.nx * abs (B[n].out_j0);
 			else
 				fseek (B[n].G.fp, (long)(B[n].G.n_byte * abs (B[n].out_j0)), SEEK_CUR);
 		}
-		
+
 		/* Allocate space for one entire row */
-		
+
 		B[n].z = (float *) GMT_memory (VNULL, (size_t)(B[n].G.header.nx), sizeof (float), GMT_program);
 		if (gmtdefs.verbose) fprintf (stderr, "%s: Blend file %s in %g/%g/%g/%g with weight %g [%d-%d]\n",
-			GMT_program, B[n].G.name, B[n].w_in, B[n].e_in, B[n].s_in, B[n].n_in, B[n].weight, B[n].out_j0, B[n].out_j1);
-		
+			GMT_program, B[n].G.header.name, B[n].w_in, B[n].e_in, B[n].s_in, B[n].n_in, B[n].weight, B[n].out_j0, B[n].out_j1);
+
 		n++;
 		if (n == (int)n_alloc) {
 			n_alloc += GMT_SMALL_CHUNK;
 			B = (struct BLEND *) GMT_memory ((void *)B, n_alloc, sizeof (struct BLEND), GMT_program);
 		}
 	}
-	
+
 	B = (struct BLEND *) GMT_memory ((void *)B, n, sizeof (struct BLEND), GMT_program);
 	*blend = B;
-	
+
 	return (n);
 }
 
 void sync_input_rows (int row, struct BLEND *B, int n_blend) {
 	int k;
-	
+
 	for (k = 0; k < n_blend; k++) {	/* Get every input grid ready for the new row */
 		if (row < B[k].out_j0 || row > B[k].out_j1) {	/* Either done with grid or haven't gotten to this range yet */
 			B[k].outside = TRUE;
@@ -415,19 +419,17 @@ void sync_input_rows (int row, struct BLEND *B, int n_blend) {
 
 void decode_R (char *string, double *w, double *e, double *s, double *n) {
 	double *p[4];
-	int i, error = 0;
-	char *text;
-	
+	int i, pos, error = 0;
+	char text[BUFSIZ];
+
 	/* Needed to decode the inner region -Rw/e/s/n string */
-	
+
 	p[0] = w;	p[1] = e;	p[2] = s;	p[3] = n;
-			
-	i = 0;
-	text = strtok (string, "/");
-	while (text && !error) {
+
+	i = pos = 0;
+	while (!error && (GMT_strtok (string, "/", &pos, text))) {
 		error += GMT_verify_expectations (GMT_io.in_col_type[i/2], GMT_scanf_arg (text, GMT_io.in_col_type[i/2], p[i]), text);
 		i++;
-		text = strtok (CNULL, "/");
 	}
 	if (error || (i != 4) || GMT_check_region (*p[0], *p[1], *p[2], *p[3])) {
 		GMT_syntax ('R');
