@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: grdcontour.c,v 1.57 2004/09/10 23:16:15 pwessel Exp $
+ *	$Id: grdcontour.c,v 1.72 2006/02/21 02:50:16 remko Exp $
  *
- *	Copyright (c) 1991-2004 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2006 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -38,6 +38,7 @@
  */
 
 #include "gmt.h"
+#include "pslib.h"
 
 #define MIN_LENGTH 0.01
 
@@ -55,36 +56,38 @@ struct SAVE {
 
 void sort_and_plot_ticks (struct SAVE *save, int n, struct GRD_HEADER *h, float *grd, double tick_gap, double tick_length, BOOLEAN tick_low, BOOLEAN tick_high, BOOLEAN tick_label, char *labels);
 void GMT_grd_minmax (float *a, struct GRD_HEADER *h, double xyz[2][3]);
+void adjust_hill_label (struct GMT_CONTOUR *G, struct GRD_HEADER *h, float *z);
 
-main (int argc, char **argv)
+int main (int argc, char **argv)
 {
 	int i, j, k, n, nn, nm, c, n_edges, n_alloc;
 	int section, n_contours, smooth_factor = 0, side, id, bad;
 	int off, nx, ny, n_save = 0, got, ncut = 0, rgb[3];
-	
+
 	BOOLEAN error = FALSE, first = TRUE, dump = FALSE, interior, begin;
-	BOOLEAN do_ticks = FALSE, use_cpt_colors = FALSE, use_cpt_annot = TRUE;
+	BOOLEAN do_ticks = FALSE, use_cpt_colors = FALSE, use_cpt_annot = TRUE, subset = FALSE;
 	BOOLEAN tick_high = FALSE, tick_low = FALSE, cpt_given = FALSE, tick_label = FALSE;
-	
+
 	char *grdfile = 0, dfile[BUFSIZ], line[BUFSIZ], *cont_type, *cont_do_tick;
-	char *cpt_file, *labels, txt_a[32], txt_b[32], cont_label[128], format[128];
-	
+	char *cpt_file = CNULL, *labels = CNULL, txt_a[GMT_LONG_TEXT], txt_b[GMT_LONG_TEXT];
+	char cont_label[GMT_LONG_TEXT], format[GMT_LONG_TEXT];
+
 	double c_int = 0.0, c_low, c_high, aval, a_int = 0.0, tick_gap = 0.2;
 	double west = 0.0, east = 0.0, south = 0.0, north = 0.0, tick_length = 0.04, *xp, *yp;
 	double *contour, cval, mult = 1.0, shift = 0.0, min, max, small, label_angle = 0.0, xyz[2][3];
 	double small_x, small_y, data_west, data_east, data_south, data_north, tmp, *cont_angle;
-	
+
 	float *grd_original;
-	
+
 	FILE *fpc = NULL;
-	
+
 	struct GRD_HEADER header;
-	
+
 	struct GMT_PEN pen[2];
 	struct GMT_CONTOUR G;
 
 	argc = GMT_begin (argc, argv);
-	
+
 	GMT_init_pen (&pen[0], GMT_PENWIDTH);
 	GMT_init_pen (&pen[1], 3.0 * GMT_PENWIDTH);
 	c_low = -DBL_MAX;
@@ -95,13 +98,13 @@ main (int argc, char **argv)
 		tick_gap = 0.5 / 2.54;
 		tick_length = 0.1 / 2.54;
 	}
-	
+
 	for (i = 1; i < argc; i++) {
 		if (argv[i][0] == '-') {
 			switch (argv[i][1]) {
-		
+
 				/* Common parameters */
-			
+
 				case 'B':
 				case 'J':
 				case 'K':
@@ -116,12 +119,13 @@ main (int argc, char **argv)
 				case 'y':
 				case 'b':
 				case 'c':
+				case 'f':
 				case '\0':
 					error += GMT_get_common_args (argv[i], &west, &east, &south, &north);
 					break;
-				
+
 				/* Supplemental parameters */
-			
+
 				case 'A':
 					/* Format can be one of two:
 					 * 3.4.x: -A[-|aint][f<fontsize>][a<angle>][/<r/g/b>][t|o]
@@ -146,7 +150,7 @@ main (int argc, char **argv)
 						c_int = atof (&argv[i][2]);
 					else {
 						c_int = 1.0;
-						cpt_given = (int) strstr (&argv[i][2], ".cpt");
+						cpt_given = (strstr (&argv[i][2], ".cpt") != NULL) ? (BOOLEAN)TRUE : (BOOLEAN)FALSE;
 						cpt_file = &argv[i][2];
 					}
 					break;
@@ -171,7 +175,7 @@ main (int argc, char **argv)
 					if (argv[i][2])
 						strcpy (G.unit, &argv[i][2]);
 					else
-						strcpy (G.unit, "z_units");
+						strcpy (G.unit, "z");
 					break;
 				case 'S':
 					smooth_factor = atoi (&argv[i][2]);
@@ -241,7 +245,7 @@ main (int argc, char **argv)
 		else
 			grdfile = argv[i];
 	}
-	
+
 	if (argc == 1 || GMT_quick) {
 		fprintf (stderr,"grdcontour %s - Contouring of 2-D gridded data sets\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: grdcontour <grdfile> -C<cont_int> -J<params>\n");
@@ -249,9 +253,9 @@ main (int argc, char **argv)
 		fprintf (stderr, "\t[-K] [-L<Low/high>] [-M[<flag>]] [-O] [-P] [-Q<cut>] [-R<west/east/south/north>] [-S<smooth>]\n");
 		fprintf (stderr, "\t[-T[+|-][<gap>[c|i|m|p]/<length>[c|i|m|p]][:LH]] [-U[<label>]] [-V] [-W[+]<type><pen>]\n");
 		fprintf (stderr, "\t[-X<x_shift>] [-Y<y_shift>] [-Z[<fact>[/shift>]][p]] [-bo[s][<n>]] [-c<ncopies>]\n\n");
-		
+
 		if (GMT_quick) exit (EXIT_FAILURE);
-		
+
 		fprintf (stderr, "\t<grdfile> is 2-D netCDF grdfile to be contoured\n");
 		fprintf (stderr, "\t-C Contours to be drawn can be specified in one of three ways:\n");
 		fprintf (stderr, "\t   1. Fixed contour interval\n");
@@ -305,9 +309,10 @@ main (int argc, char **argv)
 		GMT_explain_option ('o');
 		GMT_explain_option ('n');
 		GMT_explain_option ('c');
+		GMT_explain_option ('f');
 		exit (EXIT_FAILURE);
 	}
-	
+
 	if (a_int > 0.0 && (!fpc && c_int == 0.0)) c_int = a_int;
 	if (!fpc && c_int <= 0.0) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -C option:  Must specify contour interval, file name with levels, or cpt-file\n", GMT_program);
@@ -346,28 +351,28 @@ main (int argc, char **argv)
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -W option:  + only valid if -C sets a cpt file\n", GMT_program);
 		error++;
 	}
-	
+
 	if (error) exit (EXIT_FAILURE);
 
 	GMT_put_history (argc, argv);	/* Update .gmtcommands4 */
 
 
 	if (dump && dfile[0] == 0) strcpy (dfile,"contour");
-	
+
 	if (gmtdefs.verbose) fprintf (stderr, "%s: Allocate memory and read data file\n", GMT_program);
-	
+
 	if (!strcmp (grdfile,  "=")) {
 		fprintf (stderr, "%s: Piping of grdfile not supported!\n", GMT_program);
 		exit (EXIT_FAILURE);
 	}
-	
+
 	if (GMT_read_grd_info (grdfile, &header)) {
 		fprintf (stderr, "%s: Error opening file %s\n", GMT_program, grdfile);
 		exit (EXIT_FAILURE);
 	}
 	off = (header.node_offset) ? 0 : 1;
-	if (!strcmp (G.unit, "z_units")) strcpy (G.unit, header.z_units);
-	
+	if (!strcmp (G.unit, "z")) strcpy (G.unit, header.z_units);
+
 	/* Determine what wesn to pass to map_setup */
 
 	if (!project_info.region_supplied) {
@@ -376,7 +381,8 @@ main (int argc, char **argv)
 		south = header.y_min;
 		north = header.y_max;
 	}
-
+	else if (!(west == header.x_min && east == header.x_max && south == header.y_min && north == header.y_max))
+		subset = TRUE;
 	GMT_map_setup (west, east, south, north);
 
 	/* Determine the wesn to be used to read the grdfile */
@@ -391,10 +397,18 @@ main (int argc, char **argv)
 		ps_plotend (gmtdefs.last_page);
 		GMT_end (argc, argv);
 	}
-	
 
 	/* Read data */
 
+	if (subset) {	/* Possibly extend the subset by 1 row/col since tiles need 4 corners to work */
+		if (GMT_x_to_i (data_west, header.x_min, header.x_inc, header.xy_off, header.nx) > 0) data_west -= header.x_inc;
+		if (GMT_x_to_i (data_east, header.x_min, header.x_inc, header.xy_off, header.nx) < (header.nx)) data_east += header.x_inc;
+		if (data_east > header.x_max) data_east -= header.x_inc;
+		if (GMT_y_to_j (data_south, header.y_min, header.y_inc, header.xy_off, header.ny) < (header.ny)) data_south -= header.y_inc;
+		if (GMT_y_to_j (data_north, header.y_min, header.y_inc, header.xy_off, header.ny) > 0) data_north += header.y_inc;
+		if (data_south < header.y_min) data_south += header.y_inc;
+	}
+
 	nx = irint ( (data_east - data_west) / header.x_inc) + off;
 	ny = irint ( (data_north - data_south) / header.y_inc) + off;
 	nm = nx * ny;
@@ -406,7 +420,7 @@ main (int argc, char **argv)
 
 	n_edges = header.ny * (int )ceil (header.nx / 16.0);
 	edge = (int *) GMT_memory (VNULL, (size_t)n_edges, sizeof (int), GMT_program);
-	
+
 	if (!(mult == 1.0 && shift == 0.0)) {
 		if (gmtdefs.verbose) fprintf (stderr, "%s: Subtracting %g and multiplying grid by %g\n", GMT_program, shift, mult);
 		for (i = 0; i < nm; i++) grd_original[i] = (float)((grd_original[i] - shift) * mult);
@@ -416,12 +430,12 @@ main (int argc, char **argv)
 	}
 	if (c_low > header.z_min) header.z_min = c_low;
 	if (c_high < header.z_max) header.z_max = c_high;
-	
+
 	small = c_int * 1.0e-6;
 	if (a_int == 0.0) a_int = c_int;
-	
+
 	small_x = 0.01 * header.x_inc;	small_y = 0.01 * header.y_inc;
-	
+
 	ps_plotinit (CNULL, gmtdefs.overlay, gmtdefs.page_orientation, gmtdefs.x_origin, gmtdefs.y_origin,
 		gmtdefs.global_x_scale, gmtdefs.global_y_scale, gmtdefs.n_copies, gmtdefs.dpi,
 		GMT_INCH, gmtdefs.paper_width, gmtdefs.page_rgb, gmtdefs.encoding.name, GMT_epsinfo (argv[0]));
@@ -430,20 +444,20 @@ main (int argc, char **argv)
 	if (project_info.three_D) ps_transrotate (-z_project.xmin, -z_project.ymin, 0.0);
 
 	GMT_map_clip_on (GMT_no_rgb, 3);
-		
+
 	if (G.annot) {	/* Wants annotated contours */
 		aval = floor (header.z_min / a_int) * a_int;
 		if (aval < header.z_min) aval += a_int;
 	}
 	else
 		aval = header.z_max + 1.0;
-	
+
 	n_alloc = GMT_CHUNK;
 	contour = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double), GMT_program);
 	cont_type = (char *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (char), GMT_program);
 	cont_do_tick = (char *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (char), GMT_program);
 	cont_angle = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double), GMT_program);
-	
+
 	if (cpt_given) {	/* Presumably got a cpt-file */
 		GMT_read_cpt (cpt_file);
 		n_contours = GMT_n_colors + 1;
@@ -497,7 +511,7 @@ main (int argc, char **argv)
 		fclose (fpc);
 	}
 	else {	/* Set up contour intervals automatically from c_int and a_int */
-		min = floor (header.z_min / c_int) * c_int; if (!GMT_z_periodic && min < header.z_min) min += c_int;	
+		min = floor (header.z_min / c_int) * c_int; if (!GMT_z_periodic && min < header.z_min) min += c_int;
 		max = ceil (header.z_max / c_int) * c_int; if (max > header.z_max) max -= c_int;
 		for (c = irint (min/c_int), n_contours = 0; c <= irint (max/c_int); c++, n_contours++) {
 			if (n_contours == n_alloc) {
@@ -522,30 +536,30 @@ main (int argc, char **argv)
 	cont_type = (char *) GMT_memory ((void *)cont_type, (size_t)n_contours, sizeof (char), GMT_program);
 	cont_do_tick = (char *) GMT_memory ((void *)cont_do_tick, (size_t)n_contours, sizeof (char), GMT_program);
 	cont_angle = (double *) GMT_memory ((void *)cont_angle, (size_t)n_contours, sizeof (double), GMT_program);
-	
+
 	if (do_ticks) save = (struct SAVE *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (struct SAVE), GMT_program);
 	GMT_grd_minmax (grd_original, &header, xyz);
 	if (GMT_contlabel_prep (&G, xyz, 3)) exit (EXIT_FAILURE);	/* Prep for crossing lines, if any */
-	
+
 	/* Because we are doing single-precision, we cannot subtract incrementally but must start with the
 	 * original grid values and subtract the current contour value. */
 	 
 	grd = (float *) GMT_memory (VNULL, (size_t)nm, sizeof (float), GMT_program);
 
 	for (c = 0; c < n_contours; c++) {	/* For each contour value cval */
-		
+
 		/* Reset markers and set up new zero-contour*/
-		
+
 		cval = contour[c];
 		if (gmtdefs.verbose) fprintf (stderr, "%s: Tracing the %g contour\n", GMT_program, cval);
-		
+
 		/* New approach to avoid round-off */
-		
+
 		for (i = 0; i < nm; i++) {
 			grd[i] = grd_original[i] - (float)cval;		/* If there are NaNs they will remain NaNs */
 			if (grd[i] == 0.0) grd[i] += (float)small;	/* There will be no actual zero-values, just -ve and +ve values */
 		}
-		
+
 		section = 0;
 		first = FALSE;
 		id = (cont_type[c] == 'A' || cont_type[c] == 'a') ? 1 : 0;
@@ -556,19 +570,19 @@ main (int argc, char **argv)
 			memcpy ((void *)&G.line_pen.rgb, (void *)rgb, (size_t)(3 * sizeof (int)));
 			if (!G.got_font_rgb && G.curved_text) memcpy ((void *)&G.font_rgb, (void *)rgb, (size_t)(3 * sizeof (int)));
 		}
-		
+
 		side = 0;
 		begin = TRUE;
 		while (side < 5) {
 			while ((n = GMT_contours (grd, &header, smooth_factor, gmtdefs.interpolant, &side, edge, begin, &x, &y)) > 0) {
-		
+
 				if (fabs (x[0] - x[n-1]) < small_x && fabs (y[0] - y[n-1]) < small_y) {
 					interior = TRUE;
 					x[n-1] = x[0];	y[n-1] = y[0];	/* Ensure exact closure */
 				}
 				else
 					interior = FALSE;
-					
+
 				if (n >= ncut) {
 					if (dump) GMT_dump_contour (x, y, n, cval, section, interior, dfile);
 					if ((nn = GMT_clip_to_map (x, y, n, &xp, &yp))) {	/* Lines inside the region */
@@ -577,6 +591,8 @@ main (int argc, char **argv)
 							GMT_get_format (cval, G.unit, CNULL, format);
 							sprintf (cont_label, format, cval);
 						}
+						else
+							cont_label[0] = (char)0;
 						if (cont_do_tick[c] && interior) {
 							save[n_save].x = (double *) GMT_memory (VNULL, (size_t)nn, sizeof (double), GMT_program);
 							save[n_save].y = (double *) GMT_memory (VNULL, (size_t)nn, sizeof (double), GMT_program);
@@ -592,7 +608,7 @@ main (int argc, char **argv)
 								save = (struct SAVE *) GMT_memory ((void *)save, (size_t)n_alloc, sizeof (struct SAVE), GMT_program);
 							}
 						}
-						GMT_hold_contour (&xp, &yp, nn, cont_label, cont_type[c], cont_angle[c], interior, &G);
+						GMT_hold_contour (&xp, &yp, nn, cval, cont_label, cont_type[c], cont_angle[c], interior, &G);
 						GMT_free ((void *)xp);
 						GMT_free ((void *)yp);
 					}
@@ -605,7 +621,7 @@ main (int argc, char **argv)
 			begin = FALSE;
 		}
 	}
-	
+
 	if (pen[0].texture || pen[1].texture) ps_setdash (CNULL, 0);
 
 	if (do_ticks && n_save) {
@@ -617,19 +633,22 @@ main (int argc, char **argv)
 		}
 		GMT_free ((void *)save);
 	}
+
+	
+	if (G.hill_label) adjust_hill_label (&G, &header, grd_original);	/* Must possibly adjust label angles so that label is readable when following contours */
 	
 	GMT_contlabel_plot (&G);
 
 	GMT_map_clip_off ();
 
 	GMT_map_basemap ();
-	
+
 	ps_setpaint (gmtdefs.background_rgb);
-	
+
 	if (project_info.three_D) ps_rotatetrans (z_project.xmin, z_project.ymin, 0.0);
 
 	ps_plotend (gmtdefs.last_page);
-	
+
 	GMT_free ((void *)grd);
 	GMT_free ((void *)grd_original);
 	GMT_free ((void *)edge);
@@ -638,27 +657,28 @@ main (int argc, char **argv)
 	GMT_free ((void *)cont_angle);
 	GMT_free ((void *)cont_do_tick);
 
-	
 	if (gmtdefs.verbose) fprintf (stderr, "%s: Done!\n", GMT_program);
-	
+
 	GMT_end (argc, argv);
+
+	exit (EXIT_SUCCESS);
 }
 
 void sort_and_plot_ticks (struct SAVE *save, int n, struct GRD_HEADER *h, float *grd, double tick_gap, double tick_length, BOOLEAN tick_low, BOOLEAN tick_high, BOOLEAN tick_label, char *labels)
 {
 	int np, i, j, inside, ix, iy, done, n_ticks;
 	double x0, y0, add, sign, dx, dy, x_back, y_back, x_front, y_front;
-	double xmin, xmax, ymin, ymax, x_mean, y_mean, inc, dist, a, small, this_lon, this_lat, sa, ca, *s;
+	double xmin, xmax, ymin, ymax, x_mean = 0.0, y_mean = 0.0, inc, dist, a, small, this_lon, this_lat, sa, ca, *s;
 	char txt[2][2];
-	
+
 	/* The x/y coordinates in SAVE are now all projected to map inches */
-	
+
 	small = 0.1 * project_info.xmax / h->nx;
 	if (tick_label) {
 		txt[0][0] = labels[0];	txt[0][1] = 0;
 		txt[1][0] = labels[1];	txt[1][1] = 0;
 	}
-	
+
 	for (i = 0; i < n; i++) {	/* Mark polygons that have other polygons inside them */
 		np = save[i].n;
 		for (j = 0; save[i].do_it && j < n; j++) {
@@ -666,16 +686,16 @@ void sort_and_plot_ticks (struct SAVE *save, int n, struct GRD_HEADER *h, float
 			if (inside == 2) save[i].do_it = FALSE;
 		}
 	}
-	
+
 	/* Here, only the polygons that are innermost (containing the local max/min, will have do_it = TRUE */
-	
+
 	for (i = 0; i < n; i++) {
 		if (!save[i].do_it) continue;
 		np = save[i].n;
 		s = (double *) GMT_memory (VNULL, (size_t)np, sizeof (double), GMT_program);
-		
+
 		/* Get rectangular surrounding box for this polygon */
-		
+
 		xmin = xmax = save[i].x[0];
 		ymin = ymax = save[i].y[0];
 		for (j = 1; j < np; j++) {
@@ -684,9 +704,9 @@ void sort_and_plot_ticks (struct SAVE *save, int n, struct GRD_HEADER *h, float
 			ymin = MIN (ymin, save[i].y[j]);
 			ymax = MAX (ymax, save[i].y[j]);
 		}
-		
+
 		/* Find a point inside the polygon to test for high or low */
-		
+
 		GMT_xy_to_geo (&this_lon, &this_lat, 0.5 * (xmin + xmax), ymax);
 		ix = irint ((this_lon - h->x_min) / h->x_inc) + 1;
 		iy = irint ((h->y_max - this_lat) / h->y_inc) + 1;
@@ -703,10 +723,10 @@ void sort_and_plot_ticks (struct SAVE *save, int n, struct GRD_HEADER *h, float
 		}
 		if (iy == h->ny) iy--;	/* Check to make sure we don't exceed array bounds in next line. However, iy may thus be wrong */ 
 		save[i].high = (grd[iy*h->nx+ix] > save[i].cval);
-		
+
 		if (save[i].high && !tick_high) continue;	/* Dont tick highs */
 		if (!save[i].high && !tick_low) continue;	/* Dont tick lows */
-		
+
 		if (tick_label) {
 			x_mean = save[i].x[0];
 			y_mean = save[i].y[0];
@@ -730,7 +750,7 @@ void sort_and_plot_ticks (struct SAVE *save, int n, struct GRD_HEADER *h, float
 		y0 = 0.5 * (save[i].y[0] + save[i].y[1]) + small * sa;
 		inside = GMT_non_zero_winding (x0, y0, save[i].x, save[i].y, np);
 		sign = (inside == 2) ? 1.0 : -1.0;
-		
+
 		n_ticks = (int)(s[np-1] / tick_gap);
 		if (n_ticks == 0) {
 			GMT_free ((void *)s);
@@ -742,7 +762,7 @@ void sort_and_plot_ticks (struct SAVE *save, int n, struct GRD_HEADER *h, float
 			y_mean /= np;
 			GMT_text3D (x_mean, y_mean, project_info.z_level, gmtdefs.annot_font_size[0], gmtdefs.annot_font[0], txt[save[i].high], 0.0, 6, 0);
 		}
-		
+
 		x_back = save[i].x[np-1];
 		y_back = save[i].y[np-1];
 		dist = 0.0;
@@ -782,7 +802,7 @@ void GMT_grd_minmax (float *a, struct GRD_HEADER *h, double xyz[2][3])
 	int i, i_min, i_max;
 	float z_min, z_max;
 	double half;
-	
+
 	z_min = FLT_MAX;	z_max = -FLT_MAX;
 	i_min = i_max = 0;
 	half = (h->node_offset) ? 0.5 : 0.0;
@@ -803,3 +823,45 @@ void GMT_grd_minmax (float *a, struct GRD_HEADER *h, double xyz[2][3])
 	xyz[1][1] = h->y_max - (i_max/h->nx + half) * h->y_inc;
 	xyz[0][2] = z_min;	xyz[1][2] = z_max;
 }
+
+void adjust_hill_label (struct GMT_CONTOUR *G, struct GRD_HEADER *h, float *z)
+{	
+	int i, k, i0, j0;
+	double nx, ny, x_on, y_on, x_node, y_node, x_node_p, y_node_p, dx, dy, dz, dot, angle;
+	struct GMT_CONTOUR_LINE *C;
+	
+	for (i = 0; i < G->n_segments; i++) {
+		C = G->segment[i];	/* Pointer to current segment */
+		for (k = 0; k < C->n_labels; k++) {
+			GMT_xy_to_geo (&x_on, &y_on, C->L[k].x, C->L[k].y);	/* Retrieve original coordinates */
+			j0 = GMT_y_to_j (y_on, h->y_min, h->y_inc, h->xy_off, h->ny);
+			if (j0 < 0 || j0 >= h->ny) continue;	/* Somehow, outside y range */
+			while (GMT_io.in_col_type[0] == GMT_IS_LON && x_on < h->x_min) x_on += 360.0;
+			while (GMT_io.in_col_type[0] == GMT_IS_LON && x_on > h->x_max) x_on -= 360.0;
+			i0 = GMT_x_to_i (x_on, h->x_min, h->x_inc, h->xy_off, h->nx);
+			if (i0 < 0 || i0 >= h->nx) continue;	/* Somehow, outside x range */
+			angle = fmod (2.0 * C->L[k].angle, 360.0) * 0.5;	/* 0-180 range */
+			if (angle > 90.0) angle -= 180.0;
+			sincos ((angle + 90) * D2R, &ny, &nx);	/* Coordinate of normal to label line */
+			x_node = GMT_i_to_x (i0, h->x_min, h->x_max, h->x_inc, h->xy_off, h->nx);
+			y_node = GMT_j_to_y (j0, h->y_min, h->y_max, h->y_inc, h->xy_off, h->ny);
+			GMT_geo_to_xy (x_node, y_node, &x_node_p, &y_node_p);	/* Projected coordinates of nearest node point */
+			dx = x_node_p - C->L[k].x;
+			dy = y_node_p - C->L[k].y;
+			if (hypot (dx, dy) < GMT_CONV_LIMIT) {
+				fprintf (stderr, "%s: Unable to adjust hill label contour orientation (node point on contour)\n", GMT_program);
+				continue;
+			}
+			dz = z[j0*h->nx+i0] - C->z;
+			if (fabs (dz) < GMT_CONV_LIMIT) {
+				fprintf (stderr, "%s: Unable to adjust hill label contour orientation (node value = contour value)\n", GMT_program);
+				continue;
+			}
+			dot = dx * nx + dy * ny;	/* Dot product of n and vector from contour to node. +ve if on same side of contour line */
+			if (irint (copysign (1.0, dot * dz)) != G->hill_label)
+				C->L[k].angle += 180.0;	/* Must turn upside-down */
+		}
+	}
+}
+
+	
