@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: grdfft.c,v 1.9 2004/01/02 22:45:13 pwessel Exp $
+ *	$Id: grdfft.c,v 1.15 2005/12/17 05:59:22 pwessel Exp $
  *
- *	Copyright (c) 1991-2004 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2006 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -92,11 +92,11 @@ int parse_f_string(char *c), read_data(int argc, char **argv), get_prime_factors
 void do_continuation(double *zlevel), do_azimuthal_derivative(double *azim), do_differentiate(double *par), do_integrate(double *par), do_isostasy(double *par);
 void do_filter(void), do_spectrum(double *par, int give_wavelength), remove_plane(void), taper_edges(void);
 
-main (int argc, char **argv)
+int main (int argc, char **argv)
 {
 
 	int	op_count = 0, n_op_count = 0, par_count = 0, n_par = 0, *operation = NULL;
-	BOOLEAN error = FALSE, stop, chose_spectrum = FALSE, give_wavelength;
+	BOOLEAN error = FALSE, stop, chose_spectrum = FALSE, give_wavelength = FALSE;
 	double	*par = NULL;
 
 	for (i = 0; i < 3; i++) {
@@ -172,7 +172,6 @@ main (int argc, char **argv)
 					n_par++;
 					par = (double *) GMT_memory ((void *)par, (size_t)n_par, sizeof(double), GMT_program);
 					par[par_count] = 0.0;
-					give_wavelength = FALSE;
 					j = 2;
 					while(argv[i][j]) {
  						if (argv[i][j] == 'x' || argv[i][j] == 'X')
@@ -415,7 +414,7 @@ main (int argc, char **argv)
 	exit (EXIT_SUCCESS);
 }
 
-read_data(int argc, char **argv)
+int read_data(int argc, char **argv)
 {
 	int	worksize, factors[32];
 	double	tdummy, edummy;
@@ -499,8 +498,8 @@ read_data(int argc, char **argv)
 	delta_ky = 2 * M_PI / (ny2 * h.y_inc);
 	if (map_units) {
 		/* Give delta_kx, delta_ky units of 2pi/meters  */
-		delta_kx /= (DEG2M * cos(0.5 * (h.y_min + h.y_max) * D2R) );
-		delta_ky /= DEG2M;
+		delta_kx /= (project_info.M_PR_DEG * cosd (0.5 * (h.y_min + h.y_max)) );
+		delta_ky /= project_info.M_PR_DEG;
 	}
 
 	return(0);
@@ -787,7 +786,7 @@ double	f_wt(int k)
 			case 1:
 				freq = kx(k);
 				break;
-			case 2:
+			default:
 				freq = ky(k);
 				break;
 		}
@@ -808,9 +807,9 @@ double	f_wt(int k)
 
 int	parse_f_string(char *c)
 {
-	int	i, j, n_tokens, descending;
+	int	i, j, n_tokens, pos, descending;
 	double	fourvals[4];
-	char	line[256], *p;
+	char	line[GMT_LONG_TEXT], p[GMT_LONG_TEXT];
 	
 	strcpy(line, c);
 	i = j = 0;	/* j is Filter type:  r=0, x=1, y=2  */
@@ -826,9 +825,8 @@ int	parse_f_string(char *c)
 	
 	f_info.do_this[j] = TRUE;
 	
-	n_tokens = 0;
-	p = strtok(&line[i],"/");
-	while (p) {
+	n_tokens = pos = 0;
+	while ((GMT_strtok (&line[i], "/", &pos, p))) {
 		if (n_tokens > 3) {
 			fprintf(stderr,"%s: Too many slashes in -F.\n", GMT_program);
 			return(TRUE);
@@ -843,7 +841,6 @@ int	parse_f_string(char *c)
 			}
 		}
 		n_tokens++;
-		p = strtok(CNULL, "/");
 	}
 	
 	if (n_tokens != 4) {
@@ -901,7 +898,7 @@ void	do_spectrum(double *par, int give_wavelength)
 	int	k, nk, nused, ifreq;
 	double	delta_k, r_delta_k, freq, *power, eps_pow;
 	PFD	get_k;
-	char	format[64];
+	char	format[GMT_TEXT_LEN];
 	/*  Added by WHFS in version 3.001 to normalize power  :  */
 	double	powfactor;
 
