@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: grdimage.c,v 1.29 2004/09/23 21:17:42 pwessel Exp $
+ *	$Id: grdimage.c,v 1.41 2006/02/21 04:10:19 remko Exp $
  *
- *	Copyright (c) 1991-2004 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2006 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -27,8 +27,9 @@
  * Date:	17-SEP-2001
  * Ver:		4
  */
- 
+
 #include "gmt.h"
+#include "pslib.h"
 
 float *tmp1[3], *tmp2, *map[3], *intensity;
 unsigned char *bitimage_8, *bitimage_24;
@@ -41,35 +42,37 @@ char *c_method[2] = {
 void GMT_limits (struct GRD_HEADER *r, struct GRD_HEADER *g);
 void GMT_set_proj_limits (struct GRD_HEADER *r, struct GRD_HEADER *g);
 
-main (int argc, char **argv)
+int main (int argc, char **argv)
 {
-	int i, j, k, kk, rgb[3], nm, nm2, byte, off, nx_f, ny_f, grid_type, polarity, PS_interpolate = 1;
-	int nx, ny, dpi = 0, nx_proj = 0, ny_proj = 0, node, f_rgb[3], tiling = 0, n_grids = 0, index;
+	int i, j, k, kk, rgb[3], nm, nm2 = 0, byte, one_or_zero, nx_f, ny_f, grid_type = 0, PS_interpolate = 1;
+	int nx, ny, dpi = 0, nx_proj = 0, ny_proj = 0, node, f_rgb[3], b_rgb[3], tiling = 0, n_grids = 0, index;
 	int PS_colormask_off = 0, PS_colormask = 1;
-	
+
 	BOOLEAN error = FALSE, intens = FALSE, monochrome = FALSE, do_rgb = FALSE, need_to_project;
-	BOOLEAN set_dpi = FALSE, subset = FALSE, transparent = FALSE, tile_outline = FALSE;
-	
+	BOOLEAN set_dpi = FALSE, tile_outline = FALSE, normal_x, normal_y;
+
 	double  dx, dy, x_side, y_side, x0 = 0.0, y0 = 0.0, max_radius = 0.0;
 	double west, east, south, north, data_west, data_east, data_south, data_north;
-	
+
 	char *grdfile[3], *intensfile, *cpt_file;
-	
+
 	struct GRD_HEADER g_head[3], r_head[3], i_head, j_head;
-	
+
 	struct GMT_PEN tile_pen;
-	
+
 	argc = GMT_begin (argc, argv);
 
 	grdfile[0] = grdfile[1] = grdfile[2] = intensfile = cpt_file = CNULL;
 	west = east = south = north = 0.0;
 	GMT_init_pen (&tile_pen, GMT_PENWIDTH);
-	
+	f_rgb[0] = f_rgb[1] = f_rgb[2] = 0;
+	b_rgb[0] = b_rgb[1] = b_rgb[2] = 255;
+
 	for (i = 1; i < argc; i++) {
 		if (argv[i][0] == '-') {
 			switch (argv[i][1]) {
 				/* Common parameters */
-			
+
 				case 'B':
 				case 'J':
 				case 'K':
@@ -83,12 +86,13 @@ main (int argc, char **argv)
 				case 'Y':
 				case 'y':
 				case 'c':
+				case 'f':
 				case '\0':
 					error += GMT_get_common_args (argv[i], &west, &east, &south, &north);
 					break;
-				
+
 				/* Supplemental parameters */
-			
+
 				case 'C':
 					cpt_file = &argv[i][2];
 					break;
@@ -101,14 +105,33 @@ main (int argc, char **argv)
 						set_dpi = TRUE;
 					break;
 				case 'G':	/* 1-bit fore or background color for transparent masks */
-					transparent = TRUE;
-					polarity = 1;
-					j = 2;
-					if (argv[i][2] == 'F' || argv[i][2] == 'f') j++;
-					if (argv[i][2] == 'B' || argv[i][2] == 'b') polarity = 0, j++;
-					if (GMT_getrgb (&argv[i][j], f_rgb)) {
-						GMT_rgb_syntax ('G');
-						error++;
+					switch (argv[i][2]) {
+						case 'F':
+						case 'f':
+							if (GMT_getrgb (&argv[i][3], f_rgb)) {
+								GMT_rgb_syntax ('G');
+								error++;
+							}
+							else
+								b_rgb[0] = -1;
+							break;
+						case 'B':
+						case 'b':
+							if (GMT_getrgb (&argv[i][3], b_rgb)) {
+								GMT_rgb_syntax ('G');
+								error++;
+							}
+							else
+								f_rgb[0] = -1;
+							break;
+						default:	/* Same as -Gf */
+							if (GMT_getrgb (&argv[i][2], f_rgb)) {
+								GMT_rgb_syntax ('G');
+								error++;
+							}
+							else
+								b_rgb[0] = -1;
+							break;
 					}
 					break;
 				case 'I':
@@ -156,15 +179,15 @@ main (int argc, char **argv)
 			grdfile[n_grids++] = argv[i];
 		}
 	}
-	
+
 	if (argc == 1 || GMT_quick) {
 		fprintf (stderr,"grdimage %s - Plot grdfiles in 2-D\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: grdimage <grd_z|grd_r grd_g grd_b> -J<params> [-B<tickinfo>] [-C<cpt_file>] [-Ei|<dpi>] [-G[f|b]<rgb>]\n");
 		fprintf (stderr, "\t[-I<intensity_file>] [-K] [-M] [-O] [-P] [-Q] [-R<w/e/s/n>] [-S<radius>] [-T[s][o[<pen>]]] [-U] [-V]\n");
 		fprintf (stderr, "\t[-X<x_shift>] [-Y<y_shift>] [-c<ncopies>]\n\n");
-		
+
 		if (GMT_quick) exit (EXIT_FAILURE);
-		
+
 		fprintf (stderr,"\t<grd_z> is data set to be plotted.  Its z-values are in user units and will be\n");
 		fprintf (stderr,"\t  converted to rgb colors via the cpt file.  Alternatively, give three separate\n");
 		fprintf (stderr,"\t  grid files that contain the red, green, and blue components in the 0-255 range.\n");
@@ -192,11 +215,16 @@ main (int argc, char **argv)
 		GMT_explain_option ('V');
 		GMT_explain_option ('X');
 		GMT_explain_option ('c');
+		GMT_explain_option ('f');
 		GMT_explain_option ('.');
 		exit (EXIT_FAILURE);
 	}
-	
+
+	/* Get color palette file */
+
 	do_rgb = (n_grids == 3);
+	if (!do_rgb) GMT_read_cpt (cpt_file);
+
 	for (i = 0; i < n_grids; i++) {
 		if (!grdfile[i]) {
 			fprintf (stderr, "%s: GMT SYNTAX ERROR:  Must specify input file\n", GMT_program);
@@ -215,22 +243,22 @@ main (int argc, char **argv)
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -E option:  dpi must be positive\n", GMT_program);
 		error++;
 	}
+	if (f_rgb[0] < 0 && b_rgb[0] < 0) {
+		fprintf (stderr, "%s: GMT SYNTAX ERROR -G option:  Only one of fore/back-ground can be transparent for 1-bit images\n", GMT_program);
+		error++;
+	}
 	if (error) exit (EXIT_FAILURE);
 
 	GMT_put_history (argc, argv);	/* Update .gmtcommands4 */
-	
-	/* Get color palette file */
-	
-	if (!do_rgb) GMT_read_cpt (cpt_file);
 
 	/* Check limits and get data file */
-	
+
 	if (gmtdefs.verbose) fprintf (stderr, "%s: Allocates memory and read data file\n", GMT_program);
-	
+
 	for (i = 0; i < n_grids; i++) {
 		GMT_grd_init (&g_head[i], argc, argv, FALSE);
 		GMT_grd_init (&r_head[i], argc, argv, FALSE);
-	
+
 		if (GMT_read_grd_info (grdfile[i], &g_head[i])) {
 			fprintf (stderr, "%s: Error opening file %s\n", GMT_program, grdfile[i]);
 			exit (EXIT_FAILURE);
@@ -248,7 +276,7 @@ main (int argc, char **argv)
 			exit (EXIT_FAILURE);
 		}
 	}
-	off = (g_head[0].node_offset) ? 0 : 1;
+	one_or_zero = (g_head[0].node_offset) ? 0 : 1;
 
 	/* Determine what wesn to pass to map_setup */
 
@@ -258,9 +286,7 @@ main (int argc, char **argv)
 		south = g_head[0].y_min;
 		north = g_head[0].y_max;
 	}
-	else if (!(west == g_head[0].x_min && east == g_head[0].x_max && south == g_head[0].y_min && north == g_head[0].y_max))
-		subset = TRUE;
-
+	
 	GMT_map_setup (west, east, south, north);
 
 	/* Determine the wesn to be used to read the grdfile; or exit if file is outside -R */
@@ -278,13 +304,13 @@ main (int argc, char **argv)
 
 	nx_f = g_head[0].nx;
 	ny_f = g_head[0].ny;
-	
+
 	/* Read data */
 
-	nx = irint ( (data_east - data_west) / g_head[0].x_inc) + off;
-	ny = irint ( (data_north - data_south) / g_head[0].y_inc) + off;
+	nx = irint ( (data_east  - data_west)  / g_head[0].x_inc) + one_or_zero;
+	ny = irint ( (data_north - data_south) / g_head[0].y_inc) + one_or_zero;
 	nm = nx * ny;
-	
+
 	for (i = 0; i < n_grids; i++) {
 		tmp1[i] = (float *) GMT_memory (VNULL, (size_t)nm, sizeof (float), GMT_program);
 		if (GMT_read_grd (grdfile[i], &g_head[i], tmp1[i], data_west, data_east, data_south, data_north, GMT_pad, FALSE)) {
@@ -294,14 +320,14 @@ main (int argc, char **argv)
 	}
 
 	/* If given, get intensity file or compute intensities */
-	
+
 	if (intens) {	/* Illumination wanted */
-	
+
 		if (gmtdefs.verbose) fprintf (stderr, "%s: Allocates memory and read intensity file\n", GMT_program);
-		
+
 		GMT_grd_init (&i_head, argc, argv, FALSE);
 		GMT_grd_init (&j_head, argc, argv, FALSE);
-		
+
 		if (GMT_read_grd_info (intensfile, &i_head)) {
 			fprintf (stderr, "%s: Error opening file %s\n", GMT_program, intensfile);
 			exit (EXIT_FAILURE);
@@ -312,20 +338,20 @@ main (int argc, char **argv)
 			exit (EXIT_FAILURE);
 		}
 		tmp2 = (float *) GMT_memory (VNULL, (size_t)nm, sizeof (float), GMT_program);
-		
+
 		if (GMT_read_grd (intensfile, &i_head, tmp2, data_west, data_east, data_south, data_north, GMT_pad, FALSE)) {
 			fprintf (stderr, "%s: Error reading file %s\n", GMT_program, intensfile);
 			exit (EXIT_FAILURE);
 		}
 	}
-	
+
 	for (i = 0; i < n_grids; i++) GMT_set_proj_limits (&r_head[i], &g_head[i]);
-	need_to_project = (project_info.projection > 5 || (project_info.projection == LINEAR && (project_info.xyz_projection[0] + project_info.xyz_projection[1]) > 0));
-	
+	need_to_project = (project_info.projection != LINEAR || project_info.xyz_projection[0] == LOG10 || project_info.xyz_projection[0] == POW || project_info.xyz_projection[1] == LOG10 || project_info.xyz_projection[1] == POW);
+
 	if (!tiling && (dpi > 0 || need_to_project)) {	/* Need to resample the grd file */
-		
+
 		if (gmtdefs.verbose) fprintf (stderr, "%s: project grdfiles\n", GMT_program);
-	
+
 		if (dpi == 0) {	/* Use input # of nodes as # of projected nodes */
 			nx_proj = g_head[0].nx;
 			ny_proj = g_head[0].ny;
@@ -342,7 +368,7 @@ main (int argc, char **argv)
 		if (intens) {
 			j_head.x_min = r_head[0].x_min;	j_head.x_max = r_head[0].x_max;
 			j_head.y_min = r_head[0].y_min;	j_head.y_max = r_head[0].y_max;
-		
+
 			if (dpi == 0) {	/* Use input # of nodes as # of projected nodes */
 				nx_proj = i_head.nx;
 				ny_proj = i_head.ny;
@@ -353,7 +379,7 @@ main (int argc, char **argv)
 			GMT_free ((void *)tmp2);
 		}
 		nm = nm2;
-		project_info.xyz_pos[0] = project_info.xyz_pos[1] = 1;	/* Since we have applied the projection we do no longer have 'negative' axes */
+		/* project_info.xyz_pos[0] = project_info.xyz_pos[1] = 1; */	/* Since we have applied the projection we do no longer have 'negative' axes */
 	}
 	else {	/* Simply copy g_head[0] info to r_head[0] */
 		for (i = 0; i < n_grids; i++) {
@@ -368,41 +394,40 @@ main (int argc, char **argv)
 		}
 		grid_type = g_head[0].node_offset;
 	}
-	
+
 	ps_plotinit (CNULL, gmtdefs.overlay, gmtdefs.page_orientation, gmtdefs.x_origin, gmtdefs.y_origin,
 		gmtdefs.global_x_scale, gmtdefs.global_y_scale, gmtdefs.n_copies,
 		gmtdefs.dpi, GMT_INCH , gmtdefs.paper_width, gmtdefs.page_rgb, gmtdefs.encoding.name, GMT_epsinfo (argv[0]));
 	GMT_echo_command (argc, argv);
 	if (gmtdefs.unix_time) GMT_timestamp (argc, argv);
-	
+
 	GMT_map_clip_on (GMT_no_rgb, 3);
 
 	if (tiling) {	/* Plot image as polygonal pieces and then exit */
-		int np, index, plot_n, n_use, start;
-		double *grd_x, grd_y, *xx, *yy, *xp, *yp, d, dx2, dy2, y_bot, y_top;
+		int np, index = 0, plot_n, n_use, start;
+		double *grd_x, grd_y, *xx, *yy, *xp, *yp, dx2, dy2, y_bot, y_top;
 		struct GMT_FILL *f;
 		BOOLEAN skip;
-		
-		d = (off) ? 0.0 : 0.5;
+
 		dx2 = 0.5 * g_head[0].x_inc;
 		dy2 = 0.5 * g_head[0].y_inc;
-		
+
 		if (gmtdefs.verbose) fprintf (stderr, "%s: Tiling without interpolation\n", GMT_program);
 		grd_x = (double *) GMT_memory (VNULL, (size_t)g_head[0].nx, sizeof (double), GMT_program);
-		
+
 		if (tile_outline) GMT_setpen (&tile_pen);
 		for (i = 0; i < g_head[0].nx; i++) {
-			grd_x[i] = g_head[0].x_min + (i + d) * g_head[0].x_inc;
+			grd_x[i] = GMT_i_to_x (i, g_head[0].x_min, g_head[0].x_max, g_head[0].x_inc, g_head[0].xy_off, g_head[0].nx);
 			if (GMT_io.in_col_type[0] == GMT_IS_LON) {
 				if (grd_x[i] < project_info.w && (grd_x[i] + 360.0) <= project_info.e) grd_x[i] += 360.0;
 				if (grd_x[i] > project_info.e && (grd_x[i] - 360.0) >= project_info.w) grd_x[i] -= 360.0;
 			}
 		}
 		for (j = node = 0; j < g_head[0].ny; j++) {
-			grd_y = g_head[0].y_max - (j + d) * g_head[0].y_inc;
+			grd_y = GMT_j_to_y (j, g_head[0].y_min, g_head[0].y_max, g_head[0].y_inc, g_head[0].xy_off, g_head[0].ny);
 			y_bot = grd_y - dy2;
 			y_top = grd_y + dy2;
-			if (MAPPING) {
+			if (project_info.degree[1]) {
 				if (y_bot < -90.0) y_bot = -90.0;
 				if (y_top > +90.0) y_top = +90.0;
 			}
@@ -432,18 +457,18 @@ main (int argc, char **argv)
 				}
 				if (intens && index != -1) GMT_illuminate (intensity[node], rgb);
 				if (monochrome) rgb[0] = rgb[1] = rgb[2] = YIQ (rgb);
-				
+
 				np = GMT_graticule_path (&xx, &yy, 1, grd_x[i] - dx2, grd_x[i] + dx2, y_bot, y_top);
-				
+
 				plot_n = GMT_clip_to_map (xx, yy, np, &xp, &yp);
 				GMT_free ((void *)xx);
 				GMT_free ((void *)yy);
 				if (plot_n == 0) continue;	/* Outside */
 
 				if ((*GMT_will_it_wrap) (xp, yp, plot_n, &start)) {	/* Polygon wraps */
-					
+
 					/* First truncate agains left border */
-						
+
 					GMT_n_plot = (*GMT_truncate) (xp, yp, plot_n, start, -1);
 					n_use = GMT_compact_line (GMT_x_plot, GMT_y_plot, GMT_n_plot, FALSE, 0);
 					if (do_rgb)
@@ -454,9 +479,9 @@ main (int argc, char **argv)
 						GMT_fill (GMT_x_plot, GMT_y_plot, n_use, f, tile_outline);
 					else
 						ps_polygon (GMT_x_plot, GMT_y_plot, n_use, rgb, tile_outline);
-													
+
 					/* Then truncate agains right border */
-						
+
 					GMT_n_plot = (*GMT_truncate) (xp, yp, plot_n, start, +1);
 					n_use = GMT_compact_line (GMT_x_plot, GMT_y_plot, GMT_n_plot, FALSE, 0);
 					if (do_rgb)
@@ -467,7 +492,7 @@ main (int argc, char **argv)
 						GMT_fill (GMT_x_plot, GMT_y_plot, n_use, f, tile_outline);
 					else
 						ps_polygon (GMT_x_plot, GMT_y_plot, n_use, rgb, tile_outline);
-					
+
 				}
 				else {
 					if (do_rgb)
@@ -483,25 +508,25 @@ main (int argc, char **argv)
 				GMT_free ((void *)yp);
 			}
 		}
-		
+
 		GMT_free ((void *)grd_x);
 		for (i = 0; i < n_grids; i++) GMT_free ((void *)map[i]);
 		if (intens) GMT_free ((void *)intensity);
-	
+
 		GMT_map_clip_off();
-	
+
 		GMT_map_basemap ();
 
 		ps_plotend (gmtdefs.last_page);
-	
+
 		GMT_end (argc, argv);
 	}
-				
+
 	if (gmtdefs.verbose) {
 		if (GMT_cpt_pattern) fprintf (stderr, "%s: Warning: Patterns in cpt file only apply to -T\n", GMT_program);
 		fprintf (stderr, "%s: Evaluate pixel colors\n", GMT_program);
 	}
-	
+
 	if (monochrome || GMT_gray)
 		bitimage_8 = (unsigned char *) GMT_memory (VNULL, (size_t)nm, sizeof (char), GMT_program);
 	else {
@@ -513,10 +538,12 @@ main (int argc, char **argv)
 			bitimage_24[2] = (unsigned char)GMT_bfn[GMT_NAN].rgb[2];
 		}
 	}
+	normal_x = !(project_info.projection == LINEAR && !project_info.xyz_pos[0]);
+	normal_y = !(project_info.projection == LINEAR && !project_info.xyz_pos[1]);
 	for (j = 0, byte = PS_colormask_off; j < r_head[0].ny; j++) {
-		kk = r_head[0].nx * (project_info.xyz_pos[1] ? j : r_head[0].ny - j - 1);
+		kk = r_head[0].nx * (normal_y ? j : r_head[0].ny - j - 1);
 		for (i = 0; i < r_head[0].nx; i++) {	/* Compute rgb for each pixel */
-			node = kk + (project_info.xyz_pos[0] ? i : r_head[0].nx - i - 1);
+			node = kk + (normal_x ? i : r_head[0].nx - i - 1);
 			if (do_rgb) {
 				for (k = 0; k < 3; k++) {
 					if (GMT_is_fnan (map[k][node])) {	/* If one is NaN they are all assumed to be NaN */
@@ -545,14 +572,14 @@ main (int argc, char **argv)
 			}
 		}
 	}
-	
+
 	for (i = 0; i < n_grids; i++) GMT_free ((void *)map[i]);
 	if (intens) GMT_free ((void *)intensity);
-	
+
 	/* Get actual size of each pixel */
 
-	dx = (r_head[0].x_max - r_head[0].x_min) / (r_head[0].nx - off);
-	dy = (r_head[0].y_max - r_head[0].y_min) / (r_head[0].ny - off);
+	dx = (r_head[0].x_max - r_head[0].x_min) / (r_head[0].nx - one_or_zero);
+	dy = (r_head[0].y_max - r_head[0].y_min) / (r_head[0].ny - one_or_zero);
 
 	/* Set lower left position of image on map */
 
@@ -562,23 +589,23 @@ main (int argc, char **argv)
 		y0 -= 0.5 * dy;
 	}
 
-	if (subset && GRID_CLIP_OK) GMT_grid_clip_on (&g_head[0], GMT_no_rgb, 3);
-
 	x_side = dx * r_head[0].nx;
 	y_side = dy * r_head[0].ny;
-	
+
 	if (gmtdefs.verbose) fprintf (stderr, "%s: Creating PostScript image ", GMT_program);
 
 	if (GMT_gray) for (k = 0, GMT_b_and_w = TRUE; GMT_b_and_w && k < nm; k++) if (!(bitimage_8[k] == 0 || bitimage_8[k] == 255)) GMT_b_and_w = FALSE;
-	
+
 	if (GMT_b_and_w) {	/* Can get away with 1 bit image */
 		int nx8, shift, byte, b_or_w, k8, nx_pixels;
 		unsigned char *bit;
-		
+
+		if (gmtdefs.verbose) fprintf (stderr, "[1-bit B/W image]\n");
+
 		nx8 = (int)ceil (r_head[0].nx / 8.0);
 		nx_pixels = nx8 * 8;
 		bit = (unsigned char *) GMT_memory (VNULL, (size_t)(nx8 * r_head[0].ny), sizeof (char), GMT_program);
-		
+
 		for (j = k = k8 = 0; j < r_head[0].ny; j++) {
 			shift = byte = 0;
 			for (i = 0; i < r_head[0].nx; i++, k++) {
@@ -604,16 +631,9 @@ main (int argc, char **argv)
 			}
 		}
 		GMT_free ((void *)bitimage_8);
-		
+
 		x_side = nx_pixels * dx;
-		if (transparent) {
-			if (gmtdefs.verbose) fprintf (stderr, "[1-bit image mask painted %d/%d/%d]\n", f_rgb[0], f_rgb[1], f_rgb[2]);
-			ps_imagemask (x0, y0, x_side, y_side, bit, nx_pixels, r_head[0].ny, polarity, f_rgb);
-		}
-		else {
-			if (gmtdefs.verbose) fprintf (stderr, "[1-bit B/W image]\n");
-			ps_colorimage (x0, y0, x_side, y_side, bit, nx_pixels, r_head[0].ny, 1 * PS_interpolate);
-		}
+		ps_bitimage (x0, y0, x_side, y_side, bit, nx_pixels, r_head[0].ny, FALSE, f_rgb, b_rgb);
 		GMT_free ((void *)bit);
 	}
 	else if (GMT_gray || monochrome) {
@@ -626,16 +646,16 @@ main (int argc, char **argv)
 		GMT_color_image (x0, y0, x_side, y_side, bitimage_24, r_head[0].nx * PS_colormask, r_head[0].ny, 24 * PS_interpolate);
 		GMT_free ((void *)bitimage_24);
 	}
-		
-	if (subset && GRID_CLIP_OK) GMT_grid_clip_off();
 
 	GMT_map_clip_off();
-	
+
 	GMT_map_basemap ();
 
 	ps_plotend (gmtdefs.last_page);
-	
+
 	GMT_end (argc, argv);
+
+	return(0);
 }
 
 void GMT_set_proj_limits (struct GRD_HEADER *r, struct GRD_HEADER *g)
@@ -654,7 +674,7 @@ void GMT_set_proj_limits (struct GRD_HEADER *r, struct GRD_HEADER *g)
 
 	all_lats = (fabs (g->y_max - 90.0) < GMT_CONV_LIMIT && fabs (g->y_min + 90.0) < GMT_CONV_LIMIT);
 	all_lons = (fabs (g->x_max - g->x_min - 360.0) < GMT_CONV_LIMIT);
-	
+
 	if (AZIMUTHAL && all_lons && all_lats) {	/* Whole globe, get square box */
 		r->x_min = project_info.xmin;	r->x_max = project_info.xmax;
 		r->y_min = project_info.ymin;	r->y_max = project_info.ymax;
@@ -662,7 +682,7 @@ void GMT_set_proj_limits (struct GRD_HEADER *r, struct GRD_HEADER *g)
 	}
 
 	/* Must search for extent along perimeter */
-	
+
 	r->x_min = r->y_min = +DBL_MAX;
 	r->x_max = r->y_max = -DBL_MAX;
 
@@ -691,12 +711,4 @@ void GMT_set_proj_limits (struct GRD_HEADER *r, struct GRD_HEADER *g)
 	if (AZIMUTHAL && all_lats) {	/* Full -90/+90, use min/max for y */
 		r->y_min = project_info.ymin;	r->y_max = project_info.ymax;
 	}
-
-	/* Then truncate, if necessary, values to fit inside plotbox */
-	
-	r->x_min = MAX (r->x_min, project_info.xmin);	r->x_max = MIN (r->x_max, project_info.xmax);
-	r->y_min = MAX (r->y_min, project_info.ymin);	r->y_max = MIN (r->y_max, project_info.ymax);
-
-	r->x_inc = (r->x_max - r->x_min) / (r->nx - r->node_offset);
-	r->y_inc = (r->y_max - r->y_min) / (r->ny - r->node_offset);
 }
