@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: grdtrend.c,v 1.11 2004/09/06 21:18:10 pwessel Exp $
+ *	$Id: grdtrend.c,v 1.14 2005/12/17 05:59:22 pwessel Exp $
  *
- *	Copyright (c) 1991-2004 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2006 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -97,10 +97,10 @@ char format[BUFSIZ];
 	void	write_model_parameters(double *gtd, int n_model);	/* Do reports if gmtdefs.verbose == TRUE  */
 	void	load_gtg_and_gtd(float *data, int nx, int ny, double *xval, double *yval, double *pstuff, double *gtg, double *gtd, int n_model, float *weight, int weighted);		/* Fill normal equations matrices  */
 
-main(int argc, char **argv)
+int main(int argc, char **argv)
 {
 	int	i, j, k, ierror = 0, iterations, nxy, n_model = 0;
-	
+
 	BOOLEAN	error = FALSE, robust = FALSE, trivial, weighted;
 
 	double	chisq, old_chisq, zero_test = 1.0e-08, scale = 1.0;
@@ -110,7 +110,7 @@ main(int argc, char **argv)
 	float	*data;		/* Pointer for array from input grdfile  */
 	float	*trend;		/* Pointer for array containing fitted surface  */
 	float	*resid;		/* Pointer for array containing residual surface  */
-	float	*weight;	/* Pointer for array containing data weights  */
+	float	*weight = VNULL;/* Pointer for array containing data weights  */
 
 	double	*xval;		/* Pointer for array of change of variable:  x[i]  */
 	double	*yval;		/* Pointer for array of change of variable:  y[j]  */
@@ -120,24 +120,24 @@ main(int argc, char **argv)
 	double	*pstuff;	/* Pointer for array for Legendre polynomials of x[i],y[j]  */
 
 	struct GRD_HEADER head_d, head_w;
-	
+
 /* Execution begins here with loop over arguments:  */
 
 	argc = GMT_begin (argc, argv);
-	
+
 	for (i = 1; i < argc; i++) {
 		if (argv[i][0] == '-') {
 			switch (argv[i][1]) {
 
 				/* Common parameters */
-			
+
 				case 'V':
 				case '\0':
 					error += GMT_get_common_args (argv[i], 0, 0, 0, 0);
 					break;
-				
+
 				/* Supplemental parameters */
-			
+
 				case 'D':
 					d_filename = &argv[i][2];
 					if (d_filename == NULL) {
@@ -182,9 +182,9 @@ main(int argc, char **argv)
 		fprintf (stderr, "grdtrend %s - Fit trend surface to gridded data\n\n", GMT_VERSION);
 		fprintf (stderr,"usage:  grdtrend <input.grd> -N[r]<n_model> [-D<diff.grd>]\n");
 		fprintf (stderr,"\t[-T<trend.grd>] [-V] [-W<weight.grd>]\n\n");
-		
+
 		if (GMT_quick) exit (EXIT_FAILURE);
-		
+
 		fprintf (stderr,"\t<input.grd> is name of grdfile to fit trend to.\n");
 		fprintf (stderr,"\t-N # model parameters to fit; integer in [1,10].  Insert r for robust fit.\n");
 		fprintf (stderr,"\n\tOPTIONS:\n");
@@ -246,7 +246,7 @@ main(int argc, char **argv)
 	gtd = (double *) GMT_memory (VNULL, (size_t)n_model, sizeof (double), GMT_program);
 	old = (double *) GMT_memory (VNULL, (size_t)n_model, sizeof (double), GMT_program);
 	pstuff = (double *) GMT_memory (VNULL, (size_t)n_model, sizeof (double), GMT_program);
-	
+
 
 /* If a weight array is needed, get one:  */
 
@@ -332,7 +332,7 @@ main(int argc, char **argv)
 /* End of do the problem section.  */
 
 /* Get here when ready to do output:  */
-	
+
 	if (gmtdefs.verbose) write_model_parameters(gtd, n_model);
 	if (t_filename) {
 		strcpy (head_d.title, "trend surface");
@@ -360,7 +360,9 @@ main(int argc, char **argv)
 	GMT_free ((void *)data);
 
 	GMT_end (argc, argv);
-}		
+
+	exit (EXIT_SUCCESS);
+}
 
 void  set_up_vals(double *val, int nval, double vmin, double vmax, double dv, int pixel_reg)
 {
@@ -531,7 +533,7 @@ void	compute_robust_weight(float *resid, float *weight, int nxy, double *scale)
 			continue;
 		}
 		r = fabs(resid[i]) / (*scale);
-		
+
 		weight[i] = (float)((r <= 1.5) ? 1.0 : (3.0 - 2.25/r) / r);
 	}
 	return;
@@ -634,7 +636,7 @@ void	load_gtg_and_gtd(float *data, int nx, int ny, double *xval, double *yval, d
 
 	return;
 }
-				
+
 void gauss (double *a, double *vec, int n, int nstore, double test, int *ierror, int itriag)
 {
  
@@ -653,9 +655,8 @@ void gauss (double *a, double *vec, int n, int nstore, double test, int *ierror,
 /*                                               on TRUE useful when solving*/
 /*                                               multiple systems with same a */
         static int isub[MAX_TABLE_COLS], l1;
-        int line[MAX_TABLE_COLS], iet, ieb, i, j, k, l, j2;
+        int line[MAX_TABLE_COLS], iet, ieb, i = 0, j, k, l, j2;
         double big, testa, b, sum;
-        
 
         iet=0;  /* initial error flags, one for triagularization*/
         ieb=0;  /* one for backsolving */
@@ -678,8 +679,7 @@ void gauss (double *a, double *vec, int n, int nstore, double test, int *ierror,
                                   /* of unused portion of matrix*/
                        for( l1=0; l1<n; l1++ ) {
                                if( line[l1]==0 ) {
-                                       testa=(double) fabs(
-                                                (double) (*(a+l1*nstore+j)) );
+                                       testa=(double) fabs((double) (*(a+l1*nstore+j)));
                                        if (testa>big) {
                                                 i=l1;
                                                 big=testa;
@@ -699,9 +699,7 @@ void gauss (double *a, double *vec, int n, int nstore, double test, int *ierror,
                                 if( line[k]==0 ) {
                                         b=(*(a+k*nstore+j))*sum;
                                         for( l=j+1; l<n; l++ ) {
-                                               *(a+k*nstore+l)=
-                                                        (*(a+k*nstore+l))
-                                                        -b*(*(a+i*nstore+l));
+                                               *(a+k*nstore+l)= (*(a+k*nstore+l)) -b*(*(a+i*nstore+l));
                                                } /*end for l*/
                                        *(a+k*nstore+j)=b;
                                         } /*end if*/
@@ -774,6 +772,3 @@ void gauss (double *a, double *vec, int n, int nstore, double test, int *ierror,
  
       *ierror = iet + ieb;   /* set final error flag*/
 }
-
-
-
