@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: grdview.c,v 1.31 2004/09/17 16:59:46 pwessel Exp $
+ *	$Id: grdview.c,v 1.39 2006/02/21 01:51:31 pwessel Exp $
  *
- *	Copyright (c) 1991-2004 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2006 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -38,6 +38,7 @@
  */
  
 #include "gmt.h"
+#include "pslib.h"
 
 /* Declarations needed for binning of smooth contours */
 
@@ -82,58 +83,58 @@ struct CONT *get_cont_struct (int bin, double value);
 struct POINT *get_point (double x, double y);
 BOOLEAN monochrome = FALSE;
 
-main (int argc, char **argv)
+int main (int argc, char **argv)
 {
 	int 	i, j, ij, n_edges, k, k1, n, max, i_bin, j_bin, i_bin_old, j_bin_old;
-	int	side, way, nm, nm2, nx_f, ny_f, off, nx, ny, n_drape = 0, nm_use;
+	int	side, way, nm, nm2, nx_f, ny_f, one_of_zero, nx, ny, n_drape = 0, nm_use;
 	int	rgb_facade[3], bin, two, mx, my, sw, se, nw, ne, id, n4, nk;
 	int	c, rgb[3], i_start, i_stop, j_start, j_stop, i_inc, j_inc;
 	int	smooth = 1, tiling = 0, dpi_i = 100, q_set = 0, n_commas;
-	int	PS_colormask_off = 0, PS_colormask = 1;
-	
+	int	PS_colormask_off = 0, PS_colormask = 1, ii, jj, n_out;
+
 	BOOLEAN mesh = FALSE, draw_plane = FALSE, facade = FALSE, get_contours, bad, set_z = FALSE;
 	BOOLEAN error = FALSE, outline = FALSE, surface = FALSE, pen_not_set, bilinear = FALSE;
 	BOOLEAN first, begin, draw_contours = FALSE, intens = FALSE, drape = FALSE, tile_outline = FALSE;
 	BOOLEAN saddle, image = FALSE, subset = FALSE, no_nans, do_rgb = FALSE, drape_resample = FALSE;
-	
-	double cval, x_left, x_right, y_top, y_bottom, small, z_ave, this_intensity, *xval, *yval;
+
+	double cval, x_left, x_right, y_top, y_bottom, small = SMALL, z_ave, this_intensity = 0.0, *xval, *yval;
 	double plane_level = 0.0, dx2, dy2, take_out, threshold = 1.0;
-	double west = 0.0, east = 0.0, south = 0.0, north = 0.0, new_z_level = 0.0, del_off;
-	double data_west, data_east, data_south, data_north, delx, dely, z_val, next_up, xmesh[4], ymesh[4];
+	double west = 0.0, east = 0.0, south = 0.0, north = 0.0, new_z_level = 0.0;
+	double data_west, data_east, data_south, data_north, delx, dely, z_val, next_up = 0.0, xmesh[4], ymesh[4];
 	double x_pixel_size, y_pixel_size, *x_imask, *y_imask;
-		
-	char *topofile, *intensfile, drapefile[3][256], *cpt_file;
-	
+
+	char *topofile, *intensfile, drapefile[3][GMT_LONG_TEXT], *cpt_file;
+
 	struct CONT *start_cont, *this_cont, *last_cont;
-	struct POINT *this_point, *last_point;
-	
+	struct POINT *this_point = VNULL, *last_point;
+
 	struct GRD_HEADER header, t_head, i_head, d_head[3];
-	
+
 	struct GMT_PEN pen[2], tile_pen;
 	struct GMT_FILL fill;
 
         struct GMT_EDGEINFO edgeinfo;
-	
+
 	struct GMT_BCR t_bcr, i_bcr;
 
 	for (i = 0; i < 3; i++) rgb_facade[i] = drapefile[i][0] = 0;
-	
+
 	cpt_file = topofile = intensfile = CNULL;
-	
+
 	argc = GMT_begin (argc, argv);
-	
+
 	GMT_3D_mode = 1;	/* Only do background axis first; do foreground at end */
 	GMT_init_pen (&pen[0], 3.0 * GMT_PENWIDTH);	/* Contour pen */
 	GMT_init_pen (&pen[1], GMT_PENWIDTH);	/* Mesh pen */
 	GMT_init_pen (&tile_pen, GMT_PENWIDTH);
 	GMT_init_fill (&fill, 255 - gmtdefs.basemap_frame_rgb[0], 255 - gmtdefs.basemap_frame_rgb[0], 255 - gmtdefs.basemap_frame_rgb[0]);
-	
+
 	for (i = 1; i < argc; i++) {
 		if (argv[i][0] == '-') {
 			switch (argv[i][1]) {
-		
+
 				/* Common parameters */
-			
+
 				case 'B':
 				case 'J':
 				case 'K':
@@ -150,9 +151,9 @@ main (int argc, char **argv)
 				case '\0':
 					error += GMT_get_common_args (argv[i], &west, &east, &south, &north);
 					break;
-				
+
 				/* Supplemental parameters */
-			
+
 				case 'C':
 					cpt_file = &argv[i][2];
 					break;
@@ -188,7 +189,7 @@ main (int argc, char **argv)
 					break;
 				case 'N':
 					if (argv[i][2]) {
-						char colors[64];
+						char colors[GMT_TEXT_LEN];
 						draw_plane = TRUE;
 						n = sscanf (&argv[i][2], "%lf/%s", &plane_level, colors);
 						if (n == 2) {
@@ -276,9 +277,9 @@ main (int argc, char **argv)
 						set_z = TRUE;
 					}
 					break;
-					
+
 				/* Illegal options */
-			
+
 				default:
 					error = TRUE;
 					GMT_default_error (argv[i][1]);
@@ -290,16 +291,16 @@ main (int argc, char **argv)
 	}
 
 	if (!(mesh || surface || image || tiling)) mesh = TRUE;	/* Default is mesh plot */
-	
+
 	if (argc == 1 || GMT_quick) {
 		fprintf (stderr, "grdview %s - Plot topofiles in 3-D\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: grdview <topofile> -J<params> [-B<tickinfo>] [-C<cpt_file>] [-E<v_az/v_el>]\n");
 		fprintf (stderr, "\t[-G<drapefile> | -G<grd_r>,<grd_g>,<grd_b>] [-I<intensfile>] [-K] [-L[<flags>]] [-N<level>[/<r/g/b>]] [-O] [-P]\n");
 		fprintf (stderr, "\t[-Q<type>[g]] [-R<region>] [-S<smooth>] [-T[s][o[<pen>]]] [-U] [-V] [-W<type><pen>]\n");
 		fprintf (stderr, "\t[-X<x_shift>] [-Y<y_shift>] [-Z<z_level>] [-c<ncopies>]\n\n");
-		
+
 		if (GMT_quick) exit (EXIT_FAILURE);
-		
+
 		fprintf (stderr, "\t<topofile> is data set to be plotted\n");
 		GMT_explain_option ('j');
 		fprintf (stderr, "\n\tOPTIONS:\n");
@@ -351,15 +352,15 @@ main (int argc, char **argv)
 		GMT_explain_option ('.');
 		exit (EXIT_FAILURE);
 	}
-	
+
 	if (q_set > 1) {	/* Gave more than one -Q setting */
 		fprintf (stderr, "%s: GMT ERROR:  -Qm, -Qs, -Qc, and -Qi are mutually exclusive options\n", GMT_program);
 		error++;
-	}	
+	}
 	if (tiling && q_set) {	/* Gave both -Q and -T */
 		fprintf (stderr, "%s: GMT ERROR:  -Q and -T are mutually exclusive options\n", GMT_program);
 		error++;
-	}	
+	}
 	if (!topofile) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR:  Must specify input file\n", GMT_program);
 		error++;
@@ -399,16 +400,16 @@ main (int argc, char **argv)
 	if (error) exit (EXIT_FAILURE);
 
 	GMT_put_history (argc, argv);	/* Update .gmtcommands4 */
-	
+
 	if (cpt_file) {
 		GMT_read_cpt (cpt_file);
 		if (GMT_b_and_w) monochrome = TRUE;
 	}
-	
+
 	get_contours = ( (mesh && draw_contours) || (surface && GMT_n_colors > 1) );
-	
+
 	two = (drape && get_contours) ? 2 : 0;	/* Must read topofile with 2 boundary columns/rows */
-	
+
 	if (!strcmp (topofile, "=")) {
 		fprintf (stderr, "%s: Piping of topofile not supported!\n", GMT_program);
 		exit (EXIT_FAILURE);
@@ -418,12 +419,12 @@ main (int argc, char **argv)
 		fprintf (stderr, "%s: Error opening file %s\n", GMT_program, topofile);
 		exit (EXIT_FAILURE);
 	}
-	
+
 	if (intens && GMT_read_grd_info (intensfile, &i_head)) {
 		fprintf (stderr, "%s: Error opening file %s\n", GMT_program, intensfile);
 		exit (EXIT_FAILURE);
 	}
-	
+
 	if (drape) {
 		for (i = 0; i < n_drape; i++) {
 			if (GMT_read_grd_info (drapefile[i], &d_head[i])) {
@@ -433,12 +434,11 @@ main (int argc, char **argv)
 		}
 		two = 2; 
 	}
-	
-	off = (t_head.node_offset) ? 0 : 1;
-	del_off = (t_head.node_offset) ? 0.5 : 0.0;
-	
+
+	one_of_zero = (t_head.node_offset) ? 0 : 1;
+
 	GMT_boundcond_init (&edgeinfo);
-	
+
 	/* Determine what wesn to pass to map_setup */
 
 	if (!project_info.region_supplied) {
@@ -456,7 +456,7 @@ main (int argc, char **argv)
 		if (draw_plane && plane_level < project_info.z_bottom) project_info.z_bottom = plane_level;
 		if (draw_plane && plane_level > project_info.z_top) project_info.z_top = plane_level;
 	}
-	
+
 	GMT_map_setup (west, east, south, north);
 
 	/* Determine the wesn to be used to read the grdfile */
@@ -481,10 +481,18 @@ main (int argc, char **argv)
 
 	/* Read data */
 
+	if (subset) {	/* Possibly extend the subset by 1 row/col since tiles need 4 corners to work */
+		if (GMT_x_to_i (data_west, t_head.x_min, t_head.x_inc, t_head.xy_off, t_head.nx) > 0) data_west -= t_head.x_inc;
+		if (GMT_x_to_i (data_east, t_head.x_min, t_head.x_inc, t_head.xy_off, t_head.nx) < (t_head.nx)) data_east += t_head.x_inc;
+		if (data_east > t_head.x_max) data_east -= t_head.x_inc;
+		if (GMT_y_to_j (data_south, t_head.y_min, t_head.y_inc, t_head.xy_off, t_head.ny) < (t_head.ny)) data_south -= t_head.y_inc;
+		if (GMT_y_to_j (data_north, t_head.y_min, t_head.y_inc, t_head.xy_off, t_head.ny) > 0) data_north += t_head.y_inc;
+		if (data_south < t_head.y_min) data_south += t_head.y_inc;
+	}
 	if (gmtdefs.verbose) fprintf (stderr, "%s: Processing shape file\n", GMT_program);
-	
-	nx = irint ( (data_east - data_west) / t_head.x_inc) + off;
-	ny = irint ( (data_north - data_south) / t_head.y_inc) + off;
+
+	nx = irint ( (data_east - data_west) / t_head.x_inc) + one_of_zero;
+	ny = irint ( (data_north - data_south) / t_head.y_inc) + one_of_zero;
 	mx = nx + 2 * two;
 	my = ny + 2 * two;
 	nm = nx * ny;
@@ -502,19 +510,19 @@ main (int argc, char **argv)
 
 	delx = (t_head.node_offset) ? 0.5 * t_head.x_inc :0.0;
 	dely = (t_head.node_offset) ? 0.5 * t_head.y_inc :0.0;
-	
+
 	xval = (double *) GMT_memory (VNULL, (size_t)t_head.nx, sizeof (double), GMT_program);
 	yval = (double *) GMT_memory (VNULL, (size_t)t_head.ny, sizeof (double), GMT_program);
 	for (i = 0; i < t_head.nx; i++) xval[i] = t_head.x_min + i * t_head.x_inc + delx;
 	for (j = 0; j < t_head.ny; j++) yval[j] = t_head.y_max - j * t_head.y_inc - dely;
-		
+
 	GMT_boundcond_param_prep (&t_head, &edgeinfo);
 
 	if (drape) {	/* draping wanted */
-	
+
 		for (i = 0; i < n_drape; i++) {
 			if (gmtdefs.verbose) fprintf (stderr, "%s: Processing drape file %s\n", GMT_program, drapefile[i]);
-	
+
 			grd[i] = (float *) GMT_memory (VNULL, (size_t)(d_head[i].nx*d_head[i].ny), sizeof (float), GMT_program);
 			if (GMT_read_grd (drapefile[i], &d_head[i], grd[i], data_west, data_east, data_south, data_north, GMT_pad, FALSE)) {
 				fprintf (stderr, "%s: Error reading file %s\n", GMT_program, drapefile[i]);
@@ -533,7 +541,7 @@ main (int argc, char **argv)
 		zgrd = topo;
 		header = t_head;
 	}
-	
+
 	if (!project_info.xyz_pos[2])	/* Negative z-scale, must flip */
 		d_swap (project_info.z_bottom, project_info.z_top);
 
@@ -542,9 +550,9 @@ main (int argc, char **argv)
 	ne = nw + t_head.nx - 1;
 	sw = (t_head.ny + two - 1) * mx + two;
 	se = sw + t_head.nx - 1;
-	
+
 	grdview_init_setup (&t_head, topo, two, draw_plane, plane_level);	/* Find projected min/max in y-direction */
-			
+
 	i_start = (z_project.quadrant == 1 || z_project.quadrant == 2) ? 0 : header.nx - 2;
 	i_stop  = (z_project.quadrant == 1 || z_project.quadrant == 2) ? header.nx - 1 : -1;
 	i_inc   = (z_project.quadrant == 1 || z_project.quadrant == 2) ? 1 : -1;
@@ -564,9 +572,9 @@ main (int argc, char **argv)
 		if (gmtdefs.verbose) fprintf (stderr, "%s: Trace and bin contours...\n", GMT_program);
 		first = TRUE;
 		for (c = 0; c < GMT_n_colors+1; c++) {	/* For each color change */
-		
+
 			/* Reset markers and set up new zero-contour*/
-		
+
 			cval = (c == GMT_n_colors) ? GMT_lut[c-1].z_high : GMT_lut[c].z_low;
 
 			if (cval < header.z_min || cval > header.z_max) continue;
@@ -578,18 +586,18 @@ main (int argc, char **argv)
 				if (!GMT_is_fnan (zgrd[i])) zgrd[i] -= (float)take_out;
 				if (zgrd[i] == 0.0) zgrd[i] += (float)small;
 			}
-		
+
 			side = 0;
 			begin = TRUE;
 			while (side < 5) {
 				while ((n = GMT_contours (zgrd, &header, smooth, gmtdefs.interpolant, &side, edge, begin, &x, &y)) > 0) {
-		
+
 					begin = FALSE;
-		
+
 					i_bin_old = j_bin_old = -1;
 					for (i = 1; i < n; i++) {
-						i_bin = (int)floor (((0.5 * (x[i-1] + x[i]) - header.x_min) / header.x_inc) - del_off);
-						j_bin = (int)floor (((header.y_max - 0.5 * (y[i-1] + y[i])) / header.y_inc) - del_off) + 1;
+						i_bin = (int)floor (((0.5 * (x[i-1] + x[i]) - header.x_min) / header.x_inc) - header.xy_off);
+						j_bin = (int)floor (((header.y_max - 0.5 * (y[i-1] + y[i])) / header.y_inc) - header.xy_off) + 1;
 						if (i_bin != i_bin_old || j_bin != j_bin_old) {	/* Entering new bin */
 							bin = j_bin * header.nx + i_bin;
 							this_cont = get_cont_struct (bin, cval);
@@ -608,13 +616,13 @@ main (int argc, char **argv)
 				begin = FALSE;
 			}
 		}
-	
+
 		/* Remove temporary variables */
-		
+
 		GMT_free ((void *)edge);
-		
+
 		/* Go back to beginning and reread since grd has been destroyed */
-		
+
 		if (drape) {
 			if (GMT_read_grd_info (drapefile[0], &d_head[0])) {
 				fprintf (stderr, "%s: Error opening file %s\n", GMT_program, drapefile[0]);
@@ -641,11 +649,11 @@ main (int argc, char **argv)
 		}
 		if (gmtdefs.verbose) fprintf (stderr, "\n");
 	}
-	
+
 	if (intens) {	/* Illumination wanted */
-	
+
 		if (gmtdefs.verbose) fprintf (stderr, "%s: Processing illumination file\n", GMT_program);
-	
+
 		nm_use = (drape_resample) ? nm2 : nm;
 		intensity = (float *) GMT_memory (VNULL, (size_t)nm_use, sizeof (float), GMT_program);
 		k = (drape_resample) ? two : 0;
@@ -661,7 +669,7 @@ main (int argc, char **argv)
 		if (drape_resample) GMT_bcr_init (&i_head, GMT_pad, bilinear, threshold, &i_bcr);
 		GMT_pad[0] = GMT_pad[1] = GMT_pad[2] = GMT_pad[3] = 0;
 	}
-	
+
 	if (two) {	/* Initialize bcr stuff */
 		GMT_pad[0] = GMT_pad[1] = GMT_pad[2] = GMT_pad[3] = two;
 		GMT_bcr_init (&t_head, GMT_pad, bilinear, threshold, &t_bcr);
@@ -673,29 +681,29 @@ main (int argc, char **argv)
 	}
 
 	dx2 = 0.5 * header.x_inc;		dy2 = 0.5 * header.y_inc;
-			
+
 	no_nans = TRUE;
-	
-	/* Temporarily modify the outer edge coordinates */
-	xval[0] += dx2;	xval[t_head.nx-1] -= dx2;
-	yval[0] -= dy2;	yval[t_head.ny-1] += dy2;
-	for (j = bin = ij = 0; j < t_head.ny; j++) {	/* Nodes outside -R is set to NaN and not used */
-		ij = (two) ? (j + 2) * mx + two : bin;
-		for (i = 0; i < t_head.nx; i++, bin++, ij++) {
-			if (GMT_is_fnan (topo[ij])) no_nans = FALSE;
-			if (GMT_map_outside (xval[i], yval[j])) topo[ij] = GMT_f_NaN;
+
+	for (j = 0; j < t_head.ny - 1; j++) {	/* Nodes part of tiles completely outside -R is set to NaN and thus not considered below */
+		for (i = 0; i < t_head.nx - 1; i++, bin++, ij++) {
+			for (jj = n_out = 0; jj < 2; jj++) {	/* Loop over the 4 nodes making up one tile */
+				for (ii = 0; ii < 2; ii++) {
+					bin = (j + jj) * t_head.nx + (i + ii);
+					ij = (two) ? ((j + jj) + 2) * mx + (i + ii) + two : bin;
+					if (GMT_is_fnan (topo[ij])) no_nans = FALSE;
+					if (GMT_map_outside (xval[i+ii], yval[j+jj])) n_out++;
+				}
+			}
+			if (n_out == 4) topo[ij] = topo[ij-1] = topo[ij-t_head.nx] = topo[ij-t_head.nx-1] = GMT_f_NaN;	/* Entire tile is outside */
 		}
 	}
-	/* Reset the outer edge coordinates */
-	xval[0] -= dx2;	xval[t_head.nx-1] += dx2;
-	yval[0] += dy2;	yval[t_head.ny-1] -= dy2;
-	
+
 	max = 2 * (MAX(1,smooth) * (((header.nx > header.ny) ? header.nx : header.ny) + 2)) + 1;
 	x = (double *) GMT_memory (VNULL, (size_t)max, sizeof (double), GMT_program);
 	y = (double *) GMT_memory (VNULL, (size_t)max, sizeof (double), GMT_program);
 	z = (double *) GMT_memory (VNULL, (size_t)max, sizeof (double), GMT_program);
 	v = (double *) GMT_memory (VNULL, (size_t)max, sizeof (double), GMT_program);
-	
+
 	if (gmtdefs.verbose) fprintf (stderr, "%s: Start creating PostScript plot\n", GMT_program);
 
 	ps_plotinit (CNULL, gmtdefs.overlay, gmtdefs.page_orientation, gmtdefs.x_origin, gmtdefs.y_origin,
@@ -713,7 +721,7 @@ main (int argc, char **argv)
 	if (project_info.z_pars[0] == 0.0) GMT_map_clip_on (GMT_no_rgb, 3);
 
 	if (set_z) project_info.z_level = new_z_level;
-	
+
 	xx = (double *) GMT_memory (VNULL, (size_t)max, sizeof(double), GMT_program);
 	yy = (double *) GMT_memory (VNULL, (size_t)max, sizeof(double), GMT_program);
 	if (draw_plane) {
@@ -762,7 +770,7 @@ main (int argc, char **argv)
 				ps_line (xx, yy, header.ny, 3, TRUE, TRUE);
 			}
 		}
-		
+
 		if (project_info.region) {
 			GMT_geoz_to_xy (header.x_min + delx, header.y_min + dely, plane_level, &xx[0], &yy[0]);
 			GMT_geoz_to_xy (header.x_max - delx, header.y_min + dely, plane_level, &xx[1], &yy[1]);
@@ -786,26 +794,24 @@ main (int argc, char **argv)
 			}
 		}
 	}
-	
+
 	if (tiling) {	/* Plot image as polygonal pieces */
 		int p;
-		double *xx, *yy, d, xpos, ypos;
-		
-		d = (off) ? 0.0 : 0.5;
-		
+		double *xx, *yy, xpos, ypos;
+
 		if (gmtdefs.verbose) fprintf (stderr, "%s: Tiling without interpolation\n", GMT_program);
-		
+
 		if (tile_outline) GMT_setpen (&tile_pen);
 		for (j = k = 0; j < header.ny; j++) {
 			for (i = 0; i < header.nx; i++, k++) {	/* Compute rgb for each pixel */
 				if (GMT_is_fnan (topo[k]) && tiling == 2) continue;
 				GMT_get_rgb24 (topo[k], rgb);
 				if (intens) GMT_illuminate (intensity[k], rgb);
-				
+
 				n = GMT_graticule_path (&xx, &yy, 1, xval[i] - dx2, xval[i] + dx2, yval[j] - dy2, yval[j] + dy2);
-				
+
 				/* Do map projection and plot */
-				
+
 				for (p = 0; p < n; p++) {
 					GMT_geoz_to_xy (xx[p], yy[p], topo[k], &xpos, &ypos);
 					xx[p] = xpos;	yy[p] = ypos;
@@ -818,33 +824,34 @@ main (int argc, char **argv)
 	}
 	if (image) {	/* compute image */
 		int nx_i, ny_i, kk, ip, jp, min_i, max_i, min_j, max_j, dist, node, nm_i, nm_drape, layers, *ix, *iy;
-		int *top_jp, *bottom_jp, p;
+		int *top_jp = VNULL, *bottom_jp = VNULL, p;
 		BOOLEAN done;
-		double xp, yp, sum_w, w, sum_i, x_width, y_width, value;
-		double sum_r, sum_g, sum_b, intval, y_drape, *x_drape;
-		float *int_drape;
-		unsigned char *bitimage_24, *bitimage_8;
-		
+		double xp, yp, sum_w, w, sum_i, x_width, y_width, value, i_dpi_i;
+		double sum_r, sum_g, sum_b, intval = 0.0, y_drape, *x_drape;
+		float *int_drape = VNULL;
+		unsigned char *bitimage_24 = (unsigned char *)NULL, *bitimage_8 = (unsigned char *)NULL;
+
 		if (gmtdefs.verbose) {
 			if (GMT_cpt_pattern) fprintf (stderr, "%s: Warning: Patterns in cpt file will not work with -Qi\n", GMT_program);
 			fprintf (stderr, "%s: get and store projected vertices\n", GMT_program);
 		}
-		
+
 		ps_comment ("Plot 3-D surface using scanline conversion of polygons to raster image");
 
 		x_width = z_project.xmax - z_project.xmin;	/* Size of image in inches */
 		y_width = z_project.ymax - z_project.ymin;
 		nx_i = irint (x_width * dpi_i);	/* Size of image in pixels */
 		ny_i = irint (y_width * dpi_i);
+		i_dpi_i = 1.0 / dpi_i;
 		if (drape_resample) {
 			nm_drape = d_head[0].nx * d_head[0].ny;
 			ix = (int *) GMT_memory (VNULL, (size_t)nm_drape, sizeof (int), GMT_program);
 			iy = (int *) GMT_memory (VNULL, (size_t)nm_drape, sizeof (int), GMT_program);
 			x_drape = (double *) GMT_memory (VNULL, (size_t)d_head[0].nx, sizeof (double), GMT_program);
 			if (intens) int_drape = (float *) GMT_memory (VNULL, (size_t)nm_drape, sizeof (float), GMT_program);
-			for (i = 0; i < d_head[0].nx; i++) x_drape[i] = GMT_i_to_x (i, d_head[0].x_min, d_head[0].x_max,d_head[0].x_inc, del_off, d_head[0].nx);
+			for (i = 0; i < d_head[0].nx; i++) x_drape[i] = GMT_i_to_x (i, d_head[0].x_min, d_head[0].x_max, d_head[0].x_inc, d_head[0].xy_off, d_head[0].nx);
 			for (j = bin = 0; j < d_head[0].ny; j++) {	/* Get projected coordinates converted to pixel locations */
-				y_drape = GMT_j_to_y (j, d_head[0].y_min, d_head[0].y_max,d_head[0].y_inc, del_off, d_head[0].ny);
+				y_drape = GMT_j_to_y (j, d_head[0].y_min, d_head[0].y_max,d_head[0].y_inc, d_head[0].xy_off, d_head[0].ny);
 				for (i = 0; i < d_head[0].nx; i++, bin++) {
 					value = GMT_get_bcr_z(&t_head, x_drape[i], y_drape, topo, &edgeinfo, &t_bcr);
 					if (GMT_is_fnan (value)) {	/* Outside -R or NaNs not used */
@@ -883,12 +890,12 @@ main (int argc, char **argv)
 		}
 		GMT_free ((void *) xval);
 		GMT_free ((void *) yval);
-		
+
 		/* Allocate image array and set background to PAGE_COLOR */
-		
+
 		if (monochrome) {
 			int gray;
-			
+
 			nm_i = nx_i * ny_i;
 			layers = 1;
 			bitimage_8 = (unsigned char *) GMT_memory (VNULL, (size_t)nm_i, sizeof (char), GMT_program);
@@ -917,9 +924,9 @@ main (int argc, char **argv)
 			bottom_jp = (int *) GMT_memory (VNULL, (size_t)nx_i, sizeof (int), GMT_program);
 			for (ip = 0; ip < nx_i; ip++) bottom_jp[ip] = ny_i;
 		}
-			
+
 		/* Plot from back to front */
-		
+
 		if (gmtdefs.verbose) fprintf (stderr, "%s: Start rasterization\n", GMT_program);
 		for (j = j_start; j != j_stop; j += j_inc) {
 
@@ -945,13 +952,12 @@ main (int argc, char **argv)
 					for (ip = min_i; ip <= max_i; ip++) {
 						if (ip < 0 || ip >= nx_i) continue;
 						if (!pixel_inside (ip, jp, ix, iy, bin)) continue;
-						
+
 						if (no_nans) {	/* Update clip mask */
 							if (jp > top_jp[ip]) top_jp[ip] = jp; 
 							if (jp < bottom_jp[ip]) bottom_jp[ip] = jp;
 						}
-						
-						kk = layers * ((ny_i-jp-1) * nx_i + ip) + PS_colormask_off;
+
 						sum_r = sum_g = sum_b = sum_w = sum_i = 0.0;
 						done = FALSE;
 						for (k = 0; !done && k < 4; k++) {
@@ -985,6 +991,7 @@ main (int argc, char **argv)
 							if (intens) intval = sum_i * sum_w;
 						}
 						if (intens) GMT_illuminate (intval, rgb);
+						kk = layers * ((ny_i-jp-1) * nx_i + ip) + PS_colormask_off;
 						if (monochrome) /* YIQ transformation */
 							bitimage_8[kk] = (unsigned char) YIQ (rgb);
 						else {
@@ -999,14 +1006,14 @@ main (int argc, char **argv)
 		if (gmtdefs.verbose) fprintf (stderr, "\n");
 
 		if (no_nans) {	/* Must implement the clip path for the image */
-			
+
 			x_pixel_size = x_width / (double)nx_i;
 			y_pixel_size = y_width / (double)ny_i;
 			n4 = 4 * nx_i;
 			x_imask = (double *) GMT_memory (VNULL, (size_t)n4, sizeof (double), GMT_program);
 			y_imask = (double *) GMT_memory (VNULL, (size_t)n4, sizeof (double), GMT_program);
 			nk = n4 - 1;
-		
+
 			for (ip = k = 0; ip < nx_i; ip++, k+= 2) {
 				k1 = k + 1;
 				x_imask[k]  = x_imask[nk-k]  = z_project.xmin + ip * x_pixel_size;
@@ -1023,7 +1030,7 @@ main (int argc, char **argv)
 			GMT_free ((void *)x_imask);
 			GMT_free ((void *)y_imask);
 		}
-		
+
 		if (gmtdefs.verbose) fprintf (stderr, "%s: Creating PostScript image ", GMT_program);
 		if (monochrome) {
 			if (gmtdefs.verbose) fprintf (stderr, "[B/W image]\n");
@@ -1035,13 +1042,13 @@ main (int argc, char **argv)
 			GMT_color_image (z_project.xmin, z_project.ymin, x_width, y_width, bitimage_24, nx_i * PS_colormask, ny_i, 24);
 			GMT_free ((void *)bitimage_24);
 		}
-		
+
 		if (no_nans){
 			ps_clipoff ();
 			GMT_free ((void *)top_jp);
 			GMT_free ((void *)bottom_jp);
 		}
-		
+
 		GMT_free ((void *)ix);
 		GMT_free ((void *)iy);
 	}
@@ -1082,7 +1089,7 @@ main (int argc, char **argv)
 						ps_line (xx, yy, k, 3, FALSE, TRUE);
 					}
 					if (!pen_not_set) GMT_setpen (&pen[1]);
-				}					
+				}
 			}
 		}
 		GMT_free ((void *) xval);
@@ -1092,15 +1099,15 @@ main (int argc, char **argv)
 		int start_side, entry_side, exit_side, next_side, low, ncont, nw_se_diagonal, check;
 		int corner[2], bad_side[2][2], p, p1, p2, saddle_sign;
 		double *xcont, *ycont, *zcont, *vcont, X_vert[4], Y_vert[4], saddle_small;
-		
+
 		xcont = (double *) GMT_memory (VNULL, (size_t)max, sizeof (double), GMT_program);
 		ycont = (double *) GMT_memory (VNULL, (size_t)max, sizeof (double), GMT_program);
 		zcont = (double *) GMT_memory (VNULL, (size_t)max, sizeof (double), GMT_program);
 		vcont = (double *) GMT_memory (VNULL, (size_t)max, sizeof (double), GMT_program);
-		
+
 		ps_comment ("Start of filled surface");
 		if (outline) GMT_setpen (&pen[1]);
-		
+
 		for (j = j_start; j != j_stop; j += j_inc) {
 			y_bottom = yval[j];
 			y_top = y_bottom + header.y_inc;
@@ -1112,7 +1119,7 @@ main (int argc, char **argv)
 				for (k = bad = 0; !bad && k < 4; k++) bad += GMT_is_fnan (topo[ij+ij_inc[k]]);
 				if (bad) {
 					if (GMT_bfn[GMT_NAN].skip || project_info.three_D) continue;
-					
+
 					X_vert[0] = X_vert[3] = x_left;	X_vert[1] = X_vert[2] = x_right;
 					Y_vert[0] = Y_vert[1] = y_bottom;	Y_vert[2] = Y_vert[3] = y_top;
 					for (k = 0; k < 4; k++) GMT_geoz_to_xy (X_vert[k], Y_vert[k], 0.0, &xmesh[k], &ymesh[k]);
@@ -1120,7 +1127,7 @@ main (int argc, char **argv)
 					if (outline) ps_patch (xmesh, ymesh, 4, GMT_no_rgb, TRUE);
 					continue;
 				}
-				
+
 				if (intens) {
 					this_intensity = get_intensity (intensity, bin, header.nx);
 					if (GMT_is_dnan (this_intensity)) continue;
@@ -1132,9 +1139,9 @@ main (int argc, char **argv)
 				Y_vert[0] = Y_vert[1] = y_bottom;	Y_vert[2] = Y_vert[3] = y_top;
 
 				if (get_contours && binij[bin].first_cont) {	/* Contours go thru here */
-				
+
 					/* Determine if this bin will give us saddle trouble */
-					
+
 					start_cont = this_cont = binij[bin].first_cont->next_cont;
 					saddle = FALSE;
 					while (!saddle && this_cont->next_cont) {
@@ -1144,13 +1151,13 @@ main (int argc, char **argv)
 							this_cont = this_cont->next_cont;
 					}
 					if (saddle) {	/* Must deal with this separately */
-					
+
 						this_point = this_cont->first_point;
 						entry_side = get_side (this_point->x, this_point->y, x_left, y_bottom, header.x_inc, header.y_inc, dx2, dy2);
 						while (this_point->next_point) this_point = this_point->next_point;	/* Go to end */
 						exit_side  = get_side (this_point->x, this_point->y, x_left, y_bottom, header.x_inc, header.y_inc, dx2, dy2);
-						
-						
+
+
 						if (MIN (zgrd[bin+bin_inc[1]], zgrd[bin+bin_inc[3]]) > MAX (zgrd[bin], zgrd[bin+bin_inc[2]])) {
 							saddle_sign = +1;
 							check = TRUE;
@@ -1178,21 +1185,21 @@ main (int argc, char **argv)
 							corner[1] = 3;	bad_side[1][0] = 0;	bad_side[1][1] = 1;
 						}
 						saddle_small = saddle_sign * small;
-						
+
 						for (p = 0; p < 2; p++) {	/* For each triangular half */
-						
+
 							/* Set this points as the start anchor */
-							
+
 							low = corner[p];
 							n = 0;
 							add_node (x, y, z, v, &n, low, X_vert, Y_vert, topo, zgrd, ij+ij_inc[low], bin+bin_inc[low]);
 							start_side = next_side = low;
 							way = 0;
-					
+
 							for (this_cont = start_cont; this_cont; this_cont = this_cont->next_cont) {
-					
+
 								/* First get all the x/y pairs for this contour */
-						
+
 								for (k = 0, this_point = this_cont->first_point; this_point; this_point = this_point->next_point) {
 									xcont[k] = this_point->x;
 									ycont[k] = this_point->y;
@@ -1202,19 +1209,19 @@ main (int argc, char **argv)
 									k++;
 								}
 								ncont = k;
-								
+
 								entry_side = get_side (xcont[0], ycont[0], x_left, y_bottom, header.x_inc, header.y_inc, dx2, dy2);
 								exit_side  = get_side (xcont[ncont-1], ycont[ncont-1], x_left, y_bottom, header.x_inc, header.y_inc, dx2, dy2);
 
 								if (entry_side == bad_side[p][0] || entry_side == bad_side[p][1]) continue;
 								if (exit_side == bad_side[p][0] || exit_side == bad_side[p][1]) continue;
-								
+
 								/* OK, got the correct contour */
-								
+
 								next_up = (this_cont->next_cont) ? this_cont->next_cont->value : DBL_MAX;
-						
+
 								exit_side  = get_side (xcont[ncont-1], ycont[ncont-1], x_left, y_bottom, header.x_inc, header.y_inc, dx2, dy2);
-						
+
 								if (way == 0 || next_side == entry_side) {	/* Just hook up */
 									copy_points_fw (x, y, z, v, xcont, ycont, zcont, vcont, ncont, &n);
 									next_side = exit_side;
@@ -1224,25 +1231,25 @@ main (int argc, char **argv)
 									next_side = entry_side;
 								}
 								/* Compute the xy from the xyz triplets */
-						
+
 								for (k = 0; k < n; k++) GMT_geoz_to_xy (x[k], y[k], z[k], &xx[k], &yy[k]);
 								z_ave = (GMT_continuous) ? get_z_ave (v, next_up, n) : this_cont->value;
-						
+
 								/* Now paint the polygon piece */
-						
+
 								paint_it (xx, yy, n, z_ave-saddle_small, intens, this_intensity);
-						
+
 								/* Reset the anchor points to previous contour */
-						
+
 								n = 0;
 								copy_points_fw (x, y, z, v, xcont, ycont, zcont, vcont, ncont, &n);
 								next_side = exit_side;
 								start_side = entry_side;
 								way = (zgrd[bin+bin_inc[low]] < this_cont->value) ? -1 : 1;
 							}
-							
+
 							/* Final contour needs to add diagonal */
-					
+
 							if (corner[p] == 0 || corner[p] == 2) {
 								p1 = (next_side < 2) ? 1 : 3;
 								p2 = (next_side < 2) ? 3 : 1;
@@ -1253,40 +1260,40 @@ main (int argc, char **argv)
 							}
 							add_node (x, y, z, v, &n, p1, X_vert, Y_vert, topo, zgrd, ij+ij_inc[p1], bin+bin_inc[p1]);
 							add_node (x, y, z, v, &n, p2, X_vert, Y_vert, topo, zgrd, ij+ij_inc[p2], bin+bin_inc[p2]);
-							
+
 							/* Compute the xy from the xyz triplets */
-						
+
 							for (k = 0; k < n; k++) GMT_geoz_to_xy (x[k], y[k], z[k], &xx[k], &yy[k]);
-						
+
 							z_ave = (GMT_continuous) ? get_z_ave (v, next_up, n) : v[0];
-						
+
 							/* Now paint the polygon piece */
-						
+
 							paint_it (xx, yy, n, z_ave+saddle_small, intens, this_intensity);
-						
+
 						} /* End triangular piece */
 
 					} /* End Saddle section */
 					else {
 						/* Ok, here we do not have to worry about saddles */
-					
+
 						/* Find lowest corner (id = low) */
-					
+
 						for (k = 1, low = 0; k < 4; k++) if (zgrd[bin+bin_inc[k]] < zgrd[bin+bin_inc[low]]) low = k;
 
 						/* Set this points as the start anchor */
-					
+
 						n = 0;
 						add_node (x, y, z, v, &n, low, X_vert, Y_vert, topo, zgrd, ij+ij_inc[low], bin+bin_inc[low]);
 						start_side = next_side = low;
 						way = 1;
-					
+
 						this_cont = start_cont;
 						while (this_cont) {
-					
+
 							next_up = (this_cont->next_cont) ? this_cont->next_cont->value : DBL_MAX;
 							/* First get all the x/y pairs for this contour */
-						
+
 							for (k = 0, this_point = this_cont->first_point; this_point; this_point = this_point->next_point) {
 								xcont[k] = this_point->x;
 								ycont[k] = this_point->y;
@@ -1296,10 +1303,10 @@ main (int argc, char **argv)
 								k++;
 							}
 							ncont = k;
-						
+
 							entry_side = get_side (xcont[0], ycont[0], x_left, y_bottom, header.x_inc, header.y_inc, dx2, dy2);
 							exit_side  = get_side (xcont[ncont-1], ycont[ncont-1], x_left, y_bottom, header.x_inc, header.y_inc, dx2, dy2);
-						
+
 							while (!(next_side == entry_side || next_side == exit_side)) {	/* Must add intervening corner */
 								if (way == 1) next_side = (next_side + 1) % 4;
 								add_node (x, y, z, v, &n, next_side, X_vert, Y_vert, topo, zgrd, ij+ij_inc[next_side], bin+bin_inc[next_side]);
@@ -1314,22 +1321,22 @@ main (int argc, char **argv)
 								next_side = entry_side;
 							}
 							/* Now we must complete the polygon if necessary */
-						
+
 							while (!(start_side == next_side)) {	/* Must add intervening corner */
 								if (way == 1) next_side = (next_side + 1) % 4;
 								add_node (x, y, z, v, &n, next_side, X_vert, Y_vert, topo, zgrd, ij+ij_inc[next_side], bin+bin_inc[next_side]);
 								if (way == -1) next_side = (next_side - 1 + 4) % 4;
 							}
-						
+
 							/* Compute the xy from the xyz triplets */
-						
+
 							for (k = 0; k < n; k++) GMT_geoz_to_xy (x[k], y[k], z[k], &xx[k], &yy[k]);
 							z_ave = (GMT_continuous) ? get_z_ave (v, next_up, n) : this_cont->value;
-						
+
 							/* Now paint the polygon piece */
-						
+
 							paint_it (xx, yy, n, z_ave-small, intens, this_intensity);
-						
+
 							/* Reset the anchor points to previous contour */
 
 							n = 0;
@@ -1340,29 +1347,29 @@ main (int argc, char **argv)
 
 							this_cont = this_cont->next_cont;	/* Goto next contour */
  						}
-					
+
 						/* Final contour needs to compete with corners only */
-					
+
 						while (!(start_side == next_side)) {	/* Must add intervening corner */
 							if (way == 1) next_side = (next_side +1) % 4;
 							add_node (x, y, z, v, &n, next_side, X_vert, Y_vert, topo, zgrd, ij+ij_inc[next_side], bin+bin_inc[next_side]);
 							if (way == -1) next_side = (next_side - 1 + 4) % 4;
 						}
-						
+
 						/* Compute the xy from the xyz triplets */
-						
+
 						for (k = 0; k < n; k++) GMT_geoz_to_xy (x[k], y[k], z[k], &xx[k], &yy[k]);
-						
+
 						z_ave = (GMT_continuous) ? get_z_ave (v, next_up, n) : v[0];
-						
+
 						/* Now paint the polygon piece */
-						
+
 						paint_it (xx, yy, n, z_ave+small, intens, this_intensity);
-						
+
 					} /* End non-saddle case */
-					
+
 					/* Draw contour lines if desired */
-					
+
 					pen_not_set = TRUE;
 					for (this_cont = start_cont; draw_contours && this_cont; this_cont = this_cont->next_cont) {
 						for (k = 0, this_point = this_cont->first_point; this_point; this_point = this_point->next_point) {
@@ -1385,7 +1392,7 @@ main (int argc, char **argv)
 					}
 				}
 				else {	/* No Contours, paint tile (any of the corner values will give correct color */
-					
+
 					if (GMT_continuous) {
 						for (n = 0, z_ave = 0.0; n < 4; n++) z_ave += zgrd[bin+bin_inc[n]];
 						z_ave *= 0.25;
@@ -1393,7 +1400,7 @@ main (int argc, char **argv)
 					else
 						z_ave = zgrd[bin];
 					/* Now paint the polygon piece */
-						
+
 					for (k = 0; k < 4; k++) GMT_geoz_to_xy (X_vert[k], Y_vert[k], (double)(topo[ij+ij_inc[k]]), &xmesh[k], &ymesh[k]);
 					paint_it (xmesh, ymesh, 4, z_ave, intens, this_intensity);
 					if (outline) ps_patch (xmesh, ymesh, 4, GMT_no_rgb, TRUE);
@@ -1406,8 +1413,8 @@ main (int argc, char **argv)
 		GMT_free ((void *) ycont);
 		GMT_free ((void *) zcont);
 		GMT_free ((void *) vcont);
-	}			
-	
+	}
+
 	if (pen[1].texture || pen[0].texture) ps_setdash (CNULL, 0);
 
 	if (project_info.z_pars[0] == 0.0) GMT_map_clip_off();
@@ -1452,15 +1459,15 @@ main (int argc, char **argv)
 			ps_polygon (xx, yy, n, rgb_facade, TRUE);
 		}
 	}
-	
+
 	if (project_info.three_D) GMT_vertical_axis (2);	/* Draw background axis */
 	if (project_info.three_D) ps_rotatetrans (z_project.xmin, z_project.ymin, 0.0);
 	if (!project_info.three_D) GMT_map_basemap ();	/* Plot basemap last if not 3-D */
-	
+
 	ps_plotend (gmtdefs.last_page);
-	
+
 	/* Free memory */
-	
+
 	if (get_contours) {
 		for (ij = 0; ij < nm; ij++) {
 			if (!binij[ij].first_cont) continue;
@@ -1481,7 +1488,7 @@ main (int argc, char **argv)
 		}
 		GMT_free ((void *)binij);
 	}
-	
+
 	GMT_free ((void *)xx);
 	GMT_free ((void *)yy);
 	GMT_free ((void *)x);
@@ -1491,20 +1498,22 @@ main (int argc, char **argv)
 	GMT_free ((void *)topo);
 	if (intens) GMT_free ((void *)intensity);
 	if (drape) for (i = 0; i < 3; i++) GMT_free ((void *)grd[i]);
-	
+
 	if (gmtdefs.verbose) fprintf (stderr, "%s: Done!\n", GMT_program);
-	
+
 	GMT_end (argc, argv);
+
+	exit (EXIT_SUCCESS);
 }
 
 struct CONT *get_cont_struct (int bin, double value)
 {
 	struct CONT *cont, *new_cont;
-	
+
 	if (!binij[bin].first_cont) binij[bin].first_cont = (struct CONT *) GMT_memory (VNULL, (size_t)1, sizeof (struct CONT), GMT_program);
 
 	for (cont = binij[bin].first_cont; cont->next_cont && cont->next_cont->value <= value; cont = cont->next_cont);
-	
+
 	new_cont = (struct CONT *) GMT_memory (VNULL, (size_t)1, sizeof (struct CONT), GMT_program);
 	if (cont->next_cont) {	/* Put it in the link */
 		new_cont->next_cont = cont->next_cont;
@@ -1518,7 +1527,7 @@ struct CONT *get_cont_struct (int bin, double value)
 struct POINT *get_point (double x, double y)
 {
 	struct POINT *point;
-	
+
 	point = (struct POINT *) GMT_memory (VNULL, (size_t)1, sizeof (struct POINT), GMT_program);
 	point->x = x;
 	point->y = y;
@@ -1529,13 +1538,13 @@ void grdview_init_setup (struct GRD_HEADER *header, float *topo, int two, BOOLEA
 {
 	int i, j, ij;
 	double xtmp, ytmp, tmp, delx, dely;
-	
+
 	delx = (header->node_offset) ? 0.5 * header->x_inc :0.0;
 	dely = (header->node_offset) ? 0.5 * header->y_inc :0.0;
 	/* Find projected min/max in y-direction */
-	
+
 	z_project.ymax = z_project.ymin;	/* Reset from whatever it was */
-	
+
 	for (j = 0; j < header->ny; j++) {
 		ij = (j + two) * header->nx + two;
 		for (i = 0; i < header->nx; i++, ij++) {
@@ -1577,7 +1586,7 @@ int pixel_inside (int ip, int jp, int *ix, int *iy, int bin)
 {
 	int i, what;
 	double x[6], y[6];
-	
+
 	for (i = 0; i < 4; i++) {
 		x[i] = ix[bin+bin_inc[i]];
 		y[i] = iy[bin+bin_inc[i]];
@@ -1599,7 +1608,7 @@ int get_side (double x, double y, double x_left, double y_bottom, double xinc, d
 
 	double del_x, del_y;
 	int side;
-				
+
 	del_x = x - x_left;
 	if (del_x > dx2) del_x = xinc - del_x;
 	del_y = y - y_bottom;
@@ -1634,7 +1643,7 @@ void copy_points_bw (double x[], double y[], double z[], double v[], double xcon
 double get_z_ave (double v[], double next_up, int n) {
 	int k;
 	double z_ave;
-	
+
 	for (k = 0, z_ave = 0.0; k < n; k++) z_ave += MIN (v[k], next_up);
 	return (z_ave / n);
 }
@@ -1651,14 +1660,14 @@ void add_node (double x[], double y[], double z[], double v[], int *k, int node,
 void paint_it (double x[], double y[], int n, double z, BOOLEAN intens, double intensity) {
 	int index, rgb[3];
 	struct GMT_FILL *f;
-	
+
 	if (n < 3) return;	/* Need at least 3 points to make a polygon */
-	
+
 	index = GMT_get_rgb24 (z, rgb);
 	if (GMT_cpt_skip) return;	/* Skip this z-slice */
-	
+
 	/* Now we must paint, with colors or patterns */
-	
+
 	if ((index >= 0 && (f = GMT_lut[index].fill)) || (index < 0 && (f = GMT_bfn[index+3].fill))) {	/* Pattern */
 		GMT_fill (x, y, n, f, FALSE);
 	}
