@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: grdvolume.c,v 1.12 2004/01/02 22:45:13 pwessel Exp $
+ *	$Id: grdvolume.c,v 1.17 2005/12/17 05:59:22 pwessel Exp $
  *
- *	Copyright (c) 1991-2004 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2006 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -40,33 +40,33 @@ double vol_prism_frac_x (float *z, int ij, int nx, double x0, double x1, double
 double vol_prism_frac_y (float *z, int ij, int nx, double y0, double y1, double a, double b, double c, double d);
 double median3 (double x[]);
 
-main (int argc, char **argv)
+int main (int argc, char **argv)
 {
 	int i, j, n = 0, c, ij, nx, ny, ij_inc[4], one_or_zero, k, pos, neg, nc, n_contours, mode = 0, nz = 0, nm; 
-	
+
 	BOOLEAN error = FALSE, mapping = FALSE, contour = FALSE, full = FALSE, bad, cut[4];
 	BOOLEAN set_base = FALSE;
-	
+
 	double take_out, west, east, south, north, dv, da, cval = 0.0, x_size, y_size, m_pr_deg, x_fact, shift = 0.0, fact = 1.0;
-	double *area, *vol, *height, clow, chigh, deltac, base, this_base, small;
-	
+	double *area, *vol, *height, clow, chigh, deltac, base = GMT_d_NaN, this_base, small;
+
 	float *f;
 
 	char *grdfile = CNULL, format[BUFSIZ], map_units = '\0';
-	
+
 	struct GRD_HEADER grd;
-	
+
 	west = east = south = north = 0.0;
 	chigh = deltac = clow = 0.0;
-	
+
 	argc = GMT_begin (argc, argv);
 
 	for (i = 1; i < argc; i++) {
 		if (argv[i][0] == '-') {
 			switch (argv[i][1]) {
-			
+
 				/* Common parameters */
-			
+
 				case 'V':
 					if (argv[i][2] == 'L' || argv[i][2] == 'l') full = TRUE;
 				case 'R':
@@ -77,11 +77,18 @@ main (int argc, char **argv)
 
 
 				/* Supplemental parameters */
-				
+
 				case 'C':
 					contour = TRUE;
 					n = sscanf (&argv[i][2], "%lf/%lf/%lf", &clow, &chigh, &deltac);
-					if (n == 1) chigh = clow, deltac = 1.0;	/* So calculation of ncontours will yield 1 */
+					if (n == 3) {
+						if (clow >= chigh || deltac <= 0.0) {
+							fprintf (stderr, "%s: GMT SYNTAX ERROR -C:  high must exceed low and delta must be positive\n", GMT_program);
+							error++;
+						}
+					}
+					else
+						chigh = clow, deltac = 1.0;	/* So calculation of ncontours will yield 1 */
 					break;
 				case 'L':
 					base = (argv[i][2]) ? atof (&argv[i][2]) : GMT_d_NaN;
@@ -106,13 +113,13 @@ main (int argc, char **argv)
 		else
 			grdfile = argv[i];
 	}
-	
+
 	if (argc == 1 || GMT_quick) {
 		fprintf (stderr,"grdvolume %s - Calculating volume under a surface within a contour\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: grdvolume <grdfile> [-C<cval> or -C<low/high/delta>] [-L<base>] [-S[k]] [-T] [-Rw/s/e/n[r]] [-V] [-Z<fact>[/<shift>]]\n");
-		
+
 		if (GMT_quick) exit (EXIT_FAILURE);
-		
+
 		fprintf (stderr, "\t<grdfile> is the name of the 2-D binary data set\n");
 		fprintf (stderr, "\n\tOPTIONS:\n");
 		fprintf (stderr, "\t-C find area and volume inside the <cval> contour\n");
@@ -128,7 +135,7 @@ main (int argc, char **argv)
 		GMT_explain_option ('.');
 		exit (EXIT_FAILURE);
 	}
-	
+
 	if (!grdfile) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR:  Must specify input grd file\n", GMT_program);
 		error++;
@@ -158,7 +165,7 @@ main (int argc, char **argv)
 		fprintf (stderr, "%s: Error opening file %s\n", GMT_program, grdfile);
 		exit (EXIT_FAILURE);
 	}
-	
+
 	if (!project_info.region_supplied) {	/* No subset asked for */
 		west = grd.x_min;
 		east = grd.x_max;
@@ -254,7 +261,7 @@ main (int argc, char **argv)
 					if ((f[ij-nx] * f[ij]) < 0.0)      nc++, cut[3] = TRUE;	/* Crossing the W border */
 
 					if (nc < 2) continue;	/* Can happen if some nodes were 0 and then reset to small, thus passing the test */
-					
+
 					if (nc == 4) {	/* Saddle scenario */
 						if (f[ij] > 0) {	/* Need both SW and NE triangles */
 							SW_triangle (f, ij, nx, TRUE, &dv, &da);
@@ -264,7 +271,7 @@ main (int argc, char **argv)
 							SE_triangle (f, ij, nx, TRUE, &dv, &da);
 							NW_triangle (f, ij, nx, TRUE, &dv, &da);
 						}
-						
+
 					}
 					else if (cut[0]) {	/* Contour enters at S border ... */
 						if (cut[1])	/* and exits at E border */
@@ -307,7 +314,7 @@ main (int argc, char **argv)
 			for (i = 0; i < grd.nx-1; i++, ij++) {
 				for (k = 0, bad = FALSE; !bad && k < 4; k++) if (GMT_is_fnan (f[ij+ij_inc[k]])) bad = TRUE;
                                 if (bad) continue;
-				
+
 				dv += 0.25 * (f[ij] + f[ij+1] + f[ij-nx] + f[ij-nx+1]);
 				da += 1.0;
 				if (set_base) dv -= this_base;	/* Adjust for lower starting base */
@@ -323,7 +330,7 @@ main (int argc, char **argv)
 			area[0] += da;
 		}
 	}
-	
+
 	/* Adjust values for actual y dimension */
 
 	for (c = 0; c < n_contours; c++) {
@@ -359,23 +366,26 @@ main (int argc, char **argv)
 	GMT_free ((void *)area);
 	GMT_free ((void *)vol);
 	GMT_free ((void *)height);
-	
+
 	GMT_end (argc, argv);
+
+	exit (EXIT_SUCCESS);
 }
 
 void SW_triangle (float f[], int ij, int nx, BOOLEAN triangle, double *dv, double *da)
 {	/* Calculates area of a SW-corner triangle */
 	/* triangle = TRUE gets triangle, FALSE gives the complementary area */
-	double x1, y0;
+	double x1, y0, frac;
 
 	x1 = f[ij] / (f[ij] - f[ij+1]);
 	y0 = f[ij] / (f[ij] - f[ij-nx]);
+	frac = (x1 == 0.0) ? 0.0 : vol_prism_frac_x (f, ij, nx, 0.0, x1, 0.0, 0.0, -y0 / x1, y0);
 	if (triangle) {
-		*dv += vol_prism_frac_x (f, ij, nx, 0.0, x1, 0.0, 0.0, -y0 / x1, y0);
+		*dv += frac;
 		*da += 0.5 * x1 * y0;
 	}
 	else {
-		*dv += 0.25 * (f[ij] + f[ij+1] + f[ij-nx] + f[ij-nx+1]) - vol_prism_frac_x (f, ij, nx, 0.0, x1, 0.0, 0.0, -y0 / x1, y0);
+		*dv += 0.25 * (f[ij] + f[ij+1] + f[ij-nx] + f[ij-nx+1]) - frac;
 		*da += 1.0 - 0.5 * x1 * y0;
 	}
 }
@@ -383,19 +393,22 @@ void SW_triangle (float f[], int ij, int nx, BOOLEAN triangle, double *dv, doubl
 void NE_triangle (float f[], int ij, int nx, BOOLEAN triangle, double *dv, double *da)
 {	/* Calculates area of a NE-corner triangle */
 	/* triangle = TRUE gets triangle, FALSE gives the complementary area */
-	double x0, y1, a, x0_1, y1_1;
+	double x0, y1, a, x0_1, y1_1, frac = 0.0;
 
 	x0 = f[ij-nx] / (f[ij-nx] - f[ij+1-nx]);
 	y1 = f[ij+1] / (f[ij+1] - f[ij+1-nx]);
 	x0_1 = 1.0 - x0;
 	y1_1 = y1 - 1.0;
-	a = y1_1 / x0_1;
+	if (x0_1 != 0.0) {
+		a = y1_1 / x0_1;
+		frac = vol_prism_frac_x (f, ij, nx, x0, 1.0, a, 1.0 - a * x0, 0.0, 0.0);
+	}
 	if (triangle) {
-		*dv += vol_prism_frac_x (f, ij, nx, x0, 1.0, a, 1.0 - a * x0, 0.0, 0.0);
+		*dv += frac;
 		*da -= 0.5 * x0_1 * y1_1;	/* -ve because we need 1 - y1 */
 	}
 	else {
-		*dv += 0.25 * (f[ij] + f[ij+1] + f[ij-nx] + f[ij-nx+1]) - vol_prism_frac_x (f, ij, nx, x0, 1.0, a, 1.0 - a * x0, 0.0, 0.0);
+		*dv += 0.25 * (f[ij] + f[ij+1] + f[ij-nx] + f[ij-nx+1]) - frac;
 		*da += 1.0 + 0.5 * x0_1 * y1_1;	/* +ve because we need 1 - y1 */
 	}
 }
@@ -403,18 +416,21 @@ void NE_triangle (float f[], int ij, int nx, BOOLEAN triangle, double *dv, doubl
 void SE_triangle (float f[], int ij, int nx, BOOLEAN triangle, double *dv, double *da)
 {	/* Calculates area of a SE-corner triangle */
 	/* triangle = TRUE gets triangle, FALSE gives the complementary area */
-	double x0, y1, c, x0_1;
+	double x0, y1, c, x0_1, frac = 0.0;
 
 	x0 = f[ij] / (f[ij] - f[ij+1]);
 	y1 = f[ij+1] / (f[ij+1] - f[ij+1-nx]);
 	x0_1 = 1.0 - x0;
-	c = y1 / x0_1;
+	if (x0_1 != 0.0) {
+		c = y1 / x0_1;
+		frac = vol_prism_frac_x (f, ij, nx, x0, 1.0, 0.0, 0.0, c, -c * x0);
+	}
 	if (triangle) {
-		*dv += vol_prism_frac_x (f, ij, nx, x0, 1.0, 0.0, 0.0, c, -c * x0);
+		*dv += frac;
 		*da += 0.5 * x0_1 * y1;
 	}
 	else {
-		*dv += 0.25 * (f[ij] + f[ij+1] + f[ij-nx] + f[ij-nx+1]) - vol_prism_frac_x (f, ij, nx, x0, 1.0, 0.0, 0.0, c, -c * x0);
+		*dv += 0.25 * (f[ij] + f[ij+1] + f[ij-nx] + f[ij-nx+1]) - frac;
 		*da += 1.0 - 0.5 * x0_1 * y1;
 	}
 }
@@ -422,17 +438,18 @@ void SE_triangle (float f[], int ij, int nx, BOOLEAN triangle, double *dv, doubl
 void NW_triangle (float f[], int ij, int nx, BOOLEAN triangle, double *dv, double *da)
 {	/* Calculates area of a NW-corner triangle */
 	/* triangle = TRUE gets triangle, FALSE gives the complementary area */
-	double x1, y0, y0_1;
+	double x1, y0, y0_1, frac;
 
 	x1 = f[ij-nx] / (f[ij-nx] - f[ij+1-nx]);
 	y0 = f[ij] / (f[ij] - f[ij-nx]);
 	y0_1 = 1.0 - y0;
+	frac = (x1 == 0.0) ? 0.0 : vol_prism_frac_x (f, ij, nx, 0.0, x1, y0_1 / x1, y0, 0.0, 1.0);
 	if (triangle) {
-		*dv += vol_prism_frac_x (f, ij, nx, 0.0, x1, y0_1 / x1, y0, 0.0, 1.0);
+		*dv += frac;
 		*da += 0.5 * x1 * y0_1;
 	}
 	else {
-		*dv += 0.25 * (f[ij] + f[ij+1] + f[ij-nx] + f[ij-nx+1]) - vol_prism_frac_x (f, ij, nx, 0.0, x1, y0_1 / x1, y0, 0.0, 1.0);
+		*dv += 0.25 * (f[ij] + f[ij+1] + f[ij-nx] + f[ij-nx+1]) - frac;
 		*da += 1.0 - 0.5 * x1 * y0_1;
 	}
 }
@@ -532,7 +549,7 @@ int ors_find_kink (double y[], int n, int mode)
 	int i, ic, im;
 	double *c, *f;
 
-	if (mode == 0) {	/* Find maximum value */	
+	if (mode == 0) {	/* Find maximum value */
 
 		for (i = im = 0; i < n; i++) if (y[i] > y[im]) im = i;
 		return (im);
