@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: psmask.c,v 1.22 2004/09/10 23:16:15 pwessel Exp $
+ *	$Id: psmask.c,v 1.33 2006/01/17 13:51:35 remko Exp $
  *
- *	Copyright (c) 1991-2004 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2006 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -38,8 +38,9 @@
  */
 
 #include "gmt.h"
+#include "pslib.h"
 
-#define IJ(i,j) ((i) + (j) * nx)
+#define IJ(i,j) ((i) + (j) * h->nx)
 
 /* Global variables for contour_sub routines */
 
@@ -50,47 +51,48 @@ BOOLEAN first_dump = TRUE;
 char *grd;
 int *edge;
 double *x, *y;	/* Arrays holding the contour xy values */
-double *x_grid, *y_grid;
 
 void draw_clip_contours (double *xx, double *yy, int nn, int rgb[], int id, int flag);
 void dump_clip_contours (double *xx, double *yy, int nn, int id, char *file);
 void shrink_clip_contours (double *x, double *y, int n, double w, double e);
-int trace_clip_contours (char *grd, int nx, int *edge, double west, double north, double dx, double dy, double xinc2, double yinc2, double **xx, double **yy, int i, int j, int kk, int *max);
-int clip_contours (char *grd, int nx, int ny, double west, double north, double dx, double dy, double xinc2, double yinc2, int *edge, int first, double **x, double **y, int *max);
+int trace_clip_contours (char *grd, int *edge, struct GRD_HEADER *h, double xinc2, double yinc2, double **xx, double **yy, int i, int j, int kk, int *max);
+int clip_contours (char *grd, struct GRD_HEADER *h, double xinc2, double yinc2, int *edge, int first, double **x, double **y, int *max);
 
-main (int argc, char **argv)
+int main (int argc, char **argv)
 {
 	int i, j, ij, n, nm, n_edges, di, dj, ii, jj, n_expected_fields;
-	int nx, ny, section, n_alloc, n_read, n_fields, one_or_zero = 1;
-	
+	int section, n_alloc, n_read, n_fields, one_or_zero = 1, distance_flag = 0;
+
 	BOOLEAN error = FALSE, first = TRUE, dump = FALSE, invert = FALSE;
 	BOOLEAN end_of_clip = FALSE, find_paths = TRUE, use_poly;
-	BOOLEAN map_units = FALSE, pixel = FALSE, node_only;
-	
+	BOOLEAN map_units = FALSE, node_only;
+
 	char dfile[BUFSIZ], line[BUFSIZ];
-	
-	double west, east, north, south, dx, dy, radius = 0.0, *in, distance;
-	double x0, y0, x1, y1, xx[4], yy[4], dummy, shrink, offset = 0.5;
-	double xinc2, yinc2, idx, idy;
-	
+
+	double radius = 0.0, *in, distance;
+	double x0, y0, x1, y1, shrink = 1.0;
+	double xinc2, yinc2, idx = 1.0, idy = 1.0;
+
 	FILE *fp = NULL;
-	
+
 	struct GMT_FILL fill;
-	
-	west = east = north = south = xinc2 = yinc2 = 0.0;
+
+	struct GRD_HEADER h;
 
 	argc = GMT_begin (argc, argv);
 
+	GMT_grd_init (&h, argc, argv, FALSE);
+
 	GMT_init_fill (&fill, -1, -1, -1);
 	strcpy (dfile, "mask");
-	dx = dy = 0.0;
-	
+	xinc2 = yinc2 = 0.0;
+
 	for (i = 1; i < argc; i++) {
 		if (argv[i][0] == '-') {
 			switch (argv[i][1]) {
-		
+
 				/* Common parameters */
-			
+
 				case 'B':
 				case 'J':
 				case 'K':
@@ -107,11 +109,11 @@ main (int argc, char **argv)
 				case 'c':
 				case ':':
 				case '\0':
-					error += GMT_get_common_args (argv[i], &west, &east, &south, &north);
+					error += GMT_get_common_args (argv[i], &h.x_min, &h.x_max, &h.y_min, &h.y_max);
 					break;
-				
+
 				/* Supplemental parameters */
-			
+
 				case 'C':
 					end_of_clip = TRUE;
 					break;
@@ -123,7 +125,7 @@ main (int argc, char **argv)
 					sscanf (&argv[i][2], "%lf/%lf", &z_project.view_azimuth, &z_project.view_elevation);
 					break;
 				case 'F':
-					pixel = TRUE;
+					h.node_offset = 1;
 					break;
 				case 'G':
 					if (GMT_getfill (&argv[i][2], &fill)) {
@@ -132,14 +134,19 @@ main (int argc, char **argv)
 					}
 					break;
 				case 'I':
-					GMT_getinc (&argv[i][2], &dx, &dy);
+					if (GMT_getinc (&argv[i][2], &h.x_inc, &h.y_inc)) {
+						GMT_inc_syntax ('I', 1);
+						error = TRUE;
+					}
 					break;
 				case 'N':
 					invert = TRUE;
 					break;
 				case 'S':	/* Radius of influence */
-					GMT_getinc(&argv[i][2], &radius, &radius);
+					radius = GMT_getradius (&argv[i][2]);
 					map_units = (argv[i][strlen(argv[i])-1] == 'k');
+					if (argv[i][strlen(argv[i])-1] == 'k') distance_flag = 1;
+					if (argv[i][strlen(argv[i])-1] == 'K') distance_flag = 2;
 					break;
 				case 'T':
 					find_paths = FALSE;
@@ -157,19 +164,18 @@ main (int argc, char **argv)
 			}
 		}
 	}
-	
+
 	if (argc == 1 || GMT_quick) {
 		fprintf (stderr,"psmask %s - Clipping of 2-D data sets\n\n", GMT_VERSION);
-		fprintf (stderr, "usage: psmask <xyz-file> -I<xinc>[m|c][/<yinc>[m|c]] -J<params> -R<west/east/south/north>\n");
+		fprintf (stderr, "usage: psmask <xyz-file> -I<xinc[u][!|+]>[/<yinc>[u][!|+]] -J<params> -R<west/east/south/north>\n");
 		fprintf (stderr, "\t[-B<tickinfo>] [-C] -D[-]<file> [-Eaz/el] [-F] [-G<fill>] [-H[<nrec>]] [-K] [-M[<flag>]]\n");
 		fprintf (stderr, "\t[-N] [-O] [-P] [-S<radius>[k]] [-T] [-U[<label>]] [-V] [-X<x_shift>] [-Y<y_shift>]\n");
 		fprintf (stderr, "\t[-c<ncopies>][ -:] [-bi[s][<n>]] [-bo[s][<n>]]\n\n");
-		
+
 		if (GMT_quick) exit (EXIT_FAILURE);
-		
+
 		fprintf (stderr, "\t<xyz-file> is the datafile.  If not given, read standard input\n");
-		fprintf (stderr, "\t-I sets grid increments; enter xinc, optionally xinc/yinc.\n");
-		fprintf (stderr, "\t   Default is yinc = xinc.  Append an m [or s] to xinc or yinc to indicate minutes [or seconds]\n");
+		GMT_inc_syntax ('I', 0);
 		GMT_explain_option ('j');
 		GMT_explain_option ('R');
 		fprintf (stderr, "\n\tOPTIONS:\n");
@@ -180,20 +186,22 @@ main (int argc, char **argv)
 		fprintf (stderr, "\t   use the rest of the name for a single multi-segment file.\n");
 		fprintf (stderr, "\t-E set azimuth and elevation of viewpoint for 3-D perspective [180/90]\n");
 		fprintf (stderr, "\t-F Force pixel registration [Default is gridline registration]\n");
-		fprintf (stderr, "\t-G sets fill color r/g/b [Default is no fill]\n");
+		fprintf (stderr, "\t-G sets fill color or pattern [Default is no fill]\n");
 		GMT_explain_option ('H');
 		GMT_explain_option ('K');
 		GMT_explain_option ('M');
 		fprintf (stderr, "\t-N will invert the sense of the clipping [or tiling]\n");
 		GMT_explain_option ('O');
 		GMT_explain_option ('P');
-		fprintf (stderr, "\t-S Search radius: Nodes inside circles of <radius> centered on\n");
+		fprintf (stderr, "\t-S sets search radius in -R, -I units; append m or c for minutes or seconds.\n");
+		fprintf (stderr, "\t   This means nodes inside circles of <radius> centered on\n");
 		fprintf (stderr, "\t   the input data points are considered to be reliable estimates of the surface\n");
 		fprintf (stderr, "\t   Default is -S0, i.e., only the nearest node is considered reliable\n");
-		fprintf (stderr, "\t   Append k to indicate km (hence -R -I are in degrees)\n");
+		fprintf (stderr, "\t   Append k for km (implies -R,-I in degrees), use flat Earth approximation.\n");
+		fprintf (stderr, "\t   Append K for km (implies -R,-I in degrees), use exact geodesic distances.\n");
+		fprintf (stderr, "\t   If the current ELLIPSOID is Sphere then spherical great circle distances are used.\n");
 		fprintf (stderr, "\t-T will paint tiles.  [Default will trace data outline]\n");
 		fprintf (stderr, "\t   If set you must also specify a color/fill with -G\n");
-		fprintf (stderr, "\t   !May only be used with linear, Mercator, and basic cylindrical projections!\n");
 		GMT_explain_option ('U');
 		GMT_explain_option ('V');
 		GMT_explain_option ('X');
@@ -207,7 +215,7 @@ main (int argc, char **argv)
 		GMT_explain_option ('.');
 		exit (EXIT_FAILURE);
 	}
-	
+
 	if (!end_of_clip) {
 		if (z_project.view_azimuth > 360.0 || z_project.view_elevation <= 0.0 || z_project.view_elevation > 90.0) {
 			fprintf (stderr, "%s: GMT SYNTAX ERROR -E option:  Enter azimuth in 0-360 range, elevation in 0-90 range\n", GMT_program);
@@ -221,11 +229,12 @@ main (int argc, char **argv)
 			fprintf (stderr, "%s: GMT SYNTAX ERROR -T option:  Only available with Linear, Mercator, or basic cylindrical projections\n", GMT_program);
 			error++;
 		}
-		if (!find_paths && fill.rgb[0] == -1) {
+		if (!find_paths && !(fill.rgb[0] >= 0 || fill.use_pattern)) {
 			fprintf (stderr, "%s: GMT SYNTAX ERROR -T option:  Must also specify a tile color with -G\n", GMT_program);
 			error++;
 		}
-		if (dx <= 0.0 || dy <= 0.0) {
+		GMT_RI_prepare (&h);	/* Ensure -R -I consistency and set nx, ny */
+		if (h.x_inc <= 0.0 || h.y_inc <= 0.0) {
 			fprintf (stderr, "%s: GMT SYNTAX ERROR -I option:  Must specify positive increments\n", GMT_program);
 			error++;
 		}
@@ -242,6 +251,12 @@ main (int argc, char **argv)
 			fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data cannot have header -H\n", GMT_program);
 			error++;
 		}
+		h.xy_off = 0.5 * h.node_offset;
+		xinc2 = 0.5 * h.x_inc;
+		yinc2 = 0.5 * h.y_inc;
+		one_or_zero = 1 - h.node_offset;
+		idx = 1.0 / h.x_inc;
+		idy = 1.0 / h.y_inc;
 	}
 
 	if (error) exit (EXIT_FAILURE);
@@ -253,16 +268,8 @@ main (int argc, char **argv)
 		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 
-	if (pixel) {
-		xinc2 = 0.5 * dx;
-		yinc2 = 0.5 * dy;
-		one_or_zero = 0;
-		offset = 0.0;
-	}
-	idx = 1.0 / dx;
-	idy = 1.0 / dy;
 
-	if (!end_of_clip) GMT_map_setup (west, east, south, north);
+	if (!end_of_clip) GMT_map_setup (h.x_min, h.x_max, h.y_min, h.y_max);
 
 	if (!project_info.x_off_supplied && gmtdefs.overlay) gmtdefs.x_origin = 0.0;
 	if (!project_info.y_off_supplied && gmtdefs.overlay) gmtdefs.y_origin = 0.0;
@@ -272,31 +279,47 @@ main (int argc, char **argv)
 		gmtdefs.dpi, GMT_INCH, gmtdefs.paper_width, gmtdefs.page_rgb, gmtdefs.encoding.name, GMT_epsinfo (argv[0]));
 	GMT_echo_command (argc, argv);
 	if (gmtdefs.unix_time) GMT_timestamp (argc, argv);
-	
+
 	if (!end_of_clip) {	/* Start new clip_path */
-	
+
 		if (project_info.three_D) ps_transrotate (-z_project.xmin, -z_project.ymin, 0.0);
 
 		if (gmtdefs.verbose) fprintf (stderr, "%s: Allocate memory, read and process data file\n", GMT_program);
-	
+
 		/* Enlarge region by 1 row/column */
 
-		west -= dx;	east += dx;	south -= dy;	north += dy;
+		h.x_min -= h.x_inc;	h.x_max += h.x_inc;	h.y_min -= h.y_inc;	h.y_max += h.y_inc;
 
-		nx = irint ((east - west)   * idx) + one_or_zero;
-		ny = irint ((north - south) * idy) + one_or_zero;
-	
-		nm = nx * ny;
+		h.nx = irint ((h.x_max - h.x_min) * idx) + one_or_zero;
+		h.ny = irint ((h.y_max - h.y_min) * idy) + one_or_zero;
+
+		nm = h.nx * h.ny;
 		grd = (char *) GMT_memory (VNULL, (size_t)nm, sizeof (char), GMT_program);
-	
+
 		if (map_units) {
-			shrink = cos (0.5 * (south + north) * D2R);
-			di = (int)ceil (radius / (DEG2KM * dx * shrink));
-			dj = (int)ceil (radius / (DEG2KM * dy));
+			shrink = cos (0.5 * (h.y_min + h.y_max) * D2R);
+			di = (int)ceil (radius / (project_info.KM_PR_DEG * h.x_inc * shrink));
+			dj = (int)ceil (radius / (project_info.KM_PR_DEG * h.y_inc));
 		}
 		else {
-			di = irint (0.5 * radius / dx);
-			dj = irint (0.5 * radius / dy);
+			di = irint (0.5 * radius / h.x_inc);
+			dj = irint (0.5 * radius / h.y_inc);
+		}
+		if (distance_flag == 2 && !SPHERICAL) distance_flag = 3;	/* Use geodesics */
+		switch (distance_flag) {	/* Take different action depending on how we want distances calculated */
+
+			case 0:		/* Cartesian distance */
+				GMT_distance_func = GMT_cartesian_dist;
+				break;
+			case 1:		/* Flat Earth Approximation */
+				GMT_distance_func = GMT_flatearth_dist_km;
+				break;
+			case 2:		/* Full spherical calculation */
+				GMT_distance_func = GMT_great_circle_dist_km;
+				break;
+			case 3:		/* Full Ellipsoidal calculation */
+				GMT_distance_func = GMT_geodesic_dist_km;
+			break;
 		}
 		node_only = (di == 0 && dj == 0);
 
@@ -306,7 +329,7 @@ main (int argc, char **argv)
 			GMT_setmode (GMT_IN);
 #endif
 		}
-		
+
 		if (gmtdefs.io_header[GMT_IN]) for (i = 0; i < gmtdefs.n_header_recs; i++) GMT_fgets (line, BUFSIZ, fp);
 
 		n_expected_fields = (GMT_io.ncol[GMT_IN]) ? GMT_io.ncol[GMT_IN] : 2;
@@ -326,32 +349,32 @@ main (int argc, char **argv)
 
 			/* Determine the node closest to the data point */
 
-			i = (int)floor (((in[0] - west) * idx) + offset);
-			if (i < 0 || i >= nx) continue;
-			j = (int)floor (((north - in[1]) * idy) + offset);
-			if (j < 0 || j >= ny) continue;
+			i = GMT_x_to_i (in[0], h.x_min, h.x_inc, h.xy_off, h.nx);
+			if (i < 0 || i >= h.nx) continue;
+			j = GMT_y_to_j (in[1], h.y_min, h.y_inc, h.xy_off, h.ny);
+			if (j < 0 || j >= h.ny) continue;
 
 			if (node_only) {
-				grd[j*nx+i] = 1;
+				grd[j*h.nx+i] = 1;
 			}
 			else {
 
 				/* Set coordinate of this node */
 
-				x0 = west + i * dx + xinc2;
-				y0 = north - j * dy - yinc2;
-		
+				x0 = GMT_i_to_x (i, h.x_min, h.x_max, h.x_inc, h.xy_off, h.nx);
+				y0 = GMT_j_to_y (j, h.y_min, h.y_max, h.y_inc, h.xy_off, h.ny);
+
 				/* Set this and all nodes within radius distance to 1 */
 
 				for (ii = i - di; ii <= i + di; ii++) {
-					if (ii < 0 || ii >= nx) continue;
-					x1 = west + ii * dx + xinc2;
+					if (ii < 0 || ii >= h.nx) continue;
+					x1 = GMT_i_to_x (ii, h.x_min, h.x_max, h.x_inc, h.xy_off, h.nx);
 					for (jj = j - dj; jj <= j + dj; jj++) {
-						if (jj < 0 || jj >= ny) continue;
-						y1 = north - jj * dy - yinc2;
-						distance = (map_units) ? DEG2KM * hypot ((x1 - x0) * shrink, y1 - y0) : hypot (x1 - x0, y1 - y0);
+						if (jj < 0 || jj >= h.ny) continue;
+						y1 = GMT_j_to_y (jj, h.y_min, h.y_max, h.y_inc, h.xy_off, h.ny);
+						distance = (GMT_distance_func) (x1, y1, x0, y0);
 						if (distance > radius) continue;
-						grd[jj*nx+ii] = 1;
+						grd[jj*h.nx+ii] = 1;
 					}
 				}
 			}
@@ -363,71 +386,88 @@ main (int argc, char **argv)
 		if (invert) for (i = 0; i < nm; i++) grd[i] = 1 - grd[i];	/* Reverse sense of test */
 
 		/* Force perimeter nodes to be FALSE */
-		
-		for (i = 0, ij = (ny-1) * nx; i < nx; i++) grd[i] = grd[i+ij] = FALSE;
-		for (j = 0; j < ny; j++) grd[j*nx] = grd[(j+1)*nx-1] = FALSE;
-		
+
+		for (i = 0, ij = (h.ny-1) * h.nx; i < h.nx; i++) grd[i] = grd[i+ij] = FALSE;
+		for (j = 0; j < h.ny; j++) grd[j*h.nx] = grd[(j+1)*h.nx-1] = FALSE;
+
 		if (find_paths) {	/* Must trace the outline of ON/OFF values in grd */
 			x = (double *) GMT_memory (VNULL, (size_t)GMT_CHUNK, sizeof (double), GMT_program);
 			y = (double *) GMT_memory (VNULL, (size_t)GMT_CHUNK, sizeof (double), GMT_program);
 			n_alloc = GMT_CHUNK;
-	
-			n_edges = ny * (int )ceil (nx / 16.0);
+
+			n_edges = h.ny * (int )ceil (h.nx / 16.0);
 			edge = (int *) GMT_memory (VNULL, (size_t)n_edges, sizeof (int), GMT_program);
-	
+
 			GMT_map_basemap ();
 
 			if (gmtdefs.verbose) fprintf (stderr, "%s: Tracing the clip path\n", GMT_program);
-	
+
 			section = 0;
 			first = TRUE;
-			while ((n = clip_contours (grd, nx, ny, west, north, dx, dy, xinc2, yinc2, edge, first, &x, &y, &n_alloc)) > 0) {
-				shrink_clip_contours (x, y, n, west, east);
+			while ((n = clip_contours (grd, &h, xinc2, yinc2, edge, first, &x, &y, &n_alloc)) > 0) {
+				shrink_clip_contours (x, y, n, h.x_min, h.x_max);
 				if (dump) dump_clip_contours (x, y, n, section, dfile);
 				draw_clip_contours (x, y, n, fill.rgb, section, first);
 				first = FALSE;
 				section++;
 			}
-	
+
 			draw_clip_contours (x, y, 0, fill.rgb, section, 2);	/* Activate clip-path */
-		
+
 			GMT_free ((void *)edge);
 			GMT_free ((void *)x);
 			GMT_free ((void *)y);
 		}
 		else {	/* Just paint tiles */
+			int start, plot_n, n_use, np;
+			double *grd_x, grd_y, y_bot, y_top, *xx, *yy, *xp, *yp;
 			if (gmtdefs.verbose) fprintf (stderr, "%s: Tiling...\n", GMT_program);
-			x_grid = (double *) GMT_memory (VNULL, (size_t)nx, sizeof (double), GMT_program);
-			y_grid = (double *) GMT_memory (VNULL, (size_t)ny, sizeof (double), GMT_program);
-			for (i = 2; i < nx - 1; i++) GMT_geo_to_xy (west + (i - 0.5) * dx + xinc2, south + dy + yinc2, &x_grid[i], &dummy);
-			for (j = 2; j < ny - 1; j++) GMT_geo_to_xy (west + dx + xinc2, north - (j - 0.5) * dy - yinc2, &dummy, &y_grid[j]);
-			GMT_geo_to_xy (west + dx + xinc2, north - dy - yinc2, &x_grid[1], &y_grid[1]);
-			GMT_geo_to_xy (east - dx - xinc2, south + dy + yinc2, &x_grid[nx-1], &y_grid[ny-1]);
-			
+			grd_x = (double *) GMT_memory (VNULL, (size_t)h.nx, sizeof (double), GMT_program);
+			for (i = 0; i < h.nx; i++) grd_x[i] = GMT_i_to_x (i, h.x_min, h.x_max, h.x_inc, h.xy_off, h.nx);
+
 			use_poly = (fill.use_pattern || project_info.three_D);
 
-			for (j = 1; j < ny; j++) {
-				ij = j * nx + 1;
-				for (i = 1; i < nx; i++, ij++) {
+			for (j = 0; j < h.ny; j++) {
+				grd_y = GMT_j_to_y (j, h.y_min, h.y_max, h.y_inc, h.xy_off, h.ny);
+				y_bot = grd_y - yinc2;
+				y_top = grd_y + yinc2;
+				ij = j * h.nx + 1;
+				for (i = 0; i < h.nx; i++, ij++) {
 					if (((int)grd[ij]) == 0) continue;
+
+					np = GMT_graticule_path (&xx, &yy, 1, grd_x[i] - xinc2, grd_x[i] + xinc2, y_bot, y_top);
+					plot_n = GMT_clip_to_map (xx, yy, np, &xp, &yp);
+					GMT_free ((void *)xx);
+					GMT_free ((void *)yy);
+					if (plot_n == 0) continue;	/* Outside */
 					
-					if (use_poly) {
-						xx[0] = xx[3] = x_grid[i];	xx[1] = xx[2] = x_grid[i+1];
-						yy[0] = yy[1] = y_grid[j+1];	yy[2] = yy[3] = y_grid[j];
-						if (project_info.three_D) GMT_2D_to_3D (xx, yy, project_info.z_level, 4);
-						GMT_fill (xx, yy, 4, &fill, FALSE);
+					if ((*GMT_will_it_wrap) (xp, yp, plot_n, &start)) {	/* Polygon wraps */
+
+					/* First truncate agains left border */
+
+						GMT_n_plot = (*GMT_truncate) (xp, yp, plot_n, start, -1);
+						n_use = GMT_compact_line (GMT_x_plot, GMT_y_plot, GMT_n_plot, FALSE, 0);
+						GMT_fill (GMT_x_plot, GMT_y_plot, n_use, &fill, FALSE);
+
+						/* Then truncate agains right border */
+
+						GMT_n_plot = (*GMT_truncate) (xp, yp, plot_n, start, +1);
+						n_use = GMT_compact_line (GMT_x_plot, GMT_y_plot, GMT_n_plot, FALSE, 0);
+						GMT_fill (GMT_x_plot, GMT_y_plot, n_use, &fill, FALSE);
 					}
-					else
-						ps_rect (x_grid[i], y_grid[j+1], x_grid[i+1], y_grid[j], fill.rgb, FALSE);
+					else {
+						GMT_fill (xp, yp, plot_n, &fill, FALSE);
+					}
+					GMT_free ((void *)xp);
+					GMT_free ((void *)yp);
 				}
 			}
-			
-			GMT_free ((void *)x_grid);
-			GMT_free ((void *)y_grid);
-			
+
+			GMT_free ((void *)grd_x);
+
 			GMT_map_basemap ();
 		}
-		
+
 		if (project_info.three_D) ps_rotatetrans (z_project.xmin, z_project.ymin, 0.0);
 
 		GMT_free ((void *)grd);
@@ -438,31 +478,32 @@ main (int argc, char **argv)
 		GMT_map_basemap ();
 		if (gmtdefs.verbose) fprintf (stderr, "%s: clipping off!\n", GMT_program);
 	}
-	
+
 	ps_setpaint (gmtdefs.background_rgb);
-	
+
 	ps_plotend (gmtdefs.last_page);
-	
-	
+
 	GMT_end (argc, argv);
+
+	exit (EXIT_SUCCESS);
 }
 
 void draw_clip_contours (double *xx, double *yy, int nn, int rgb[], int id, int flag)
 {
 	int i;
 	double x, y;
-	char comment[64];
-	
+	char comment[GMT_TEXT_LEN];
+
 	if (nn < 2 && flag < 2) return;
-	
+
 	for (i = 0; i < nn; i++) {
 		x = xx[i];	y = yy[i];
 		GMT_geo_to_xy (x, y, &xx[i], &yy[i]);
 	}
 	nn = GMT_compact_line (xx, yy, nn, FALSE, 0);
-	
+
 	if (project_info.three_D) GMT_2D_to_3D (xx, yy, project_info.z_level, nn);
-	
+
 	if (nn > 0) {
 		sprintf (comment, "Start of clip path sub-segment %d", id);
 		ps_comment (comment);
@@ -471,7 +512,7 @@ void draw_clip_contours (double *xx, double *yy, int nn, int rgb[], int id, int
 	if (nn > 0) {
 		sprintf (comment, "End of clip path sub-segment %d", id);
 		ps_comment (comment);
-	}		
+	}
 }
 
 void dump_clip_contours (double *xx, double *yy, int nn, int id, char *file)
@@ -480,9 +521,9 @@ void dump_clip_contours (double *xx, double *yy, int nn, int id, char *file)
 	double out[2];
 	char fname[BUFSIZ];
 	FILE *fp;
-	
+
 	if (nn < 2) return;
-	
+
 	if (file[0] == '-') {	/* Want a single multi-segment file */
 		sprintf (fname, "%s", &file[1]);
 		if (first_dump && (fp = GMT_fopen (fname, GMT_io.w_mode)) == NULL) {
@@ -512,26 +553,26 @@ void dump_clip_contours (double *xx, double *yy, int nn, int id, char *file)
 	GMT_fclose (fp);
 }
 
-int clip_contours (char *grd, int nx, int ny, double west, double north, double dx, double dy, double xinc2, double yinc2, int *edge, int first, double **x, double **y, int *max)
+int clip_contours (char *grd, struct GRD_HEADER *h, double xinc2, double yinc2, int *edge, int first, double **x, double **y, int *max)
 {
 	/* The routine finds the zero-contour in the grd dataset.  it assumes that
 	 * no node has a value exactly == 0.0.  If more than max points are found
 	 * trace_clip_contours will try to allocate more memory in blocks of GMT_CHUNK points
 	 */
 	 
-	 static int i0, j0;
-	 int i, j, ij, n = 0, n_edges, edge_word, edge_bit;
-	 BOOLEAN go_on = TRUE;
+	static int i0, j0;
+	int i, j, ij, n = 0, n_edges, edge_word, edge_bit;
+	BOOLEAN go_on = TRUE;
 	 
 	 
-	 n_edges = ny * (int) ceil (nx / 16.0);
-	 offset = n_edges / 2;
+	n_edges = h->ny * (int) ceil (h->nx / 16.0);
+	offset = n_edges / 2;
 	 
 	 /* Reset edge-flags to zero, if necessary */
 	 if (first) {
 	 	i0 = 0;	/* Begin with upper left bin which is i = 0 and j = 1 */
 	 	j0 = 1;
-		p[0] = p[4] = 0;	p[1] = 1;	p[2] = 1 - nx;	p[3] = -nx;
+		p[0] = p[4] = 0;	p[1] = 1;	p[2] = 1 - h->nx;	p[3] = -h->nx;
 		i_off[0] = i_off[2] = i_off[3] = i_off[4] = 0;	i_off[1] =  1;
 		j_off[0] = j_off[1] = j_off[3] = j_off[4] = 0;	j_off[2] = -1;
 		k_off[0] = k_off[2] = k_off[4] = 0;	k_off[1] = k_off[3] = 1;
@@ -539,17 +580,17 @@ int clip_contours (char *grd, int nx, int ny, double west, double north, double
 	 }
 
 	/* Loop over interior boxes */
-		
-	for (j = j0; go_on && j < ny; j++) {
-		ij = i0 + j * nx;
-		for (i = i0; go_on && i < nx-1; i++, ij++) {	/* nx-1 since the last bin starts at nx-2 and ends at nx-1 */
+
+	for (j = j0; go_on && j < h->ny; j++) {
+		ij = i0 + j * h->nx;
+		for (i = i0; go_on && i < h->nx-1; i++, ij++) {	/* nx-1 since the last bin starts at nx-2 and ends at nx-1 */
 			edge_word = ij / 32 + offset;
 			edge_bit = ij % 32;
-			if (!(edge[edge_word] & bit[edge_bit]) && ((grd[ij]+grd[ij-nx]) == 1)) { /* Start tracing contour */
-				*x[0] = west + i*dx + xinc2;
-				*y[0] = north - (j-0.5) * dy - yinc2;
+			if (!(edge[edge_word] & bit[edge_bit]) && ((grd[ij]+grd[ij-h->nx]) == 1)) { /* Start tracing contour */
+				*x[0] = GMT_i_to_x (i, h->x_min, h->x_max, h->x_inc, h->xy_off, h->nx);
+				*y[0] = GMT_j_to_y (j, h->y_min, h->y_max, h->y_inc, h->xy_off, h->ny);
 				edge[edge_word] |= bit[edge_bit];
-				n = trace_clip_contours (grd, nx, edge, west, north, dx, dy, xinc2, yinc2, x, y, i, j, 3, max);
+				n = trace_clip_contours (grd, edge, h, xinc2, yinc2, x, y, i, j, 3, max);
 				go_on = FALSE;
 				i0 = i + 1;
 				j0 = j;	/* Return to finish this row later */
@@ -557,69 +598,69 @@ int clip_contours (char *grd, int nx, int ny, double west, double north, double
 		}
 		if (go_on) i0 = 0;	/* Go to start of next row unless we found something */
 	}
-	
+
 	return (n);
 }
 
-int trace_clip_contours (char *grd, int nx, int *edge, double west, double north, double dx, double dy, double xinc2, double yinc2, double **xx, double **yy, int i, int j, int kk, int *max)
+int trace_clip_contours (char *grd, int *edge, struct GRD_HEADER *h, double xinc2, double yinc2, double **xx, double **yy, int i, int j, int kk, int *max)
 {
 	int n = 1, k, k0, ij, ij0, n_cuts, kk_opposite, first_k, more;
 	int edge_word, edge_bit, m;
 	double xk[4], yk[4], x0, y0;
-	
+
 	m = *max - 2;
 	
 	more = TRUE;
 	do {
-		ij = i + j * nx;
-		x0 = west + i * dx + xinc2;
-		y0 = north - j * dy - yinc2;
+		ij = i + j * h->nx;
+		x0 = GMT_i_to_x (i, h->x_min, h->x_max, h->x_inc, h->xy_off, h->nx);
+		y0 = GMT_j_to_y (j, h->y_min, h->y_max, h->y_inc, h->xy_off, h->ny);
 		n_cuts = 0;
 		k0 = kk;
 
 		for (k = 0; k < 4; k++) {	/* Loop over box sides */
 
 			/* Skip where we already have a cut (k == k0) */
-			
+
 			if (k == k0) continue;
-			
+
 			/* Skip edge already has been used */
-			
+
 			ij0 = IJ (i + i_off[k], j + j_off[k]);
 			edge_word = ij0 / 32 + k_off[k] * offset;
 			edge_bit = ij0 % 32;
 			if (edge[edge_word] & bit[edge_bit]) continue;
-			
+
 			/* Skip if no zero-crossing on this edge */
-			
+
 			if ((grd[ij+p[k+1]] + grd[ij+p[k]]) != 1) continue;
-			
+
 			/* Here we have a crossing */
-			
+
 			if (k%2) {	/* Cutting a S-N line */
 				if (k == 1) {
-					xk[1] = x0 + dx;
-					yk[1] = y0 + 0.5*dy;
+					xk[1] = x0 + h->x_inc;
+					yk[1] = y0 + 0.5*h->y_inc;
 				}
 				else {
 					xk[3] = x0;
-					yk[3] = y0 + 0.5*dy;
+					yk[3] = y0 + 0.5*h->y_inc;
 				}
 			}
 			else {	/* Cutting a E-W line */
 				if (k == 0) {
-					xk[0] = x0 + 0.5*dx;
+					xk[0] = x0 + 0.5*h->x_inc;
 					yk[0] = y0;
 				}
 				else {
-					xk[2] = x0 + 0.5*dx;
-					yk[2] = y0 + dy;
+					xk[2] = x0 + 0.5*h->x_inc;
+					yk[2] = y0 + h->y_inc;
 				}
 			}
 			kk = k;
 			n_cuts++;
 		}
-		
+
 		if (n > m) {	/* Must try to allocate more memory */
 			*max += GMT_CHUNK;
 			m += GMT_CHUNK;
@@ -627,14 +668,14 @@ int trace_clip_contours (char *grd, int nx, int *edge, double west, double north
 			*yy = (double *) GMT_memory ((void *)*yy, (size_t)(*max), sizeof (double), "trace_clip_contours");
 		}
 		if (n_cuts == 0) {	/* Close interior contour and return */
-			if (fmod ((*xx[0] - xinc2), dx) == 0.0)	/* On side 1 or 3 */
+			if (fmod ((*xx[0] - xinc2), h->x_inc) == 0.0)	/* On side 1 or 3 */
 				first_k = ((*xx[0] - x0) == 0.0) ? 3 : 1;
 			else 	/* On side 0 or 2 */
 				first_k = ((*yy[0] - y0) == 0.0) ? 0 : 2;
 			kk_opposite = (first_k + 2) % 4;
 			if (k0 != kk_opposite) {
-				(*xx)[n] = x0 + 0.5*dx;
-				(*yy)[n] = y0 + 0.5*dy;
+				(*xx)[n] = x0 + 0.5*h->x_inc;
+				(*yy)[n] = y0 + 0.5*h->y_inc;
 				n++;
 			}
 			(*xx)[n] = (*xx)[0];
@@ -646,8 +687,8 @@ int trace_clip_contours (char *grd, int nx, int *edge, double west, double north
 			/* Add center of box if this and previous cut NOT on opposite edges */
 			kk_opposite = (k0 + 2) % 4;
 			if (kk != kk_opposite) {
-				(*xx)[n] = x0 + 0.5*dx;
-				(*yy)[n] = y0 + 0.5*dy;
+				(*xx)[n] = x0 + 0.5*h->x_inc;
+				(*yy)[n] = y0 + 0.5*h->y_inc;
 				n++;
 			}
 			(*xx)[n] = xk[kk];
@@ -657,8 +698,8 @@ int trace_clip_contours (char *grd, int nx, int *edge, double west, double north
 		else {	/* Saddle point, we decide to connect to the point nearest previous point */
 			kk = (k0 + 1)%4;	/* Pick next edge since it is arbitrarily where we go */
 			/* First add center of box */
-			(*xx)[n] = x0 + 0.5*dx;
-			(*yy)[n] = y0 + 0.5*dy;
+			(*xx)[n] = x0 + 0.5*h->x_inc;
+			(*yy)[n] = y0 + 0.5*h->y_inc;
 			n++;
 			(*xx)[n] = xk[kk];
 			(*yy)[n] = yk[kk];
@@ -668,15 +709,15 @@ int trace_clip_contours (char *grd, int nx, int *edge, double west, double north
 			ij0 = IJ (i + i_off[kk], j + j_off[kk]);
 			edge_word = ij0 / 32 + k_off[kk] * offset;
 			edge_bit = ij0 % 32;
-			edge[edge_word] |= bit[edge_bit];	
+			edge[edge_word] |= bit[edge_bit];
 		}
-		
+
 		/* Get next box (i,j,kk) */
-		
+
 		i -= (kk-2)%2;
 		j -= (kk-1)%2;
 		kk = (kk+2)%4;
-		
+
 	} while (more);
 	return (n);
 }
@@ -688,7 +729,7 @@ void shrink_clip_contours (double *x, double *y, int n, double w, double e)
 
 	for (i = 0; i < n; i++) {
 		if (x[i] < w)
-			x[i] = w;	
+			x[i] = w;
 		if (x[i] > e)
 			x[i] = e;
 		if (y[i] < project_info.s)
