@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: psxy.c,v 1.69 2004/09/02 18:15:23 pwessel Exp $
+ *	$Id: psxy.c,v 1.80 2005/12/17 05:59:22 pwessel Exp $
  *
- *	Copyright (c) 1991-2004 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2006 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -29,6 +29,7 @@
  */
 
 #include "gmt.h"
+#include "pslib.h"
 
 #define NONE		99
 #define UNSPECIFIED	999
@@ -44,7 +45,7 @@
 #define HEXAGON		9
 #define ITRIANGLE	10
 #define ELLIPSE		11
-#define VECTOR		12	
+#define VECTOR		12
 #define VECTOR2		13
 #define TEXT		14
 #define PIE		15
@@ -87,6 +88,7 @@ struct PSXY_SYMBOL {
 	int u;			/* Measure unit id (0 = cm, 1 = inch, 2 = m, 3 = point */
 	double size;		/* Current symbol size */
 	double given_size;	/* Symbol size read from file or command line */
+	BOOLEAN equal_area;	/* Symbol should be scaled to give same area as circle */
 	BOOLEAN read_size;	/* TRUE when we must read symbol size from file */
 	int font_no;		/* font to use for the -Sl symbol */
 
@@ -108,11 +110,11 @@ struct PSXY_SYMBOL {
 	int v_just;		/* How to justify vector: head point given (3), head (2), center(1), tail (0 - Default) */
 	BOOLEAN v_double_heads;		/* If TRUE, Add 8 (|= 8) to outline to specify double-headed vector (FALSE is single-headed) */
 
-	char string[64];	/* Character code to plot (could be octal) */
+	char string[GMT_TEXT_LEN];	/* Character code to plot (could be octal) */
 
 	struct FRONTLINE f;	/* parameters needed for a front */
 	struct CUSTOM_SYMBOL *custom;	/* pointer to a custom symbol */
-	
+
 	struct GMT_CONTOUR G;	/* For labelled lines */
 } S;
 
@@ -128,45 +130,46 @@ void plot_x_whiskerbar (double x, double y, double hinge[], double error_width,
 void plot_y_whiskerbar (double x, double y, double hinge[], double error_width, double error_width2, int rgb[], int line);
 void extract_label (char *line, char *label);
 
-main (int argc, char **argv)
+int main (int argc, char **argv)
 {
 
 	int 	i, j, n, j0, n_files = 0, fno, xy_errors[2], two, three, four, error_type[2] = {0,0};
 	int	n_alloc = GMT_CHUNK, plot_n, n_use, start, n_args, n_fields, n_cols_start = 2, this_outline;
-	int 	n_required, n_total_read = 0, n_expected = 0, error_cols[2] = {1,4};
-	
+	int 	n_required, n_total_read = 0, n_expected = 0, reset_cap = 0, error_cols[2] = {1,4};
+
 	BOOLEAN	error = FALSE, nofile = TRUE, polygon = FALSE, read_symbol = FALSE, penset_OK = TRUE;
 	BOOLEAN	outline = FALSE, default_outline = FALSE, draw_arc = TRUE, error_bars = FALSE;
 	BOOLEAN done, get_rgb = FALSE, greenwich, default_polygon = FALSE, resample, xy_offset = FALSE;
 	BOOLEAN old_GMT_world_map, skip_if_outside = TRUE, close_polygon = FALSE, cpt_given = FALSE;
 	BOOLEAN error_x = FALSE, error_y = FALSE, fill_set = FALSE, def_err_xy = FALSE, clip_set = FALSE;
-	
+
 	double west = 0.0, east = 0.0, south = 0.0, north = 0.0, s, c;
 	double plot_x, plot_y, x_1, x_2, y_1, y_2, y0, error_width = 0.1, error_width2;
-	double direction, length, major, minor, x2, y2, z, dir1, dir2, x_len, y_len, dx, dy;
-	double v_w, h_l, h_w, tmp, step = 1.0, *in, font_size, x_offset, y_offset;
-	
-	char line[BUFSIZ], *cpt, *p, *symb_arg = CNULL, txt_a[32], txt_b[32];
-	
+	double direction = 0.0, length = 0.0, major = 0.0, minor = 0.0, x2, y2, z;
+	double dir1 = 0.0, dir2 = 0.0, x_len = 0.0, y_len = 0.0, dx, dy, size;
+	double v_w, h_l, h_w, tmp, step = 1.0, *in, font_size, x_offset = 0.0, y_offset = 0.0;
+
+	char line[BUFSIZ], *cpt = CNULL, *p, *symb_arg = CNULL, txt_a[GMT_LONG_TEXT], txt_b[GMT_LONG_TEXT];
+
 	FILE *fp = NULL;
-	
+
 	struct GMT_PEN pen, epen, default_pen;
 	struct GMT_FILL fill, default_fill, test_fill;
-	
+
 	xy_errors[0] = xy_errors[1] = 0;	/* These will be col # of where to find this info in data */
-	
+
 	argc = GMT_begin (argc, argv);
-	
+
 	GMT_init_pen (&pen, GMT_PENWIDTH);
 	GMT_init_pen (&epen, GMT_PENWIDTH);
 	GMT_init_fill (&fill, -1, -1, -1);	/* Default is no fill */
 	GMT_contlabel_init (&S.G);
-	
+
 	/* Initialize PSXY_SYMBOL structure */
 
 	S.symbol = S.n_required = S.convert_angles = 0;
 	S.size = S.base = S.v_norm = S.v_shrink = 0.0;
-	S.read_size = S.user_unit  = S.shrink = S.read_vector = FALSE;
+	S.equal_area = S.read_size = S.user_unit  = S.shrink = S.read_vector = FALSE;
 	S.v_width = 0.03;
 	memset ((void *)S.string, 0, (size_t)(16 * sizeof (char)));
 	S.h_length = 0.12;
@@ -177,15 +180,15 @@ main (int argc, char **argv)
 		S.v_width = 0.075 / 2.54;	S.h_length = 0.3 / 2.54;
 		S.h_width = 0.25 / 2.54; error_width = 0.25 / 2.54;
 	}
-	
+
 	/* Check and interpret the command line arguments */
-	
+
 	for (i = 1; i < argc; i++) {
 		if (argv[i][0] == '-') {
 			switch(argv[i][1]) {
-		
+
 				/* Common parameters */
-			
+	
 				case 'B':
 				case 'H':
 				case 'J':
@@ -206,16 +209,16 @@ main (int argc, char **argv)
 				case '\0':
 					error += GMT_get_common_args (argv[i], &west, &east, &south, &north);
 					break;
-				
+		
 				/* Supplemental parameters */
-			
+	
 				case 'A':	/* Turn off draw_arc mode */
 					if (argv[i][2])
 						step = atof (&argv[i][2]);	/* Undocumented test feature */
 					else
 						draw_arc = FALSE;
 					break;
-					
+			
 				case 'C':	/* Vary symbol color with z */
 					cpt = &argv[i][2];
 					cpt_given = TRUE;
@@ -295,9 +298,9 @@ main (int argc, char **argv)
 					}
 					default_outline = outline = TRUE;
 					break;
-					
-				/* Illegal options */
 			
+				/* Illegal options */
+	
 				default:		/* Options not recognized */
 					error = TRUE;
 					GMT_default_error (argv[i][1]);
@@ -307,16 +310,16 @@ main (int argc, char **argv)
 		else
 			n_files++;
 	}
-	
+
 	if (argc == 1 || GMT_quick) {	/* Display usage */
 		fprintf (stderr,"psxy %s - Plot lines, polygons, and symbols on maps\n\n", GMT_VERSION);
 		fprintf (stderr,"usage: psxy <infiles> -J<params> -R<west/east/south/north> [-A] [-B<tickinfo>]\n");
 		fprintf (stderr, "\t[-C<cpt>] [-D<dx>/<dy>] [-E[x|y|X|Y][cap][/<pen>]] [-G<fill>] [-H[<nrec>]] [-K] [-L]\n");
 		fprintf (stderr, "\t[-M<flag>] [-N] [-O] [-P] [-S[<symbol>][<size>]] [-U[<label>]] [-V] [-W[<pen>]]\n");
 		fprintf (stderr, "\t[-X<x_shift>] [-Y<y_shift>] [-c<ncopies>] [-:] [-bi[s][<n>]] [-f[i|o]<colinfo>]\n\n");
-		
+
 		if (GMT_quick) exit (EXIT_FAILURE);
-		
+
 		fprintf (stderr, "\t<infiles> is one or more files.  If no, read standard input\n");
 		GMT_explain_option ('j');
 		GMT_explain_option ('R');
@@ -404,9 +407,9 @@ main (int argc, char **argv)
 	}
 
 	/* Check that the options selected are mutually consistent */
-	
+
 	if (S.symbol > 0 && cpt_given) get_rgb = TRUE;	/* Need to read z-vales from input data file */
-	
+
 	if (error_bars && (S.read_vector || S.symbol == ELLIPSE || S.symbol == FRONT || S.symbol == QUOTED_LINE)) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -E option: Incompatible with -Se, -Sf, -Sq, -Sv\n", GMT_program);
 		error++;
@@ -442,22 +445,24 @@ main (int argc, char **argv)
 
 	if (S.symbol == 0 && (fill_set || close_polygon)) polygon = TRUE;
 	if (S.symbol == FRONT || S.symbol == QUOTED_LINE) polygon = FALSE;
-	
+
 	if (S.symbol && !(fill_set || cpt_given)) outline = TRUE;	/* User wants symbol but no fill - implicitly set outline */
-	
+
 	default_pen = pen;
 	default_fill = fill;
 	default_outline = outline;
 	default_polygon = polygon;
 
+	error_width2 = 0.5 * error_width;
+
 	if (error_bars && S.symbol == 0)	/* Assume user only wants error bars */
 		S.symbol = NONE;
-		
+
 	if (cpt_given) {
 		GMT_read_cpt (cpt);
 		if (get_rgb) n_cols_start++;
 	}
-		
+
 	if (n_files > 0)
 		nofile = FALSE;
 	else
@@ -474,7 +479,6 @@ main (int argc, char **argv)
 		if (xy_errors[1]) n_cols_start += error_cols[error_type[1]], error_y = TRUE;
 		xy_errors[0] += (S.read_size + get_rgb);	/* Move 0-2 columns over */
 		xy_errors[1] += (S.read_size + get_rgb);
-		error_width2 = 0.5 * error_width;
 	}
 	two   = (get_rgb) ? 3 : 2;
 	three = (get_rgb) ? 4 : 3;
@@ -504,16 +508,16 @@ main (int argc, char **argv)
 
 	resample = (draw_arc && MAPPING);
 	greenwich = (project_info.w <= 0.0 || project_info.e <= 0.0);
-	
+
 	ps_plotinit (CNULL, gmtdefs.overlay, gmtdefs.page_orientation, gmtdefs.x_origin, gmtdefs.y_origin,
 		gmtdefs.global_x_scale, gmtdefs.global_y_scale, gmtdefs.n_copies,
 		gmtdefs.dpi, GMT_INCH, gmtdefs.paper_width, gmtdefs.page_rgb, gmtdefs.encoding.name, GMT_epsinfo (argv[0]));
 	GMT_echo_command (argc, argv);
 	if (gmtdefs.unix_time) GMT_timestamp (argc, argv);
-	
+
 	if (penset_OK) GMT_setpen (&pen);
 	if (S.symbol == TEXT && fill_set && !outline) ps_setpaint (fill.rgb);
-	
+
 	if (S.symbol == TEXT) {
 		ps_setfont (S.font_no);			/* Set the required font */
 	}
@@ -522,10 +526,16 @@ main (int argc, char **argv)
 		clip_set = TRUE;
 	}
 	if (S.symbol == ELLIPSE) skip_if_outside = FALSE;	/* So we can see ellipses that have centers outside -R */
-	
+	if (S.symbol == POINT && !error_bars) {	/* Temporarily change linecap to round (1)*/
+		ps_setlinecap (1);
+		reset_cap = 1;
+	}
+	else if ((read_symbol || error_bars) && gmtdefs.ps_line_cap != 1) {	/* Must set/rest cap around each ps_point call */
+		reset_cap = 2;
+	}
 	old_GMT_world_map = GMT_world_map;
 	if (S.convert_angles && MAPPING) S.convert_angles = 2;
-	
+
 	if (S.symbol <= 0) {
 		xx = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double), GMT_program);
 		yy = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double), GMT_program);
@@ -557,9 +567,9 @@ main (int argc, char **argv)
 				GMT_fgets (line, BUFSIZ, fp);
 				if (i == 0 && S.G.label_type == 2) extract_label (&line[1], S.G.label);	/* Set initial header as potential label */
 			}
-		}		
+		}
 		if (S.symbol > 0) {	/* symbol part (not counting FRONT and QUOTED_LINE) */
-		
+
 			GMT_world_map = (S.symbol == ELLIPSE && S.convert_angles) ? FALSE : TRUE;
 			if (read_symbol) n_expected = BUFSIZ;
 			while ((n_fields = GMT_input (fp, &n_expected, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {
@@ -602,7 +612,7 @@ main (int argc, char **argv)
 					exit (EXIT_FAILURE);
 				}
 				n_total_read++;
-			
+	
 				if (S.read_size) {
 					S.size = in[two] * GMT_u2u[S.u][GMT_INCH];
 				}
@@ -624,7 +634,7 @@ main (int argc, char **argv)
 					if (S.symbol == CROSS || S.symbol == POINT || S.symbol == TEXT) ps_setpaint (fill.rgb);
 				}
 				if (GMT_cpt_skip) continue;	/* Chosen cpt file indicates skip for this z */
-				
+		
 				if (S.read_vector) {
 					direction = in[two];
 					length = in[three] * GMT_u2u[S.u][GMT_INCH];
@@ -652,14 +662,14 @@ main (int argc, char **argv)
 					x_len = 0.5 * in[two+S.read_size];
 					y_len = 0.5 * in[three+S.read_size];
 				}
-				
+		
 				/* Skip zero-size symbols */
-				
-				if (!(S.symbol == POINT || S.symbol == BAR) && S.symbol < ELLIPSE && S.size <= 0.0) {
+		
+				if (!(S.symbol == BAR) && S.symbol < ELLIPSE && S.size <= 0.0) {
 					if (gmtdefs.verbose) fprintf (stderr, "%s: Warning: Symbol size <= 0.0 near line %d (skipped)\n", GMT_program, n_total_read);
 					continue;
 				}
-				
+		
 				if (S.read_vector && S.v_just < 3 && length <= 0.0) {
 					if (gmtdefs.verbose) fprintf (stderr, "%s: Warning: Vector length <= 0.0 near line %d (skipped)\n", GMT_program, n_total_read);
 					continue;
@@ -708,13 +718,14 @@ main (int argc, char **argv)
 					case NONE:
 						break;
 					case STAR:
-						ps_star (plot_x, plot_y, S.size, fill.rgb, outline);
+						size = (S.equal_area) ? 1.67289326141 * S.size : S.size;
+						ps_star (plot_x, plot_y, size, fill.rgb, outline);
 						break;
 					case BAR:
 						if (S.user_unit) {	/* Width measured in x units */
 							GMT_geo_to_xy (in[0]-S.size, S.base, &x_1, &y_1);
 							GMT_geo_to_xy (in[0]+S.size, in[1], &x_2, &y_2);
-					
+			
 							ps_rect (x_1, y_1, x_2, y_2, fill.rgb, outline);
 						}
 						else
@@ -724,31 +735,40 @@ main (int argc, char **argv)
 						ps_cross (plot_x, plot_y, S.size);
 						break;
 					case POINT:
-						ps_cross (plot_x, plot_y, POINTSIZE);
+						if (reset_cap == 2) ps_setlinecap (1);
+						ps_point (plot_x, plot_y, S.size);
+						if (reset_cap == 2) ps_setlinecap (gmtdefs.ps_line_cap);
 						break;
 					case CIRCLE:
 						ps_circle (plot_x, plot_y, S.size, fill.rgb, outline);
 						break;
 					case SQUARE:
-						ps_square (plot_x, plot_y, S.size, fill.rgb, outline);
+						size = (S.equal_area) ? 1.25331413732 * S.size : S.size;
+						ps_square (plot_x, plot_y, size, fill.rgb, outline);
 						break;
 					case HEXAGON:
-						ps_hexagon (plot_x, plot_y, S.size, fill.rgb, outline);
+						size = (S.equal_area) ? 1.09963611079 * S.size : S.size;
+						ps_hexagon (plot_x, plot_y, size, fill.rgb, outline);
 						break;
 					case PENTAGON:
-						ps_pentagon (plot_x, plot_y, S.size, fill.rgb, outline);
+						size = (S.equal_area) ? 1.14948092619 * S.size : S.size;
+						ps_pentagon (plot_x, plot_y, size, fill.rgb, outline);
 						break;
 					case OCTAGON:
-						ps_octagon (plot_x, plot_y, S.size, fill.rgb, outline);
+						size = (S.equal_area) ? 1.05390736526 * S.size : S.size;
+						ps_octagon (plot_x, plot_y, size, fill.rgb, outline);
 						break;
 					case TRIANGLE:
-						ps_triangle (plot_x, plot_y, S.size, fill.rgb, outline);
+						size = (S.equal_area) ? 1.55512030156 * S.size : S.size;
+						ps_triangle (plot_x, plot_y, size, fill.rgb, outline);
 						break;
 					case ITRIANGLE:
-						ps_itriangle (plot_x, plot_y, S.size, fill.rgb, outline);
+						size = (S.equal_area) ? 1.55512030156 * S.size : S.size;
+						ps_itriangle (plot_x, plot_y, size, fill.rgb, outline);
 						break;
 					case DIAMOND:
-						ps_diamond (plot_x, plot_y, S.size, fill.rgb, outline);
+						size = (S.equal_area) ? 1.25331413732 * S.size : S.size;
+						ps_diamond (plot_x, plot_y, size, fill.rgb, outline);
 						break;
 					case TEXT:
 						font_size = S.size * 72.0;
@@ -899,7 +919,7 @@ main (int argc, char **argv)
 					if (S.G.label_type == 2) {	/* Update segment header */
 						extract_label (&GMT_io.segment_header[1], S.G.label);
 					}
-					
+			
 					n_fields = GMT_input (fp, &n_expected, &in);
 					n_total_read++;
 				}
@@ -931,11 +951,11 @@ main (int argc, char **argv)
 						yy = (double *) GMT_memory ((void *)yy, (size_t)n_alloc, sizeof (double), GMT_program);
 					}
 				}
-		
+
 				xx = (double *) GMT_memory ((void *)xx, (size_t)n, sizeof (double), GMT_program);
 				yy = (double *) GMT_memory ((void *)yy, (size_t)n, sizeof (double), GMT_program);
 				n_alloc = n;
-				
+		
 				if (!GMT_cpt_skip) {
 					if (resample) n = GMT_fix_up_path (&xx, &yy, n, greenwich, step);
 
@@ -954,34 +974,34 @@ main (int argc, char **argv)
 						memcpy ((void *)yp, (void *)GMT_y_plot, (size_t)(plot_n * sizeof (double)));
 						memcpy ((void *)plot_pen, (void *)GMT_pen, (size_t)(plot_n * sizeof (int)));
 					}
-		
+
 					if (polygon) {
-					
+			
 						if ((*GMT_will_it_wrap) (xp, yp, plot_n, &start)) {	/* Polygon wraps */
-					
+			
 							/* First truncate agains left border */
-						
+				
 							GMT_n_plot = (*GMT_truncate) (xp, yp, plot_n, start, -1);
 							n_use = GMT_compact_line (GMT_x_plot, GMT_y_plot, GMT_n_plot, FALSE, 0);
 							GMT_fill (GMT_x_plot, GMT_y_plot, n_use, &fill, outline);
-								
-							/* Then truncate agains right border */
 						
+							/* Then truncate agains right border */
+				
 							GMT_n_plot = (*GMT_truncate) (xp, yp, plot_n, start, +1);
 							n_use = GMT_compact_line (GMT_x_plot, GMT_y_plot, GMT_n_plot, FALSE, 0);
 							GMT_fill (GMT_x_plot, GMT_y_plot, n_use, &fill, outline);
-					
+			
 						}
 						else
 						GMT_fill (xp, yp, plot_n, &fill, outline);
 					}
 					else if (S.symbol == QUOTED_LINE) {
 						S.G.line_pen = pen;
-						GMT_hold_contour (&xp, &yp, plot_n, "N/A", 'A', S.G.label_angle, close_polygon, &S.G);
+						GMT_hold_contour (&xp, &yp, plot_n, 0.0, "N/A", 'A', S.G.label_angle, close_polygon, &S.G);
 					}
 					else
 						GMT_plot_line (xp, yp, plot_pen, plot_n);
-										
+								
 					if (S.symbol == FRONT) GMT_draw_fence (xp, yp, plot_n, &S.f, &fill, outline); 	/* Must draw front crossbars */
 					GMT_free ((void *)xp);
 					GMT_free ((void *)yp);
@@ -991,30 +1011,34 @@ main (int argc, char **argv)
 		}
 		if (fp != GMT_stdin) GMT_fclose (fp);
 	}
-	
+
+	if (reset_cap == 1) ps_setlinecap (gmtdefs.ps_line_cap);	/* Reset linecap to default */
+
 	if (S.symbol == QUOTED_LINE) GMT_contlabel_plot (&S.G);
-	
+
 	if (clip_set) GMT_map_clip_off ();
 
 	GMT_world_map = old_GMT_world_map;
-	
+
 	GMT_map_basemap ();
 
 	ps_plotend (gmtdefs.last_page);
-	
+
 	if (S.symbol == 0) {
 		GMT_free ((void *)xx);
 		GMT_free ((void *)yy);
 	}
-	
+
 	GMT_end (argc, argv);
+
+	exit (EXIT_SUCCESS);
 }
 
 int decode_symbol_option (char *text, struct PSXY_SYMBOL *p)
 {
 	int decode_error = 0, slash, bset = 0, k, j, n, len, one, colon;
 	BOOLEAN check, old_style;
-	char symbol_type, txt_a[32], txt_b[32], txt_c[32], text_cp[128], *c;
+	char symbol_type, txt_a[GMT_LONG_TEXT], txt_b[GMT_LONG_TEXT], txt_c[GMT_LONG_TEXT], text_cp[GMT_LONG_TEXT], *c;
 	p->n_required = 0;	/* columns needed in addition to x & y */
 	p->user_unit = p->convert_angles = p->shrink = p->read_vector = FALSE;
 
@@ -1068,10 +1092,12 @@ int decode_symbol_option (char *text, struct PSXY_SYMBOL *p)
 		n = sscanf (text, "%c", &symbol_type);
 		if (p->size == 0.0) p->size = p->given_size;
 		if (text[1] && (p->u = GMT_get_unit (text[1])) < 0) decode_error = TRUE;
+		p->equal_area = FALSE;
 	}
 	else if (strchr ("-aAbCcDdeEfGgHhIiNnpqrSsTtVvwxy", (int) text[0]) && (text[1] == '\n' || text[1] == '\0')) {	/* Symbol, but no size given (size assumed given as input data) */
 		n = sscanf (text, "%c", &symbol_type);
 		if (p->size == 0.0) p->size = p->given_size;
+		p->equal_area = FALSE;
 	}
 	else if (text[0] == 'b') {	/* Bar with size */
 		for (j = 1, bset = 0; text[j] && !bset; j++) if (text[j] == 'b') bset = j;
@@ -1093,6 +1119,7 @@ int decode_symbol_option (char *text, struct PSXY_SYMBOL *p)
 		n = sscanf (text, "%c%s", &symbol_type, txt_a);
 		symbol_type = text[0];
 		p->given_size = p->size = GMT_convert_units (txt_a, GMT_INCH);
+		p->equal_area = FALSE;
 	}
 
 	check = TRUE;
@@ -1105,7 +1132,7 @@ int decode_symbol_option (char *text, struct PSXY_SYMBOL *p)
 			p->size *= 0.5;		/* We will use +- to get full width */
 			break;
 		case 'A':
-			p->size *= 1.6728932614;	/* To equal area of circle with same size */
+			p->equal_area = TRUE;	/* To equal area of circle with same size */
 		case 'a':
 			p->symbol = STAR;
 			break;
@@ -1119,7 +1146,7 @@ int decode_symbol_option (char *text, struct PSXY_SYMBOL *p)
 			p->symbol = CIRCLE;
 			break;
 		case 'D':
-			p->size *= 1.2533141373;	/* To equal area of circle with same size */
+			p->equal_area = TRUE;	/* To equal area of circle with same size */
 		case 'd':
 			p->symbol = DIAMOND;
 			break;
@@ -1141,7 +1168,7 @@ int decode_symbol_option (char *text, struct PSXY_SYMBOL *p)
 				*c = 0;	/* Effectively chops off the offset modifier */
 			}
 			len = strlen (text_cp) - 1;
-			
+	
 			old_style = FALSE;
 			switch (text_cp[len]) {
 				case 'f':	/* Fault front */
@@ -1164,9 +1191,9 @@ int decode_symbol_option (char *text, struct PSXY_SYMBOL *p)
 					p->f.f_symbol = F_BOX;
 					len--;
 					break;
-					
+			
 				/* Old style (backward compatibility) */
-				
+		
 				case 'L':	/* Left triangle */
 					p->f.f_symbol = F_TRIANGLE;
 				case 'l':	/* Left ticked fault */
@@ -1204,20 +1231,20 @@ int decode_symbol_option (char *text, struct PSXY_SYMBOL *p)
 			}
 
 			text_cp[len] = 0;	/* Gets rid of the [dir][type] flags, if present */
-			
+	
 			/* Pull out and get spacing and size */
-			
+	
 			sscanf (&text_cp[1], "%[^/]/%s", txt_a, txt_b);
 			p->f.f_gap = GMT_convert_units (txt_a, GMT_INCH);
 			p->f.f_len = GMT_convert_units (txt_b, GMT_INCH);
 			break;
 		case 'H':
-			p->size *= 1.09963611078;	/* To equal area of circle with same size */
+			p->equal_area = TRUE;	/* To equal area of circle with same size */
 		case 'h':
 			p->symbol = HEXAGON;
 			break;
 		case 'I':
-			p->size *= 1.55512030155;	/* To equal area of circle with same size */
+			p->equal_area = TRUE;	/* To equal area of circle with same size */
 		case 'i':
 			p->symbol = ITRIANGLE;
 			break;
@@ -1237,17 +1264,18 @@ int decode_symbol_option (char *text, struct PSXY_SYMBOL *p)
 			p->string[k] = 0;
 			break;
 		case 'N':
-			p->size *= 1.32130639968;	/* To equal area of circle with same size */
+			p->equal_area = TRUE;	/* To equal area of circle with same size */
 		case 'n':
 			p->symbol = PENTAGON;
 			break;
 		case 'G':
-			p->size *= 0.94805944897;	/* To equal area of circle with same size */
+			p->equal_area = TRUE;	/* To equal area of circle with same size */
 		case 'g':
 			p->symbol = OCTAGON;
 			break;
 		case 'p':
 			p->symbol = POINT;
+			if (p->size == 0.0) p->size = POINTSIZE;
 			check = FALSE;
 			break;
 		case 'q':	/* Quoted lines: -Sq[d|n|l|x]<info>[:<labelinfo>] */
@@ -1263,12 +1291,12 @@ int decode_symbol_option (char *text, struct PSXY_SYMBOL *p)
 			check = FALSE;
 			break;
 		case 'S':
-			p->size *= 1.25331413731;	/* To equal area of circle with same size */
+			p->equal_area = TRUE;	/* To equal area of circle with same size */
 		case 's':
 			p->symbol = SQUARE;
 			break;
 		case 'T':
-			p->size *= 1.55512030155;	/* To equal area of circle with same size */
+			p->equal_area = TRUE;	/* To equal area of circle with same size */
 		case 't':
 			p->symbol = TRIANGLE;
 			break;
@@ -1416,17 +1444,17 @@ void GMT_plot_ellipse (double lon, double lat, double major, double minor, doubl
 
 		x_prime = x * cos_azimuth - y * sin_azimuth;
 		y_prime = x * sin_azimuth + y * cos_azimuth;
-		
+
 		/* Convert m back to lon lat */
 
 		rho = hypot (x_prime, y_prime);
-	
+
 		c = rho / project_info.EQ_RAD;
 		sincos (c, &sin_c, &cos_c);
 		py[i] = d_asin (cos_c * sinp + (y_prime * sin_c * cosp / rho)) * R2D;
-		if ((lat - 90.0) > -GMT_CONV_LIMIT)
+		if ((lat - 90.0) > -GMT_CONV_LIMIT)	/* origin in Northern hemisphere */
 			px[i] = lon + R2D * d_atan2 (x_prime, -y_prime);
-		else if ((lat + 90.0) < GMT_CONV_LIMIT)
+		else if ((lat + 90.0) < GMT_CONV_LIMIT)	/* origin in SOuthern hemisphere */
 			px[i] = lon + R2D * d_atan2 (x_prime, y_prime);
 		else
 			px[i] = lon +
@@ -1442,19 +1470,19 @@ void GMT_plot_ellipse (double lon, double lat, double major, double minor, doubl
 	memcpy ((void *)GMT_y_plot, (void *)yp, (size_t)(plot_n * sizeof (double)));
 
 	if ((*GMT_will_it_wrap) (xp, yp, plot_n, &start)) {	/* Polygon wraps */
-					
+			
 		/* First truncate agains left border */
-						
+				
 		GMT_n_plot = (*GMT_truncate) (xp, yp, plot_n, start, -1);
 		n_use = GMT_compact_line (GMT_x_plot, GMT_y_plot, GMT_n_plot, FALSE, 0);
 		GMT_fill (GMT_x_plot, GMT_y_plot, n_use, &fill, outline);
-								
-		/* Then truncate agains right border */
 						
+		/* Then truncate agains right border */
+				
 		GMT_n_plot = (*GMT_truncate) (xp, yp, plot_n, start, +1);
 		n_use = GMT_compact_line (GMT_x_plot, GMT_y_plot, GMT_n_plot, FALSE, 0);
 		GMT_fill (GMT_x_plot, GMT_y_plot, n_use, &fill, outline);
-						
+				
 	}
 	else
 		GMT_fill (xp, yp, plot_n, &fill, outline);
@@ -1469,9 +1497,9 @@ void GMT_draw_fence (double x[], double y[], int n, struct FRONTLINE *f, struct
 {
 	int i, ngap;
 	BOOLEAN skip;
-	double *s, xx[4], yy[4], dist, w, frac, dx, dy, angle, dir1, dir2;
+	double *s, xx[4], yy[4], dist = 0.0, w, frac, dx, dy, angle, dir1, dir2;
 	double gap, x0, y0, len2, len3, cosa, sina, sa, ca, offx, offy;
-	
+
 	if (n < 2) return;
 
 	s = (double *) GMT_memory (VNULL, (size_t)n, sizeof (double), GMT_program);
@@ -1482,7 +1510,7 @@ void GMT_draw_fence (double x[], double y[], int n, struct FRONTLINE *f, struct
 		if (GMT_world_map && dx > w) dx = copysign (2 * w - fabs (dx), -dx);
 		s[i] = s[i-1] + hypot (dx, y[i] - y[i-1]);
 	}
-		
+
 	if (f->f_gap > 0.0) {	/* Gap distance given, calculate number of them and round off gap distance */
 		ngap = irint (s[n-1] / f->f_gap);
 		gap = s[n-1] / (double)ngap;
@@ -1494,11 +1522,11 @@ void GMT_draw_fence (double x[], double y[], int n, struct FRONTLINE *f, struct
 		gap = s[n-1] / (ngap - 1);
 		if (ngap == 1) dist = 0.5 * s[n-1];
 	}
-	
+
 	len2 = 0.5 * f->f_len;
-	len3 = 0.866025404 * f->f_len;	
+	len3 = 0.866025404 * f->f_len;
 	if (f->f_sense == F_CENTERED) len3 = len2;
-	
+
 	i = 0;
 	while (i < n) {
 		while ((s[i] - dist) > -SMALL) {	/* Time for tick */
@@ -1526,9 +1554,9 @@ void GMT_draw_fence (double x[], double y[], int n, struct FRONTLINE *f, struct
 				i++;
 				continue;
 			}
-			
+	
 			switch (f->f_symbol) {
-				case F_TRIANGLE:	/* Triangle */	
+				case F_TRIANGLE:	/* Triangle */
 					switch (f->f_sense) {
 						case F_CENTERED:
 							sincos (angle, &sina, &cosa);
@@ -1556,8 +1584,8 @@ void GMT_draw_fence (double x[], double y[], int n, struct FRONTLINE *f, struct
 							break;
 					}
 					break;
-				
-				case F_CIRCLE:	/* Circles */	
+		
+				case F_CIRCLE:	/* Circles */
 					switch (f->f_sense) {
 						case F_CENTERED:
 							ps_circle (x0, y0, f->f_len, g->rgb, outline);
@@ -1573,8 +1601,8 @@ void GMT_draw_fence (double x[], double y[], int n, struct FRONTLINE *f, struct
 							break;
 					}
 					break;
-				
-				case F_BOX:	/* Squares */	
+		
+				case F_BOX:	/* Squares */
 					switch (f->f_sense) {
 						case F_CENTERED:	/* Full square centered on line */
 							sincos (angle, &sina, &cosa);
@@ -1604,7 +1632,7 @@ void GMT_draw_fence (double x[], double y[], int n, struct FRONTLINE *f, struct
 					}
 					ps_patch (xx, yy, 4, g->rgb, outline);
 					break;
-				
+		
 				case F_SLIP: /* draw strike-slip arrows */
 					sincos (angle, &sina, &cosa);
 				        offx = gmtdefs.annot_offset[0] * sina; /* get offsets from front line */
@@ -1638,8 +1666,8 @@ void GMT_draw_fence (double x[], double y[], int n, struct FRONTLINE *f, struct
 					yy[3] = yy[2] - len2 * sa;
 					ps_plot (xx[3], yy[3], -2); 
 					break;
-					
-				case F_FAULT:	/* Normal fault ticks */	
+			
+				case F_FAULT:	/* Normal fault ticks */
 					xx[0] = xx[1] = x0;	yy[0] = yy[1] = y0;
 					if (f->f_sense == F_CENTERED) {
 						angle -= M_PI_2;
@@ -1668,7 +1696,7 @@ void GMT_draw_fence (double x[], double y[], int n, struct FRONTLINE *f, struct
 
 void plot_x_errorbar (double x, double y, double delta_x, double error_width, double error_width2, int line) {
 	double x_1, x_2, y_1, y_2;
-	
+
 	GMT_geo_to_xy (x - delta_x, y, &x_1, &y_1);
 	GMT_geo_to_xy (x + delta_x, y, &x_2, &y_2);
 	if (GMT_is_dnan (x_1)) {
@@ -1716,7 +1744,7 @@ void plot_x_whiskerbar (double x, double y, double hinge[], double error_width,
 	int i;
 	static int q[4] = {0, 25, 75, 100};
 	double xx[4], yy[4];
-	
+
 	for (i = 0; i < 4; i++) {	/* for 0, 25, 75, 100% hinges */
 		GMT_geo_to_xy (hinge[i], y, &xx[i], &yy[i]);
 		if (GMT_is_dnan (xx[i])) {
@@ -1726,19 +1754,19 @@ void plot_x_whiskerbar (double x, double y, double hinge[], double error_width,
 	}
 	yy[1] -= error_width2;
 	yy[2] += error_width2;
-	
+
 	ps_plot (xx[0], yy[1], 3);		/* Left whisker */
 	ps_plotr (0.0, error_width, 2);
 	ps_plot (xx[0], yy[0], 3);
 	ps_plot (xx[1], yy[0], -2);
-	
+
 	ps_plot (xx[3], yy[1], 3);		/* Right whisker */
 	ps_plotr (0.0, error_width, 2);
 	ps_plot (xx[3], yy[0], 3);
 	ps_plot (xx[2], yy[0], -2);
 
 	ps_rect (xx[1], yy[1], xx[2], yy[2], rgb, TRUE);	/* Main box */
-	
+
 	ps_plot (x, yy[1], 3);			/* Median line */
 	ps_plot (x, yy[2], -2);
 }
@@ -1747,7 +1775,7 @@ void plot_y_whiskerbar (double x, double y, double hinge[], double error_width,
 	int i;
 	static int q[4] = {0, 25, 75, 100};
 	double xx[4], yy[4];
-	
+
 	for (i = 0; i < 4; i++) {	/* for 0, 25, 75, 100% hinges */
 		GMT_geo_to_xy (x, hinge[i], &xx[i], &yy[i]);
 		if (GMT_is_dnan (yy[i])) {
@@ -1757,19 +1785,19 @@ void plot_y_whiskerbar (double x, double y, double hinge[], double error_width,
 	}
 	xx[1] -= error_width2;
 	xx[2] += error_width2;
-	
+
 	ps_plot (xx[1], yy[0], 3);		/* bottom whisker */
 	ps_plotr (error_width, 0.0, 2);
 	ps_plot (xx[0], yy[0], 3);
 	ps_plot (xx[0], yy[1], -2);
-	
+
 	ps_plot (xx[1], yy[3], 3);		/* Top whisker */
 	ps_plotr (error_width, 0.0, 2);
 	ps_plot (xx[0], yy[3], 3);
 	ps_plot (xx[0], yy[2], -2);
 
 	ps_rect (xx[2], yy[2], xx[1], yy[1], rgb, TRUE);	/* Main box */
-	
+
 	ps_plot (xx[1], y, 3);			/* Median line */
 	ps_plot (xx[2], y, -2);
 }
@@ -1778,12 +1806,12 @@ void extract_label (char *line, char *label)
 {
 	int i = 0, j, j0;
 	char *p;
-	
+
 	if ((p = strstr (line, " -L")) || (p = strstr (line, "	-L"))) 	/* Get label specified wih -L option */
 		i = p + 3 - line;
 	else {								/* Bypass whitespace and pick first word */
 		while (line[i] && (line[i] == ' ' || line[i] == '\t')) i++;
-		
+
 	}
 	if ((p = strchr (&line[i], '\"'))) {	/* Gave several words as label */
 		for (j0 = i, j = i + 1; line[j] != '\"'; j++);
