@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: spectrum1d.c,v 1.13 2004/04/25 09:10:46 pwessel Exp $
+ *	$Id: spectrum1d.c,v 1.19 2005/12/17 05:59:22 pwessel Exp $
  *
- *	Copyright (c) 1991-2004 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2006 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -57,7 +57,7 @@
  *			BCH-J: Upgraded -C: optional output flags
  *	References:	Julius S. Bendat & Allan G. Piersol,
  *			"Random Data", 2nd revised edition, 566pp.,
- *			1986, John Wiley & Sons, New York.
+ *			1986, John Wiley & Sons, New York. [ B&P below]
  *
  *			Peter D. Welch, "The use of Fast Fourier
  *			Transform for the estimation of power spectra:
@@ -99,24 +99,24 @@ void	detrend_and_hanning(void);
 void	write_output(char *output_choice, int n_outputs);
 void	free_space(void);
 
-main (int argc, char **argv)
+int main (int argc, char **argv)
 {
 
-	int	i, j, n_outputs, window_test, error = FALSE;
+	int	i, j, n_outputs, window_test = 2, error = FALSE;
 	char	output_choice[N_OUTPUT_CHOICES];
 
 	argc = GMT_begin (argc, argv);
 
-	n_outputs = 0;	
+	n_outputs = 0;
 	for (i=0; i < N_OUTPUT_CHOICES; i++) output_choice[i] = 0;
 	sprintf(namestem, "spectrum");
 
 	for (i = 1; i < argc; i++) {
 		if (argv[i][0] == '-') {
 			switch (argv[i][1]) {
-		
+
 				/* Common parameters */
-			
+
 				case 'H':
 				case 'V':
 				case 'b':
@@ -124,9 +124,9 @@ main (int argc, char **argv)
 				case '\0':
 					error += GMT_get_common_args (argv[i], 0, 0, 0, 0);
 					break;
-				
+
 				/* Supplemental parameters */
-				
+
 				case 'C':
 					y_given = TRUE;
 					j = 2;
@@ -184,7 +184,6 @@ main (int argc, char **argv)
 					break;
 				case 'S':
 					window = atoi(&argv[i][2]);
-					window_test = 2;
 					while (window_test < window) {
 						window_test += window_test;
 					}
@@ -210,9 +209,9 @@ main (int argc, char **argv)
 		fprintf (stderr, "spectrum1d %s - compute auto- [and cross- ] spectra from one [or two] timeseries\n\n", GMT_VERSION);
 		fprintf(stderr,"usage:  spectrum1d -S<segment_size> [-C[<xycnpago>]] [-D<dt>] [-H[<nrec>]] [-N<name_stem>]\n");
 		fprintf(stderr,"\t[-V] [-W] [-bi[s][<n>]] [-bo[s][<n>]] [-f[i|o]<colinfo>]\n\n");
-		
+
 		if (GMT_quick) exit (EXIT_FAILURE);
-		
+
 		fprintf(stderr,"\t-S Use data subsets of <segment_size> elements.\n");
 		fprintf(stderr,"\t   <segment_size> must be radix 2;\n");
 		fprintf(stderr,"\t   std. err. = 1/sqrt(n_data/segment_size).\n");
@@ -263,7 +262,7 @@ main (int argc, char **argv)
 	if (error) exit (EXIT_FAILURE);
 
 	GMT_put_history (argc, argv);	/* Update .gmtcommands4 */
-	
+
 	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
 		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
@@ -289,6 +288,8 @@ main (int argc, char **argv)
 	free_space();
 
 	GMT_end (argc, argv);
+
+	exit (EXIT_SUCCESS);
 }
 
 void	alloc_arrays(void)
@@ -348,7 +349,7 @@ void	compute_spectra(void)
 	int	n_windows, w, i, t_start, t_stop, t, f;
 	int	narray;
 	float	work = 0.0;
-	double	dw, spec_scale, x_varp, y_varp, one_on_nw, co_quad;
+	double	dw, spec_scale, x_varp, y_varp = 1.0, one_on_nw, co_quad;
 	double	xreal, ximag, yreal, yimag, xpower, ypower, co_spec, quad_spec;
 
 	narray = window;
@@ -460,9 +461,9 @@ void	write_output(char *output_choice, int n_outputs)
 {
 	int	i, j;
 	double	delta_f, eps_pow, out[3], *f_or_w;
-	char	fname[128];
+	char	fname[GMT_LONG_TEXT];
 	FILE	*fpout;
-	
+
 	delta_f = 1.0 / (window * dt);
 	eps_pow = 1.0 / sqrt(d_n_windows);	/* Multiplicative error bars for power spectra  */
 
@@ -472,7 +473,7 @@ void	write_output(char *output_choice, int n_outputs)
 	/* loop through output choices */
 	for (j = 0; j < n_outputs; j++) {
 	  switch (output_choice[j]) {
-	  case 'x':		/* write x power */
+	  case 'x':		/* write x power [ B&P 2nd Ed. eqn. 9.32 ] */
 	    sprintf(fname, "%s.xpower", namestem);
 	    if ( (fpout = GMT_fopen(fname, GMT_io.w_mode)) == NULL) {
 	      fprintf(stderr,"%s:  Cannot open w %s\n", GMT_program, fname);
@@ -488,7 +489,7 @@ void	write_output(char *output_choice, int n_outputs)
 	    GMT_fclose(fpout);
 	    break;
 
-	  case 'y':		/* Write y power  */
+	  case 'y':		/* Write y power [ B&P 2nd Ed. eqn. 9.32 ] */
 	    sprintf(fname, "%s.ypower", namestem);
 	    if ( (fpout = GMT_fopen(fname, GMT_io.w_mode)) == NULL) {
 	      fprintf(stderr,"%s:  Cannot open w %s\n", GMT_program, fname);
@@ -503,7 +504,7 @@ void	write_output(char *output_choice, int n_outputs)
 	    }
 	    GMT_fclose(fpout);
 	    break;
-	  case 'c':		/* Write Coherent Output power  */
+	  case 'c':		/* Write Coherent Output power [ B&P 2nd Ed. eqn. 9.71 ] */
 	    sprintf(fname, "%s.cpower", namestem);
 	    if ( (fpout = GMT_fopen(fname, GMT_io.w_mode)) == NULL) {
 	      fprintf(stderr,"%s:  Cannot open w %s\n", GMT_program, fname);
@@ -518,7 +519,7 @@ void	write_output(char *output_choice, int n_outputs)
 	    }
 	    GMT_fclose(fpout);
 	    break;
-	  case 'n':		/* Write Noise Output power  */
+	  case 'n':		/* Write Noise Output power [ B&P 2nd Ed. eqn. 9.73 & Table 9.6 ] */
 	    sprintf(fname, "%s.npower", namestem);
 	    if ( (fpout = GMT_fopen(fname, GMT_io.w_mode)) == NULL) {
 	      fprintf(stderr,"%s:  Cannot open w %s\n", GMT_program, fname);
@@ -533,7 +534,7 @@ void	write_output(char *output_choice, int n_outputs)
 	    }
 	    GMT_fclose(fpout);
 	    break;
-	  case 'g':		/* Write Gain spectrum  */
+	  case 'g':		/* Write Gain spectrum [ B&P 2nd Ed. eqn. 9.90 & Table 9.6 ] */
 	    sprintf(fname, "%s.gain", namestem);
 	    if ( (fpout = GMT_fopen(fname, GMT_io.w_mode)) == NULL) {
 	      fprintf(stderr,"%s:  Cannot open w %s\n", GMT_program, fname);
@@ -543,12 +544,20 @@ void	write_output(char *output_choice, int n_outputs)
 	    for (i = 0; i < n_spec; i++) {
 	      out[0] = f_or_w[i];
 	      out[1] = spec[i].gain;
-	      out[2] = out[1] * eps_pow * sqrt( (1.0 - spec[i].coh) / 2.0 * spec[i].coh);
+	      out[2] = out[1] * eps_pow * sqrt( (1.0 - spec[i].coh) / (2.0 * spec[i].coh) );
 	      GMT_output (fpout, 3, out);
 	    }
 	    GMT_fclose(fpout);
 	    break;
 	  case 'a':		/* Write Real Admittance spectrum  */
+	    /*   We don't know the correct error estimate and it is not
+		in Bendat and Piersol, or Priestly, or any standard text.
+		Smith needs to derive this, and should make a note to
+		check the expression given by Marcia Maia et al in Geophys. 
+		J. Int, 100, 337-348, 1990, equation 10, page 341.
+		Meanwhile we will default to use the expression related to
+		that for the gain spectrum:
+	    */
 	    sprintf(fname, "%s.admit", namestem);
 	    if ( (fpout = GMT_fopen(fname, "w")) == NULL) {
 	      fprintf(stderr,"%s:  Cannot open w %s\n", GMT_program, fname);
@@ -558,12 +567,12 @@ void	write_output(char *output_choice, int n_outputs)
 	    for (i = 0; i < n_spec; i++) {
 	      out[0] = f_or_w[i];
 	      out[1] = spec[i].radmit;
-	      out[2] = fabs (eps_pow * sqrt( (1.0 - spec[i].coh) / 2.0 * spec[i].coh) * out[1]);
+	      out[2] = fabs (eps_pow * sqrt( (1.0 - spec[i].coh) / (2.0 * spec[i].coh) ) * out[1]);
 	      GMT_output (fpout, 3, out);
 	    }
 	    GMT_fclose(fpout);
 	    break;
-	  case 'p':		/* Write Phase spectrum  */
+	  case 'p':		/* Write Phase spectrum [ B&P 2nd Ed. eqn. 9.91 & Table 9.6 ] */
 	    sprintf(fname, "%s.phase", namestem);
 	    if ( (fpout = GMT_fopen(fname, GMT_io.w_mode)) == NULL) {
 	      fprintf(stderr,"%s:  Cannot open w %s\n", GMT_program, fname);
@@ -573,11 +582,11 @@ void	write_output(char *output_choice, int n_outputs)
 	    for (i = 0; i < n_spec; i++) {
 	      out[0] = f_or_w[i];
 	      out[1] = spec[i].phase;
-	      out[2] = eps_pow * sqrt( (1.0 - spec[i].coh) / 2.0 * spec[i].coh);
+	      out[2] = eps_pow * sqrt( (1.0 - spec[i].coh) / (2.0 * spec[i].coh) );
 	      GMT_output (fpout, 3, out);
 	    }
 	    GMT_fclose(fpout);
-	  case 'o':		/* Write Coherency spectrum  */
+	  case 'o':		/* Write Coherency spectrum [ B&P 2nd Ed. eqn. 9.82 ] */
 	    sprintf(fname, "%s.coh", namestem);
 	    if ( (fpout = GMT_fopen(fname, GMT_io.w_mode)) == NULL) {
 	      fprintf(stderr,"%s:  Cannot open w %s\n", GMT_program, fname);
@@ -596,7 +605,6 @@ void	write_output(char *output_choice, int n_outputs)
 	}
 
 	GMT_free ((void *)f_or_w);
-
 }
 
 void	free_space (void)
