@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: surface.c,v 1.18 2004/04/25 09:10:46 pwessel Exp $
+ *	$Id: surface.c,v 1.24 2005/12/17 05:59:22 pwessel Exp $
  *
- *	Copyright (c) 1991-2004 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2006 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -77,6 +77,7 @@
  * 24-JAN-2000 PW/WHFS: Fixed bug related to use of -L option for "near-node" constraints.
  * 10-JUL-2000 3.3.5 PW: Added -L plain.
  * Version:	4
+   Version:	4.1: 14-SEP-2005 by PW, Added enhanced -I
  *
  *
  */
@@ -89,15 +90,15 @@ int n_alloc = GMT_CHUNK;
 int npoints=0;			/* Number of data points */
 int nx=0;			/* Number of nodes in x-dir. */
 int ny=0;			/* Number of nodes in y-dir. (Final grid) */
-int	mx = 0;
-int	my = 0;
-int	ij_sw_corner, ij_se_corner, ij_nw_corner, ij_ne_corner;
+int mx = 0;
+int my = 0;
+int ij_sw_corner, ij_se_corner, ij_nw_corner, ij_ne_corner;
 int block_nx;			/* Number of nodes in x-dir for a given grid factor */
 int block_ny;			/* Number of nodes in y-dir for a given grid factor */
 int max_iterations=250;		/* Max iter per call to iterate */
 int total_iterations = 0;
 int grid, old_grid;		/* Node spacings  */
-int	grid_east;
+int grid_east;
 int n_fact = 0;			/* Number of factors in common (ny-1, nx-1) */
 int factors[32];		/* Array of common factors */
 int long_verbose = FALSE;
@@ -106,25 +107,23 @@ int set_low = 0;		/* 0 unconstrained,1 = by min data value, 2 = by user value */
 int set_high = 0;		/* 0 unconstrained,1 = by max data value, 2 = by user value */
 int constrained = FALSE;	/* TRUE if set_low or set_high is TRUE */
 double low_limit, high_limit;	/* Constrains on range of solution */
-double x_min, x_max, y_min, y_max;	/* minmax coordinates */
 float *lower, *upper;		/* arrays for minmax values, if set */
-double xinc, yinc;		/* Size of each grid cell (final size) */
 double grid_xinc, grid_yinc;	/* size of each grid cell for a given grid factor */
 double r_xinc, r_yinc, r_grid_xinc, r_grid_yinc;	/* Reciprocals  */
 double converge_limit = 0.0;	/* Convergence limit */
 double radius = 0.0;			/* Search radius for initializing grid  */
-double	tension = 0.0;		/* Tension parameter on the surface  */
-double	boundary_tension = 0.0;
-double	interior_tension = 0.0;
-double	a0_const_1, a0_const_2;	/* Constants for off grid point equation  */
-double	e_2, e_m2, one_plus_e2;
+double tension = 0.0;		/* Tension parameter on the surface  */
+double boundary_tension = 0.0;
+double interior_tension = 0.0;
+double a0_const_1, a0_const_2;	/* Constants for off grid point equation  */
+double e_2, e_m2, one_plus_e2;
 double eps_p2, eps_m2, two_plus_ep2, two_plus_em2;
-double	x_edge_const, y_edge_const;
-double	l_epsilon = 1.0;
-double	z_mean;
-double	z_scale = 1.0;		/* Root mean square range of z after removing planar trend  */
-double	r_z_scale = 1.0;	/* reciprocal of z_scale  */
-double	plane_c0, plane_c1, plane_c2;	/* Coefficients of best fitting plane to data  */
+double x_edge_const, y_edge_const;
+double l_epsilon = 1.0;
+double z_mean;
+double z_scale = 1.0;		/* Root mean square range of z after removing planar trend  */
+double r_z_scale = 1.0;	/* reciprocal of z_scale  */
+double plane_c0, plane_c1, plane_c2;	/* Coefficients of best fitting plane to data  */
 double small;			/* Let data point coincide with node if distance < small */
 float *u;			/* Pointer to grid array */
 float *v2;			/* Pointer to v.2.0 grid array */
@@ -133,8 +132,8 @@ char mode_type[2] = {'I','D'};	/* D means include data points when iterating
 				 * I means just interpolate from larger grid */
 char format[BUFSIZ];
 
-int	offset[25][12];		/* Indices of 12 nearby points in 25 cases of edge conditions  */
-double		coeff[2][12];	/* Coefficients for 12 nearby points, constrained and unconstrained  */
+int offset[25][12];		/* Indices of 12 nearby points in 25 cases of edge conditions  */
+double coeff[2][12];		/* Coefficients for 12 nearby points, constrained and unconstrained  */
 
 double relax_old, relax_new = 1.4;	/* Coefficients for relaxation factor to speed up convergence */
 
@@ -157,6 +156,7 @@ struct SUGGESTION {	/* Used to find top ten list of faster grid dimensions  */
 	double	factor;	/* Speed up by a factor of factor  */
 };
 
+struct GRD_HEADER h;
 
 FILE	*fp_in = NULL;	/* File pointer  */
 
@@ -166,27 +166,21 @@ void set_grid_parameters(void), read_data(void), throw_away_unusables(void), rem
 void load_constraints(char *low, char *high), smart_divide(void), set_offset(void), set_index(void), initialize_grid(void), set_coefficients(void);
 void find_nearest_point(void), fill_in_forecast(void), check_errors(void), replace_planar_trend(void), write_output(struct GRD_HEADER *h, char *grdfile);
 
-main(int argc, char **argv)
+int main(int argc, char **argv)
 {
 
 	void	suggest_sizes_for_surface(int nx, int ny);
 
 	int	i, error = FALSE, size_query = FALSE, n_files = 0;
 
-	char	modifier, *grdfile, low[128], high[128];
-
-	struct GRD_HEADER h;
+	char	modifier, *grdfile, low[GMT_LONG_TEXT], high[GMT_LONG_TEXT];
 
 	grdfile = 0;
-	
+
 	argc = GMT_begin (argc, argv);
 
 	GMT_grd_init (&h, argc, argv, FALSE);
 
-	x_min = y_min = 0.0;
-	x_max = y_max = 0.0;
-	xinc = yinc = 0.0;
-
 	/* New in v4.3:  Default to unconstrained:  */
 	set_low = set_high = 0; 
 
@@ -204,7 +198,7 @@ main(int argc, char **argv)
 				case 'b':	/* Input triplets are binary, not ascii */
 				case 'f':
 				case '\0':
-                                      error += GMT_get_common_args (argv[i], &x_min, &x_max, &y_min, &y_max);
+                                      error += GMT_get_common_args (argv[i], &h.x_min, &h.x_max, &h.y_min, &h.y_max);
                                       break;
                               
 				/* Supplemental parameters */
@@ -219,7 +213,10 @@ main(int argc, char **argv)
 					grdfile = &argv[i][2];
 					break;
 				case 'I':
-					GMT_getinc (&argv[i][2], &xinc, &yinc);
+					if (GMT_getinc (&argv[i][2], &h.x_inc, &h.y_inc)) {
+						GMT_inc_syntax ('I', 1);
+						error = TRUE;
+					}
 					break;
 				case 'L':	/* Set limits */
 						/* This is new, to use -Ll and -Lu:  */
@@ -307,22 +304,20 @@ main(int argc, char **argv)
 			n_files++;
 		}
 	}
-	
+
 	if (argc == 1 || GMT_quick) {	/* Display usage */
 		fprintf (stderr, "surface %s - Adjustable tension continuous curvature surface gridding\n\n", GMT_VERSION);
-		fprintf (stderr, "usage: surface [xyz-file] -G<output_grdfile_name> -I<xinc>[m|c][/<yinc>[m|c]]\n");
+		fprintf (stderr, "usage: surface [xyz-file] -G<output_grdfile_name> -I<xinc[u][!|+]>[/<yinc>[u][!|+]]\n");
 		fprintf (stderr, "\t-R<west>/<east>/<south>/<north> [-A<aspect_ratio>] [-C<convergence_limit>] [-H[<nrec>]]\n");
 		fprintf (stderr, "\t[-Ll<limit>] [-Lu<limit>] [-N<n_iterations>] ] [-S<search_radius>[m]] [-T<tension>[i][b]]\n");
 		fprintf (stderr, "\t[-Q] [-V[l]] [-Z<over_relaxation_parameter>] [-:] [-bi[s][<n>]] [-f[i|o]<colinfo>]\n\n");
-		
+
 		if (GMT_quick) exit (EXIT_FAILURE);
-		
+
 		fprintf (stderr, "\tsurface will read from standard input or a single <xyz-file>.\n\n");
 		fprintf (stderr, "\tRequired arguments to surface:\n");
 		fprintf (stderr, "\t-G sets output grd File name\n");
-		fprintf (stderr, "\t-I sets the Increment of the grid; enter xinc, optionally xinc/yinc.\n");
-		fprintf (stderr, "\t\tDefault is yinc = xinc.  Append an m [or c] to xinc or yinc to indicate minutes [or seconds]\n");
-		fprintf (stderr, "\t\te.g.  -I10m/5m grids longitude every 10 minutes, latitude every 5 minutes.\n\n");
+		GMT_inc_syntax ('I', 0);
 		GMT_explain_option ('R');
 		fprintf (stderr, "\n\tOPTIONS:\n");
 		fprintf (stderr, "\t-A<aspect_ratio>  = 1.0  by default which gives an isotropic solution.\n");
@@ -372,7 +367,8 @@ main(int argc, char **argv)
 		fprintf (stderr, "%s: GMT SYNTAX ERROR:  Must specify -R option\n", GMT_program);
 		error++;
 	}
-	if (xinc <= 0.0 || yinc <= 0.0) {
+	GMT_RI_prepare (&h);	/* Ensure -R -I consistency and set nx, ny */
+	if (h.x_inc <= 0.0 || h.y_inc <= 0.0) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -I option.  Must specify positive increment(s)\n", GMT_program);
 		error++;
 	}
@@ -387,7 +383,7 @@ main(int argc, char **argv)
 	if (!grdfile) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR option -G:  Must specify output file\n", GMT_program);
 		error++;
-	}	
+	}
 	if (GMT_io.binary[GMT_IN] && gmtdefs.io_header[GMT_IN]) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data cannot have header -H\n", GMT_program);
 		error++;
@@ -401,8 +397,8 @@ main(int argc, char **argv)
 		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Only one input file allowed.  Preprocess you files with blockmean/median/mode first\n", GMT_program);
 		error++;
 	}
-	
-	
+
+
 	if (error) exit (EXIT_FAILURE);
 
 	GMT_put_history (argc, argv);	/* Update .gmtcommands4 */
@@ -412,12 +408,6 @@ main(int argc, char **argv)
 		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 
-	h.x_min = x_min;
-	h.x_max = x_max;
-	h.y_min = y_min;
-	h.y_max = y_max;
-	h.x_inc = xinc;
-	h.y_inc = yinc;
 
 	GMT_grd_RI_verify (&h, 1);
 
@@ -427,21 +417,19 @@ main(int argc, char **argv)
 	}
 	relax_old = 1.0 - relax_new;
 
-	nx = irint ((x_max - x_min)/xinc) + 1;
-	ny = irint ((y_max - y_min)/yinc) + 1;
-	h.nx = nx;
-	h.ny = ny;
+	nx = h.nx;
+	ny = h.ny;
 	mx = nx + 4;
 	my = ny + 4;
-	r_xinc = 1.0 / xinc;
-	r_yinc = 1.0 / yinc;
+	r_xinc = 1.0 / h.x_inc;
+	r_yinc = 1.0 / h.y_inc;
 
 	/* New stuff here for v4.3:  Check out the grid dimensions:  */
 	grid = gcd_euclid (nx-1, ny-1);
 
 	if (gmtdefs.verbose || size_query) {
 		sprintf (format, "W: %s E: %s S: %s N: %s nx: %%d ny: %%d\n", gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format);
-		fprintf (stderr, format, x_min, x_max, y_min, y_max, nx-1, ny-1);
+		fprintf (stderr, format, h.x_min, h.x_max, h.y_min, h.y_max, nx-1, ny-1);
 	}
 	if (grid == 1 && gmtdefs.verbose) fprintf(stderr,"%s:  WARNING:  Your grid dimensions are mutually prime.\n", GMT_program);
 	if (( grid == 1 && gmtdefs.verbose) || size_query) suggest_sizes_for_surface(nx-1, ny-1);
@@ -450,7 +438,7 @@ main(int argc, char **argv)
 	/* New idea: set grid = 1, read data, setting index.  Then throw
 		away data that can't be used in end game, constraining
 		size of briggs->b[6] structure.  */
-	
+
 	grid = 1;
 	set_grid_parameters();
 	if (fp_in == NULL) {
@@ -464,9 +452,9 @@ main(int argc, char **argv)
 	remove_planar_trend();
 	rescale_z_values();
 	load_constraints(low, high);
-	
+
 	/* Set up factors and reset grid to first value  */
-	
+
 	grid = gcd_euclid(nx-1, ny-1);
 	n_fact = get_prime_factors(grid, factors);
 	set_grid_parameters();
@@ -479,7 +467,7 @@ main(int argc, char **argv)
 	/* Now the data are ready to go for the first iteration.  */
 
 	/* Allocate more space  */
-	
+
 	briggs = (struct BRIGGS *) GMT_memory (VNULL, (size_t)npoints, sizeof(struct BRIGGS), GMT_program);
 	iu = (char *) GMT_memory (VNULL, (size_t)(mx * my), sizeof(char), GMT_program);
 	u = (float *) GMT_memory (VNULL, (size_t)(mx * my), sizeof(float), GMT_program);
@@ -487,9 +475,9 @@ main(int argc, char **argv)
 	if (radius > 0) initialize_grid(); /* Fill in nodes with a weighted avg in a search radius  */
 
 	if (gmtdefs.verbose) fprintf(stderr,"Grid\tMode\tIteration\tMax Change\tConv Limit\tTotal Iterations\n");
-	
+
 	set_coefficients();
-	
+
 	old_grid = grid;
 	find_nearest_point ();
 	iterate (1);
@@ -505,11 +493,11 @@ main(int argc, char **argv)
 		find_nearest_point ();
 		iterate (1);
 	}
-	
+
 	if (gmtdefs.verbose) check_errors ();
 
 	replace_planar_trend();
-	
+
 	GMT_free ((void *) data);
 	GMT_free ((void *) briggs);
 	GMT_free ((void *)iu);
@@ -520,14 +508,16 @@ main(int argc, char **argv)
 
 	GMT_free ((void *) u);
 
-	
+
 	GMT_end (argc, argv);
+
+	exit (EXIT_SUCCESS);
 }
 
 void	set_coefficients(void)
 {
 	double	e_4, loose, a0;
-	
+
 	loose = 1.0 - interior_tension;
 	e_2 = l_epsilon * l_epsilon;
 	e_4 = e_2 * e_2;
@@ -536,16 +526,16 @@ void	set_coefficients(void)
 	one_plus_e2 = 1.0 + e_2;
 	two_plus_ep2 = 2.0 + 2.0*eps_p2;
 	two_plus_em2 = 2.0 + 2.0*eps_m2;
-	
+
 	x_edge_const = 4 * one_plus_e2 - 2 * (interior_tension / loose);
 	e_m2 = 1.0 / e_2;
 	y_edge_const = 4 * (1.0 + e_m2) - 2 * (interior_tension * e_m2 / loose);
 
-	
+
 	a0 = 1.0 / ( (6 * e_4 * loose + 10 * e_2 * loose + 8 * loose - 2 * one_plus_e2) + 4*interior_tension*one_plus_e2);
 	a0_const_1 = 2 * loose * (1.0 + e_4);
 	a0_const_2 = 2.0 - interior_tension + 2 * loose * e_2;
-	
+
 	coeff[1][4] = coeff[1][7] = -loose;
 	coeff[1][0] = coeff[1][11] = -loose * e_4;
 	coeff[0][4] = coeff[0][7] = -loose * a0;
@@ -556,10 +546,10 @@ void	set_coefficients(void)
 	coeff[0][2] = coeff[0][9] = coeff[0][5] * e_2;
 	coeff[1][1] = coeff[1][3] = coeff[1][8] = coeff[1][10] = -2 * loose * e_2;
 	coeff[0][1] = coeff[0][3] = coeff[0][8] = coeff[0][10] = coeff[1][1] * a0;
-	
+
 	e_2 *= 2;		/* We will need these in boundary conditions  */
 	e_m2 *= 2;
-	
+
 	ij_sw_corner = 2 * my + 2;			/*  Corners of array of actual data  */
 	ij_se_corner = ij_sw_corner + (nx - 1) * my;
 	ij_nw_corner = ij_sw_corner + (ny - 1);
@@ -571,7 +561,7 @@ void	set_offset(void)
 {
 	int	add_w[5], add_e[5], add_s[5], add_n[5], add_w2[5], add_e2[5], add_s2[5], add_n2[5];
 	int	i, j, kase;
-	
+
 	add_w[0] = -my; add_w[1] = add_w[2] = add_w[3] = add_w[4] = -grid_east;
 	add_w2[0] = -2 * my;  add_w2[1] = -my - grid_east;  add_w2[2] = add_w2[3] = add_w2[4] = -2 * grid_east;
 	add_e[4] = my; add_e[0] = add_e[1] = add_e[2] = add_e[3] = grid_east;
@@ -612,46 +602,46 @@ void fill_in_forecast (void) {
 	int index_new;
 	double delta_x, delta_y, a0, a1, a2, a3;
 	double old_size;
-	
-		
+
+
 	old_size = 1.0 / (double)old_grid;
 
 	/* first do from southwest corner */
-	
+
 	for (i = 0; i < nx-1; i += old_grid) {
-		
+
 		for (j = 0; j < ny-1; j += old_grid) {
-			
+
 			/* get indices of bilinear square */
 			index_0 = ij_sw_corner + i * my + j;
 			index_1 = index_0 + old_grid * my;
 			index_2 = index_1 + old_grid;
 			index_3 = index_0 + old_grid;
-			
+
 			/* get coefficients */
 			a0 = u[index_0];
 			a1 = u[index_1] - a0;
 			a2 = u[index_3] - a0;
 			a3 = u[index_2] - a0 - a1 - a2;
-			
+
 			/* find all possible new fill ins */
-			
+
 			for (ii = i;  ii < i + old_grid; ii += grid) {
 				delta_x = (ii - i) * old_size;
 				for (jj = j;  jj < j + old_grid; jj += grid) {
 					index_new = ij_sw_corner + ii * my + jj;
 					if (index_new == index_0) continue;
 					delta_y = (jj - j) * old_size;
-					u[index_new] = (float)(a0 + a1 * delta_x + delta_y * ( a2 + a3 * delta_x));	
+					u[index_new] = (float)(a0 + a1 * delta_x + delta_y * ( a2 + a3 * delta_x));
 					iu[index_new] = 0;
 				}
 			}
 			iu[index_0] = 5;
 		}
 	}
-	
+
 	/* now do linear guess along east edge */
-	
+
 	for (j = 0; j < (ny-1); j += old_grid) {
 		index_0 = ij_se_corner + j;
 		index_3 = index_0 + old_grid;
@@ -688,7 +678,7 @@ int compare_points (const void *point_1v, const void *point_2v)
 		*/
 	int block_i, block_j, index_1, index_2;
 	double x0, y0, dist_1, dist_2;
-	
+
 struct DATA *point_1, *point_2;
 	point_1 = (struct DATA *)point_1v;
 	point_2 = (struct DATA *)point_2v;
@@ -703,8 +693,8 @@ struct DATA *point_1, *point_2;
 	else {	/* Points are in same grid cell, find the one who is nearest to grid point */
 		block_i = point_1->index/block_ny;
 		block_j = point_1->index%block_ny;
-		x0 = x_min + block_i * grid_xinc;
-		y0 = y_min + block_j * grid_yinc;
+		x0 = h.x_min + block_i * grid_xinc;
+		y0 = h.y_min + block_j * grid_yinc;
 		dist_1 = (point_1->x - x0) * (point_1->x - x0) + (point_1->y - y0) * (point_1->y - y0);
 		dist_2 = (point_2->x - x0) * (point_2->x - x0) + (point_2->y - y0) * (point_2->y - y0);
 		if (dist_1 < dist_2)
@@ -729,8 +719,8 @@ void set_index (void) {
 	int i, j, k, k_skipped = 0;
 
 	for (k = 0; k < npoints; k++) {
-		i = (int)floor(((data[k].x-x_min)*r_grid_xinc) + 0.5);
-		j = (int)floor(((data[k].y-y_min)*r_grid_yinc) + 0.5);
+		i = (int)floor(((data[k].x-h.x_min)*r_grid_xinc) + 0.5);
+		j = (int)floor(((data[k].y-h.y_min)*r_grid_yinc) + 0.5);
 		if (i < 0 || i >= block_nx || j < 0 || j >= block_ny) {
 			data[k].index = OUTSIDE;
 			k_skipped++;
@@ -738,11 +728,11 @@ void set_index (void) {
 		else
 			data[k].index = i * block_ny + j;
 	}
-	
+
 	qsort ((void *)data, (size_t)npoints, sizeof (struct DATA), compare_points);
-	
+
 	npoints -= k_skipped;
-	
+
 }
 
 void find_nearest_point(void) {
@@ -750,14 +740,14 @@ void find_nearest_point(void) {
 	double x0, y0, dx, dy, xys, xy1, btemp;
 	double b0, b1, b2, b3, b4, b5;
 	float z_at_node;
-	
+
 	last_index = -1;
 	small = 0.05 * ((grid_xinc < grid_yinc) ? grid_xinc : grid_yinc);
 
 	for (i = 0; i < nx; i += grid)	/* Reset grid info */
 		for (j = 0; j < ny; j += grid)
 			iu[ij_sw_corner + i*my + j] = 0;
-	
+
 	briggs_index = 0;
 	for (k = 0; k < npoints; k++) {	/* Find constraining value  */
 		if (data[k].index != last_index) {
@@ -765,8 +755,8 @@ void find_nearest_point(void) {
 			block_j = data[k].index%block_ny;
 			last_index = data[k].index;
 	 		iu_index = ij_sw_corner + (block_i * my + block_j) * grid;
-	 		x0 = x_min + block_i*grid_xinc;
-	 		y0 = y_min + block_j*grid_yinc;
+	 		x0 = h.x_min + block_i*grid_xinc;
+	 		y0 = h.y_min + block_j*grid_yinc;
 	 		dx = (data[k].x - x0)*r_grid_xinc;
 	 		dy = (data[k].y - y0)*r_grid_yinc;
 	 		if (fabs(dx) < small && fabs(dy) < small) {	/* Close enough to assign value to node */
@@ -824,13 +814,13 @@ void find_nearest_point(void) {
 	 }
 }
 
-						
+
 void set_grid_parameters(void)
-{			
+{
 	block_ny = (ny - 1) / grid + 1;
 	block_nx = (nx - 1) / grid + 1;
-	grid_xinc = grid * xinc;
-	grid_yinc = grid * yinc;
+	grid_xinc = grid * h.x_inc;
+	grid_yinc = grid * h.y_inc;
 	grid_east = grid * my;
 	r_grid_xinc = 1.0 / grid_xinc;
 	r_grid_yinc = 1.0 / grid_yinc;
@@ -849,9 +839,9 @@ void initialize_grid(void)
 	 rfact = -4.5/(radius*radius);
 	 
 	 for (i = 0; i < block_nx; i ++ ) {
-	 	x0 = x_min + i*grid_xinc;
+	 	x0 = h.x_min + i*grid_xinc;
 	 	for (j = 0; j < block_ny; j ++ ) {
-	 		y0 = y_min + j*grid_yinc;
+	 		y0 = h.y_min + j*grid_yinc;
 	 		imin = i - irad;
 	 		if (imin < 0) imin = 0;
 	 		imax = i + irad;
@@ -906,11 +896,11 @@ void new_initialize_grid(void)
 	while (k < npoints) {
 		block_i = data[k].index / block_ny;
 		block_j = data[k].index % block_ny;
-		x0 = x_min + block_i*grid_xinc;
-		y0 = y_min + block_j*grid_yinc;
+		x0 = h.x_min + block_i*grid_xinc;
+		y0 = h.y_min + block_j*grid_yinc;
 		u_index = ij_sw_corner + (block_i*my + block_j) * grid;
 		k_index = data[k].index;
-		
+
 		dy = (data[k].y - y0) * dy_scale;
 		dx = (data[k].x - x0) * dx_scale;
 		sum_w = 1.0 / (1.0 + dx*dx + dy*dy);
@@ -918,7 +908,7 @@ void new_initialize_grid(void)
 		k++;
 
 		while (k < npoints && data[k].index == k_index) {
-			
+
 			dy = (data[k].y - y0) * dy_scale;
 			dx = (data[k].x - x0) * dx_scale;
 			weight = 1.0 / (1.0 + dx*dx + dy*dy);
@@ -935,19 +925,19 @@ void new_initialize_grid(void)
 
 void read_data(void)
 {
-	int	i, j, k, kmax, kmin, n_fields, n_expected_fields;
+	int	i, j, k, kmax = 0, kmin = 0, n_fields, n_expected_fields;
 	double	*in, zmin = DBL_MAX, zmax = -DBL_MAX;
 	char	buffer[BUFSIZ];
 
 	data = (struct DATA *) GMT_memory (VNULL, (size_t)n_alloc, sizeof(struct DATA), GMT_program);
-	
+
 	/* Read in xyz data and computes index no and store it in a structure */
-	
+
 	k = 0;
 	z_mean = 0;
 	n_expected_fields = (GMT_io.binary[GMT_IN]) ? GMT_io.ncol[GMT_IN] : 3;
 	if (!GMT_io.binary[GMT_IN] && gmtdefs.io_header[GMT_IN]) for (i = 0; i < gmtdefs.n_header_recs; i++) GMT_fgets (buffer, BUFSIZ, fp_in);
-			
+
 	while ((n_fields = GMT_input (fp_in, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
 
 		if (GMT_io.status & GMT_IO_MISMATCH) {
@@ -956,10 +946,10 @@ void read_data(void)
 		}
 
 		if (GMT_is_dnan (in[2])) continue;
-		
-		i = (int)floor(((in[0]-x_min)*r_grid_xinc) + 0.5);
+
+		i = (int)floor(((in[0]-h.x_min)*r_grid_xinc) + 0.5);
 		if (i < 0 || i >= block_nx) continue;
-		j = (int)floor(((in[1]-y_min)*r_grid_yinc) + 0.5);
+		j = (int)floor(((in[1]-h.y_min)*r_grid_yinc) + 0.5);
 		if (j < 0 || j >= block_ny) continue;
 
 		data[k].index = i * block_ny + j;
@@ -975,16 +965,16 @@ void read_data(void)
 			data = (struct DATA *) GMT_memory ((void *)data, (size_t)n_alloc, sizeof(struct DATA), GMT_program);
 		}
 	}
-	
+
 	if (fp_in != GMT_stdin) GMT_fclose (fp_in);
 
 	npoints = k;
-	
+
 	if (npoints == 0) {
 		fprintf (stderr, "%s:  No datapoints inside region, aborts\n", GMT_program);
 		exit (EXIT_FAILURE);
 	}
-	
+
 	z_mean /= k;
 	if (gmtdefs.verbose) {
 		sprintf(format, "%s %s %s\n", gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format);
@@ -994,7 +984,7 @@ void read_data(void)
 		fprintf(stderr, format, (double)data[kmax].x, (double)data[kmax].y, (double)data[kmax].z);
 	}
 	data = (struct DATA *) GMT_memory ((void *)data, (size_t)npoints, sizeof(struct DATA), GMT_program);
-	
+
 	if (set_low == 1)
 		low_limit = data[kmin].z;
 	else if (set_low == 2 && low_limit > data[kmin].z) {
@@ -1013,9 +1003,9 @@ void read_data(void)
 void write_output(struct GRD_HEADER *h, char *grdfile)
 {	/* Uses v.2.0 netCDF grd format */
 	int	index, i, j;
-		
+
 	strcpy (h->title, GMT_program);
-	
+
 	v2 = (float *) GMT_memory (VNULL, (size_t)(nx * ny), sizeof (float), GMT_program);
 	index = ij_sw_corner;
 	for(i = 0; i < nx; i++, index += my) for (j = 0; j < ny; j++) v2[j*nx+i] = u[index + ny - j - 1];
@@ -1025,18 +1015,18 @@ void write_output(struct GRD_HEADER *h, char *grdfile)
 	}
 	GMT_free ((void *)v2);
 }
-	
+
 int	iterate(int mode)
 {
 
 	int	i, j, k, ij, kase, briggs_index, ij_v2;
 	int	x_case, y_case, x_w_case, x_e_case, y_s_case, y_n_case;
 	int	iteration_count = 0;
-	
+
 	double	current_limit = converge_limit / grid;
 	double	change, max_change = 0.0, busum, sum_ij;
 	double	b0, b1, b2, b3, b4, b5;
-	
+
 	double	x_0_const = 4.0 * (1.0 - boundary_tension) / (2.0 - boundary_tension);
 	double	x_1_const = (3 * boundary_tension - 2.0) / (2.0 - boundary_tension);
 	double	y_denom = 2 * l_epsilon * (1.0 - boundary_tension) + boundary_tension;
@@ -1047,16 +1037,16 @@ int	iterate(int mode)
 
 	do {
 		briggs_index = 0;	/* Reset the constraint table stack pointer  */
-		
+
 		max_change = -1.0;
-		
+
 		/* Fill in auxiliary boundary values (in new way) */
-		
+
 		/* First set d2[]/dn2 = 0 along edges:  */
 		/* New experiment : (1-T)d2[]/dn2 + Td[]/dn = 0  */
-		
-		
-		
+
+
+
 		for (i = 0; i < nx; i += grid) {
 			/* set d2[]/dy2 = 0 on south side:  */
 			ij = ij_sw_corner + i * my;
@@ -1066,9 +1056,9 @@ int	iterate(int mode)
 			ij = ij_nw_corner + i * my;
 			/* u[ij + 1] = 2 * u[ij] - u[ij - grid];  */
 			u[ij + 1] = (float)(y_0_const * u[ij] + y_1_const * u[ij - grid]);
-			
+
 		}
-		
+
 		for (j = 0; j < ny; j += grid) {
 			/* set d2[]/dx2 = 0 on west side:  */
 			ij = ij_sw_corner + j;
@@ -1079,35 +1069,35 @@ int	iterate(int mode)
 			/* u[ij + my] = 2 * u[ij] - u[ij - grid_east];  */
 			u[ij + my] = (float)(x_1_const * u[ij - grid_east] + x_0_const * u[ij]);
 		}
-			
+
 		/* Now set d2[]/dxdy = 0 at each corner:  */
-		
+
 		ij = ij_sw_corner;
 		u[ij - my - 1] = u[ij + grid_east - 1] + u[ij - my + grid] - u[ij + grid_east + grid];
-				
+
 		ij = ij_nw_corner;
 		u[ij - my + 1] = u[ij + grid_east + 1] + u[ij - my - grid] - u[ij + grid_east - grid];
-				
+
 		ij = ij_se_corner;
 		u[ij + my - 1] = u[ij - grid_east - 1] + u[ij + my + grid] - u[ij - grid_east + grid];
-				
+
 		ij = ij_ne_corner;
 		u[ij + my + 1] = u[ij - grid_east + 1] + u[ij + my - grid] - u[ij - grid_east - grid];
-		
+
 		/* Now set (1-T)dC/dn + Tdu/dn = 0 at each edge :  */
 		/* New experiment:  only dC/dn = 0  */
-		
+
 		x_w_case = 0;
 		x_e_case = block_nx - 1;
 		for (i = 0; i < nx; i += grid, x_w_case++, x_e_case--) {
-		
+
 			if(x_w_case < 2)
 				x_case = x_w_case;
 			else if(x_e_case < 2)
 				x_case = 4 - x_e_case;
 			else
 				x_case = 2;
-				
+
 			/* South side :  */
 			kase = x_case * 5;
 			ij = ij_sw_corner + i * my;
@@ -1125,18 +1115,18 @@ int	iterate(int mode)
 					+ two_plus_em2 * (u[ij + offset[kase][9]] - u[ij + offset[kase][2]]) );
 				/*  - tense * eps_m2 * (u[ij + offset[kase][2]] - u[ij + offset[kase][9]]) / (1.0 - tense);  */
 		}
-		
+
 		y_s_case = 0;
 		y_n_case = block_ny - 1;
 		for (j = 0; j < ny; j += grid, y_s_case++, y_n_case--) {
-				
+
 			if(y_s_case < 2)
 				y_case = y_s_case;
 			else if(y_n_case < 2)
 				y_case = 4 - y_n_case;
 			else
 				y_case = 2;
-			
+
 			/* West side :  */
 			kase = y_case;
 			ij = ij_sw_corner + j;
@@ -1155,37 +1145,37 @@ int	iterate(int mode)
 				/*  - tense * (u[ij + offset[kase][6]] - u[ij + offset[kase][5]]) / (1.0 - tense);  */
 		}
 
-			
-			
+
+
 		/* That's it for the boundary points.  Now loop over all data  */
-		
+
 		x_w_case = 0;
 		x_e_case = block_nx - 1;
 		for (i = 0; i < nx; i += grid, x_w_case++, x_e_case--) {
-		
+
 			if(x_w_case < 2)
 				x_case = x_w_case;
 			else if(x_e_case < 2)
 				x_case = 4 - x_e_case;
 			else
 				x_case = 2;
-			
+
 			y_s_case = 0;
 			y_n_case = block_ny - 1;
-			
+
 			ij = ij_sw_corner + i * my;
-			
+
 			for (j = 0; j < ny; j += grid, ij += grid, y_s_case++, y_n_case--) {
-	
+
 				if (iu[ij] == 5) continue;	/* Point is fixed  */
-				
+
 				if(y_s_case < 2)
 					y_case = y_s_case;
 				else if(y_n_case < 2)
 					y_case = 4 - y_n_case;
 				else
 					y_case = 2;
-				
+
 				kase = x_case * 5 + y_case;
 				sum_ij = 0.0;
 
@@ -1195,7 +1185,7 @@ int	iterate(int mode)
 					}
 				}
 				else {				/* Point is constrained  */
-				
+
 					b0 = briggs[briggs_index].b[0];
 					b1 = briggs[briggs_index].b[1];
 					b2 = briggs[briggs_index].b[2];
@@ -1237,10 +1227,10 @@ int	iterate(int mode)
 					sum_ij = (sum_ij + a0_const_2 * (busum + b5))
 						/ (a0_const_1 + a0_const_2 * b4);
 				}
-				
+
 				/* New relaxation here  */
 				sum_ij = u[ij] * relax_old + sum_ij * relax_new;
-				
+
 				if (constrained) {	/* Must check limits.  Note lower/upper is v2 format and need ij_v2! */
 					ij_v2 = (ny - j - 1) * nx + i;
 					if (set_low && !GMT_is_fnan (lower[ij_v2]) && sum_ij < lower[ij_v2])
@@ -1248,7 +1238,7 @@ int	iterate(int mode)
 					else if (set_high && !GMT_is_fnan (upper[ij_v2]) && sum_ij > upper[ij_v2])
 						sum_ij = upper[ij_v2];
 				}
-					
+
 				change = fabs(sum_ij - u[ij]);
 				u[ij] = (float)sum_ij;
 				if (change > max_change) max_change = change;
@@ -1261,7 +1251,7 @@ int	iterate(int mode)
 			grid, mode_type[mode], iteration_count, max_change, current_limit, total_iterations);
 
 	} while (max_change > current_limit && iteration_count < max_iterations);
-	
+
 	if (gmtdefs.verbose && !long_verbose) fprintf(stderr, format,
 		grid, mode_type[mode], iteration_count, max_change, current_limit, total_iterations);
 
@@ -1271,17 +1261,17 @@ int	iterate(int mode)
 void check_errors (void) {
 
 	int	i, j, k, ij, n_nodes, move_over[12];	/* move_over = offset[kase][12], but grid = 1 so move_over is easy  */
-	
+
 	double	x0, y0, dx, dy, mean_error, mean_squared_error, z_est, z_err, curvature, c;
 	double	du_dx, du_dy, d2u_dx2, d2u_dxdy, d2u_dy2, d3u_dx3, d3u_dx2dy, d3u_dxdy2, d3u_dy3;
-	
+
 	double	x_0_const = 4.0 * (1.0 - boundary_tension) / (2.0 - boundary_tension);
 	double	x_1_const = (3 * boundary_tension - 2.0) / (2.0 - boundary_tension);
 	double	y_denom = 2 * l_epsilon * (1.0 - boundary_tension) + boundary_tension;
 	double	y_0_const = 4 * l_epsilon * (1.0 - boundary_tension) / y_denom;
 	double	y_1_const = (boundary_tension - 2 * l_epsilon * (1.0 - boundary_tension) ) / y_denom;
-	
-	
+
+
 	move_over[0] = 2;
 	move_over[1] = 1 - my;
 	move_over[2] = 1;
@@ -1297,7 +1287,7 @@ void check_errors (void) {
 
 	mean_error = 0;
 	mean_squared_error = 0;
-	
+
 	/* First update the boundary values  */
 
 	for (i = 0; i < nx; i ++) {
@@ -1324,28 +1314,28 @@ void check_errors (void) {
 	u[ij + my + 1] = u[ij - my + 1] + u[ij + my - 1] - u[ij - my - 1];
 
 	for (i = 0; i < nx; i ++) {
-				
+
 		ij = ij_sw_corner + i * my;
 		u[ij + move_over[11]] = 
 			(float)(u[ij + move_over[0]] + eps_m2*(u[ij + move_over[1]] + u[ij + move_over[3]]
 				- u[ij + move_over[8]] - u[ij + move_over[10]])
 				+ two_plus_em2 * (u[ij + move_over[9]] - u[ij + move_over[2]]) );
-					
+
 		ij = ij_nw_corner + i * my;
 		u[ij + move_over[0]] = 
 			-(float)(-u[ij + move_over[11]] + eps_m2 * (u[ij + move_over[1]] + u[ij + move_over[3]]
 				- u[ij + move_over[8]] - u[ij + move_over[10]])
 				+ two_plus_em2 * (u[ij + move_over[9]] - u[ij + move_over[2]]) );
 	}
-		
+
 	for (j = 0; j < ny; j ++) {
-			
+
 		ij = ij_sw_corner + j;
 		u[ij+move_over[4]] = 
 			u[ij + move_over[7]] + (float)(eps_p2 * (u[ij + move_over[3]] + u[ij + move_over[10]]
 			-u[ij + move_over[1]] - u[ij + move_over[8]])
 			+ two_plus_ep2 * (u[ij + move_over[5]] - u[ij + move_over[6]]));
-				
+
 		ij = ij_se_corner + j;
 		u[ij + move_over[7]] = 
 			- (float)(-u[ij + move_over[4]] + eps_p2 * (u[ij + move_over[3]] + u[ij + move_over[10]]
@@ -1355,14 +1345,14 @@ void check_errors (void) {
 
 	/* That resets the boundary values.  Now we can test all data.  
 		Note that this loop checks all values, even though only nearest were used.  */
-	
+
 	for (k = 0; k < npoints; k++) {
 		i = data[k].index/ny;
 		j = data[k].index%ny;
 	 	ij = ij_sw_corner + i * my + j;
 	 	if ( iu[ij] == 5 ) continue;
-	 	x0 = x_min + i*xinc;
-	 	y0 = y_min + j*yinc;
+	 	x0 = h.x_min + i*h.x_inc;
+	 	y0 = h.y_min + j*h.y_inc;
 	 	dx = (data[k].x - x0)*r_xinc;
 	 	dy = (data[k].y - y0)*r_yinc;
  
@@ -1382,11 +1372,11 @@ void check_errors (void) {
 	 				- ( u[ij + move_over[1]] + u[ij + move_over[8]] - 2 * u[ij + move_over[5]] ) );
 
 	 	/* 3rd order Taylor approx:  */
-	 		
+	 
 	 	z_est = u[ij] + dx * (du_dx +  dx * ( (0.5 * d2u_dx2) + dx * (d3u_dx3 / 6.0) ) )
 				+ dy * (du_dy +  dy * ( (0.5 * d2u_dy2) + dy * (d3u_dy3 / 6.0) ) )
 	 			+ dx * dy * (d2u_dxdy) + (0.5 * dx * d3u_dx2dy) + (0.5 * dy * d3u_dxdy2);
-	 		
+	 
 	 	z_err = z_est - data[k].z;
 	 	mean_error += z_err;
 	 	mean_squared_error += (z_err * z_err);
@@ -1417,15 +1407,15 @@ void	remove_planar_trend(void)
 	int	i;
 	double	a, b, c, d, xx, yy, zz;
 	double	sx, sy, sz, sxx, sxy, sxz, syy, syz;
-	
+
 	sx = sy = sz = sxx = sxy = sxz = syy = syz = 0.0;
-	
+
 	for (i = 0; i < npoints; i++) {
 
-		xx = (data[i].x - x_min) * r_xinc;
-		yy = (data[i].y - y_min) * r_yinc;
+		xx = (data[i].x - h.x_min) * r_xinc;
+		yy = (data[i].y - h.y_min) * r_yinc;
 		zz = data[i].z;
-		
+
 		sx += xx;
 		sy += yy;
 		sz += zz;
@@ -1435,14 +1425,14 @@ void	remove_planar_trend(void)
 		syy +=(yy * yy);
 		syz +=(yy * zz);
 	}
-	
+
 	d = npoints*sxx*syy + 2*sx*sy*sxy - npoints*sxy*sxy - sx*sx*syy - sy*sy*sxx;
-	
+
 	if (d == 0.0) {
 		plane_c0 = plane_c1 = plane_c2 = 0.0;
 		return;
 	}
-	
+
 	a = sz*sxx*syy + sx*sxy*syz + sy*sxy*sxz - sz*sxy*sxy - sx*sxz*syy - sy*syz*sxx;
 	b = npoints*sxz*syy + sz*sy*sxy + sy*sx*syz - npoints*sxy*syz - sz*sx*syy - sy*sy*sxz;
 	c = npoints*sxx*syz + sx*sy*sxz + sz*sx*sxy - npoints*sxy*sxz - sx*sx*syz - sz*sy*sxx;
@@ -1453,9 +1443,9 @@ void	remove_planar_trend(void)
 
 	for (i = 0; i < npoints; i++) {
 
-		xx = (data[i].x - x_min) * r_xinc;
-		yy = (data[i].y - y_min) * r_yinc;
-		
+		xx = (data[i].x - h.x_min) * r_xinc;
+		yy = (data[i].y - h.y_min) * r_yinc;
+
 		data[i].z -= (float)(plane_c0 + plane_c1 * xx + plane_c2 * yy);
 	}
 }
@@ -1479,18 +1469,18 @@ void	throw_away_unusables(void)
 		It assumes grid = 1 and set_grid_parameters has been
 		called.  We sort, mark redundant data as OUTSIDE, and
 		sort again, chopping off the excess.
-		
+
 		Experimental modification 5 Dec 1988 by Smith, as part
 		of a new implementation using core memory for b[6]
 		coefficients, eliminating calls to temp file.
 	*/
-	
+
 	int	last_index, n_outside, k;
-	
+
 	/* Sort the data  */
-	
+
 	qsort ((void *)data, (size_t)npoints, sizeof (struct DATA), compare_points);
-	
+
 	/* If more than one datum is indexed to same node, only the first should be kept.
 		Mark the additional ones as OUTSIDE
 	*/
@@ -1506,7 +1496,7 @@ void	throw_away_unusables(void)
 		}
 	}
 	/* Sort again; this time the OUTSIDE points will be thrown away  */
-	
+
 	qsort ((void *)data, (size_t)npoints, sizeof (struct DATA), compare_points);
 	npoints -= n_outside;
 	data = (struct DATA *) GMT_memory ((void *)data, (size_t)npoints, sizeof(struct DATA), GMT_program);
@@ -1522,11 +1512,11 @@ void	rescale_z_values(void)
 	double	ssz = 0.0;
 
 	for (i = 0; i < npoints; i++) ssz += (data[i].z * data[i].z);
-	
+
 	/* Set z_scale = rms(z):  */
-	
+
 	z_scale = sqrt (ssz / npoints);
-	
+
 	if (z_scale == 0.0) {
 		fprintf(stderr,"%s: WARNING: Input data lie exactly on a plane - no solution (aborting).\n", GMT_program);
 		r_z_scale = z_scale = 1.0;
@@ -1544,9 +1534,9 @@ void load_constraints (char *low, char *high)
 	int i, j, ij;
 	double yy;
 	struct GRD_HEADER hdr;
-		
+
 	/* Load lower/upper limits, verify range, deplane, and rescale */
-	
+
 	if (set_low > 0) {
 		lower = (float *) GMT_memory (VNULL, (size_t)(nx * ny), sizeof (float), GMT_program);
 		if (set_low < 3)
@@ -1572,7 +1562,7 @@ void load_constraints (char *low, char *high)
 			if (n_trimmed) fprintf (stderr, "%s: %d lower limit values > min data, reset to min data!\n", GMT_program, n_trimmed);
 */
 		}
-			
+
 		for (j = ij = 0; j < ny; j++) {
 			yy = ny - j - 1;
 			for (i = 0; i < nx; i++, ij++) {
@@ -1649,7 +1639,7 @@ double	guess_surface_time(int nx, int ny)
 
 	int	gcd;		/* Current value of the gcd  */
 	int	nxg, nyg;	/* Current value of the grid dimensions  */
-	int	nfactors;	/* Number of prime factors of current gcd  */
+	int	nfactors = 0;	/* Number of prime factors of current gcd  */
 	int	factor;		/* Currently used factor  */
 	/* Doubles are used below, even though the values will be integers,
 		because the multiplications might reach sizes of O(n**3)  */
@@ -1715,7 +1705,7 @@ void	suggest_sizes_for_surface(int nx, int ny)
 	int	n_sug = 0;	/* N of suggestions found  */
 	int	compare_sugs(const void *point_1, const void *point_2);	/* Sort suggestions decreasing  */
 	struct SUGGESTION *sug = NULL;
-	
+
 	users_time = guess_surface_time(nx, ny);
 	xstop = 2*nx;
 	ystop = 2*ny;
