@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: trend1d.c,v 1.15 2004/04/25 09:10:46 pwessel Exp $
+ *	$Id: trend1d.c,v 1.19 2005/12/17 05:59:22 pwessel Exp $
  *
- *	Copyright (c) 1991-2004 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2006 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -121,7 +121,7 @@ struct	DATA {
 }	*data;
 
 
-main(int argc, char **argv)
+int main(int argc, char **argv)
 {
 	void	read_data(struct DATA **data, int *n_data, double *xmin, double *xmax, int weighted_input, double **work, FILE *fp);
 	void write_output(struct DATA *data, int n_data, char *output_choice, int n_outputs), transform_x(struct DATA *data, int n_data, int model_type, double xmin, double xmax);
@@ -150,7 +150,7 @@ main(int argc, char **argv)
 	FILE	*fp = NULL;
 
 	argc = GMT_begin (argc, argv);
-	
+
 	model_type = POLYNOMIAL;
 	n_outputs = 0;
 	n_model_max = 0;
@@ -161,7 +161,7 @@ main(int argc, char **argv)
 			switch (argv[i][1]) {
 
 				/* Common parameters */
-			
+
 				case 'H':
 				case 'V':
 				case ':':
@@ -170,9 +170,9 @@ main(int argc, char **argv)
 				case '\0':
 					error += GMT_get_common_args (argv[i], 0, 0, 0, 0);
 					break;
-				
+
 				/* Supplemental parameters */
-				
+
 				case 'F':
 					j = 2;
 				       while (argv[i][j]) {
@@ -247,7 +247,7 @@ main(int argc, char **argv)
 		fprintf(stderr,"trend1d %s - Fit a [weighted] [robust] polynomial [or Fourier] model for y = f(x) to ascii xy[w]\n\n", GMT_VERSION);
 		fprintf(stderr,"usage:  trend1d -F<xymrw> -N[f]<n_model>[r] [<xy[w]file>] [-C<condition_#>]\n");
 		fprintf(stderr,"\t[-H[<nrec>]] [-I[<confidence>]] [-V] [-W] [-:] [-bi[s][<n>]] [-bo[s][<n>]] [-f[i|o]<colinfo>]\n\n");
-		
+
 		if (GMT_quick) exit (EXIT_FAILURE);
 
 		fprintf(stderr,"\t-F Choose at least 1, up to 5, any order, of xymrw for ascii output to stdout.\n");
@@ -277,15 +277,15 @@ main(int argc, char **argv)
 	if (c_no <= 1.0) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -C option.  Condition number must be larger than unity\n", GMT_program);
 		error++;
-	}	
+	}
 	if (confid < 0.0 || confid > 1.0) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -C option.  Give 0 < confidence level < 1.0\n", GMT_program);
 		error++;
-	}	
+	}
 	if (n_outputs > N_OUTPUT_CHOICES) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -F option.  Too many output columns specified (%d)\n", GMT_program, n_outputs);
 		error++;
-	}	
+	}
         if (n_outputs == 0) {
                 fprintf (stderr, "%s: GMT SYNTAX ERROR -F option.  Must specify at least one output columns \n", GMT_program);
                 error++;
@@ -293,7 +293,7 @@ main(int argc, char **argv)
 	if (n_model_max <= 0.0) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -N option.  A positive number of terms must be specified\n", GMT_program);
 		error++;
-	}	
+	}
 	if (GMT_io.binary[GMT_IN] && gmtdefs.io_header[GMT_IN]) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data cannot have header -H\n", GMT_program);
 		error++;
@@ -460,6 +460,8 @@ main(int argc, char **argv)
 	free_the_memory(gtg, v, gtd, lambda, workb, workz, c_model, o_model, w_model, data, work);
 
 	GMT_end (argc, argv);
+
+	exit (EXIT_SUCCESS);
 }
 
 void	read_data(struct DATA **data, int *n_data, double *xmin, double *xmax, int weighted_input, double **work, FILE *fp)
@@ -476,7 +478,7 @@ void	read_data(struct DATA **data, int *n_data, double *xmin, double *xmax, int
 #endif
 	}
 	(*data) = (struct DATA *) GMT_memory (VNULL, (size_t)n_alloc, sizeof(struct DATA), GMT_program);
-	
+
 	if (gmtdefs.io_header[GMT_IN]) for (i = 0; i < gmtdefs.n_header_recs; i++) GMT_fgets (buffer, BUFSIZ, fp);
 	i = 0;
 	n_expected_fields = (GMT_io.binary[GMT_IN]) ? GMT_io.ncol[GMT_IN] : 2 + weighted_input;
@@ -508,7 +510,7 @@ void	read_data(struct DATA **data, int *n_data, double *xmin, double *xmax, int
 		}
 	}
 	if (fp != GMT_stdin) GMT_fclose(fp);
-	
+
 	*data = (struct DATA *) GMT_memory ((void *)*data, (size_t)i, sizeof(struct DATA), GMT_program);
 	*work = (double *) GMT_memory (VNULL, (size_t)i, sizeof(double), GMT_program);
 
@@ -527,7 +529,7 @@ void	allocate_array_space(int np, double **gtg, double **v, double **gtd, double
 	*o_model = (double *) GMT_memory (VNULL, (size_t)np, sizeof(double), GMT_program);
 	*w_model = (double *) GMT_memory (VNULL, (size_t)np, sizeof(double), GMT_program);
 }
-	
+
 void	write_output(struct DATA *data, int n_data, char *output_choice, int n_outputs)
 {
 	int	i, j;
@@ -838,9 +840,9 @@ void GMT_cheb_to_pol (double c[], int n, double a, double b)
 	for (j = n - 1; j >= 1; j--) d[j] = d[j-1] - dd[j];
 	/* d[0] = -dd[0] + 0.5 * c[0]; */	/* This is what Num. Rec. says, but we do not do the approx with 0.5 * c[0] */
 	d[0] = -dd[0] + c[0];
-	
+
 	/* Next step is to undo the scaling so we can use coefficients with x */
-	
+
 	cnst = fac = 2.0 / (b - a);
 	for (j = 1; j < n; j++) {
 		d[j] *= fac;
@@ -848,11 +850,11 @@ void GMT_cheb_to_pol (double c[], int n, double a, double b)
 	}
 	cnst = 0.5 * (a + b);
 	for (j = 0; j <= n - 2; j++) for (k = n - 2; k >= j; k--) d[k] -= cnst * d[k+1];
-	
+
 	/* Return the new coefficients via c */
-	
+
 	memcpy ((void *)c, (void *)d, (size_t)(n * sizeof (double)));
-	
+
 	GMT_free ((void *) d);
 	GMT_free ((void *) dd);
 }
