@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: xyz2grd.c,v 1.19 2004/04/25 09:10:46 pwessel Exp $
+ *	$Id: xyz2grd.c,v 1.36 2006/02/21 00:53:51 remko Exp $
  *
- *	Copyright (c) 1991-2004 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2006 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -37,41 +37,43 @@
  
 #include "gmt.h"
 
-main (int argc, char **argv)
+int main (int argc, char **argv)
 {
-	BOOLEAN error = FALSE, got_input = FALSE, pixel = FALSE, z_only = FALSE, count = FALSE, average = TRUE;
-	BOOLEAN nodata_set = FALSE, b_only = FALSE, just_swap = FALSE, done = FALSE, nofile = TRUE;
+	BOOLEAN error = FALSE, got_input = FALSE, z_only = FALSE, count = FALSE, average = TRUE;
+	BOOLEAN nodata_set = FALSE, b_only = FALSE, just_swap = FALSE, done = FALSE, nofile = TRUE, ESRI = FALSE;
 
-	int i, ij, gmt_ij, ii, jj, z, nm, n_read = 0, n_filled = 0, n_used = 0, one_or_zero, fno, n_args, *flag;
+	int i, gmt_ij, ii, jj, z = 2, nm = 0, n_read = 0, n_filled = 0, n_used = 0, fno, n_args, *flag = VNULL;
+	int ij = -1;	/* Will be incremented to 0 or set first time around */
 	int n_empty = 0, n_stuffed = 0, n_bad = 0, n_expected_fields, n_fields, n_confused = 0, n_files = 0, high_low = 0;
-	
-	double w, e, s, n, dx = 0.0, dy = 0.0, *in, d_value, off, idx, idy, no_data_d;
+	int  e_set = FALSE;
+
+	double *in, d_value, no_data_d, wesn[4];
+
+	float no_data_f, ESRI_nan = 0.0, *a = VNULL;
 
-	float no_data_f, *a;
-	
 	char *grdfile, *zfile, line[BUFSIZ], input[BUFSIZ];
-	
+
 	FILE *fp = NULL, *fps = NULL;
-	
+
 	struct GRD_HEADER grd;
 
 	struct GMT_Z_IO r;
-	
+
 	argc = GMT_begin (argc, argv);
 
 	grdfile = zfile = CNULL;
 	input[0] = 0;
-	w = e = s = n = 0.0;
 	no_data_f = GMT_f_NaN;
 	no_data_d = GMT_d_NaN;
 
 	GMT_init_z_io (&r, TRUE);
-	
+	GMT_grd_init (&grd, argc, argv, FALSE);
+
 	for (i = 1; i < argc; i++) {
 		if (argv[i][0] == '-') {
 			switch (argv[i][1]) {
 				/* Common parameters */
-			
+
 				case 'b':
 					b_only = TRUE;
 				case 'H':
@@ -80,13 +82,14 @@ main (int argc, char **argv)
 				case ':':
 				case 'f':
 				case '\0':
-					error += GMT_get_common_args (argv[i], &w, &e, &s, &n);
+					error += GMT_get_common_args (argv[i], &grd.x_min, &grd.x_max, &grd.y_min, &grd.y_max);
 					break;
 
 				case 'A':
 					switch (argv[i][2]) {
 						case 'n':	/* Count the number of values for each node */
 							count = TRUE;
+							average = FALSE;
 							break;
 						case 'l':	/* Return the lowest (minimum) value at each node */
 							high_low = -1;
@@ -94,12 +97,12 @@ main (int argc, char **argv)
 						case 'u':	/* Return the upper (maximum)  value at each node */
 							high_low = +1;
 							break;
-						case 'z':	/* Return the mean value at each node */
+						case 'z':	/* Return the sum of multiple values at each node */
 						case '\0':
-							average = FALSE;
+							average = count = FALSE;
 							break;
 						default:
-							fprintf (stderr, "%s: GMT SYNTAX ERROR -A option:  Select -An, Al, Au or -A[z]\n", GMT_program);
+							fprintf (stderr, "%s: GMT SYNTAX ERROR -A option:  Select -An, -Al, -Au, or -A[z]\n", GMT_program);
 							error++;
 						break;
 					}
@@ -108,14 +111,24 @@ main (int argc, char **argv)
 					strcpy (input, &argv[i][2]);
 					got_input = TRUE;
 					break;
+				case 'E':
+					ESRI = TRUE;
+					if (argv[i][2]) {
+						ESRI_nan = (float)atof (&argv[i][2]);
+						e_set = TRUE;
+					}
+					break;
 				case 'F':
-					pixel = TRUE;
+					grd.node_offset = 1;
 					break;
 				case 'G':
 					grdfile = &argv[i][2];
 					break;
 				case 'I':
-					GMT_getinc (&argv[i][2], &dx, &dy);
+					if (GMT_getinc (&argv[i][2], &grd.x_inc, &grd.y_inc)) {
+						GMT_inc_syntax ('I', 1);
+						error = TRUE;
+					}
 					break;
 				case 'L':	/* Obsolete, but backward compatibility prevails [use -f instead] */
 					GMT_io.in_col_type[0] = GMT_io.out_col_type[0] = GMT_IS_LON;
@@ -150,26 +163,29 @@ main (int argc, char **argv)
 		else
 			n_files++;
 	}
-		
+
 	if (argc == 1 || GMT_quick) {
 		fprintf (stderr, "xyz2grd %s - Converting [xy]z data to a GMT grdfile\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: xyz2grd [<[xy]zfile(s)>] -G<grdfile> -I<dx[m|c]>[/<dy[m|c]>] -R<west/east/south/north>\n");
-		fprintf (stderr, "\t[-A[n|z|u|l]] [-D<xunit/yunit/zunit/scale/offset/title/remark>] [-F]\n");
+		fprintf (stderr, "\t[-A[n|z|u|l]] [-D<xname/yname/zname/scale/offset/title/remark>] [E[<nodata>]] [-F]\n");
 		fprintf (stderr, "\t[-H[<nrec>]] [-N<nodata>] [-S[<zfile]] [-V] [-Z[<flags>]] [-:]\n\t[-bi[s][<n>]] [-f[i|o]<colinfo>]\n");
-		
+
 		if (GMT_quick) exit (EXIT_FAILURE);
-		
+
 		fprintf (stderr, "\txyzfile(s) is a 1- or 3-column file [or standard input] with [x,y,]z values\n");
 		fprintf (stderr, "\t-G to name the output grdfile.\n");
 		fprintf (stderr, "\t-I specifies grid size(s).  Append m [or c] to <dx> and/or <dy> for minutes [or seconds]\n");
 		GMT_explain_option ('R');
 		fprintf (stderr, "\n\tOPTIONS:\n");
-		fprintf (stderr, "\t-A (or -Az): Add multiple entries at the same node.\n");
+		fprintf (stderr, "\t-A (or -Az): Sum up multiple entries at the same node.\n");
 		fprintf (stderr, "\t   Append n (-An): Count number of multiple entries per node instead.\n");
 		fprintf (stderr, "\t   Append u (-Au): Keep maximum value if multiple entries per node.\n");
 		fprintf (stderr, "\t   Append l (-Al): Keep minimum value if multiple entries per node.\n");
 		fprintf (stderr, "\t   [Default (no -A option) will compute mean values]\n");
 		fprintf (stderr, "\t-D to enter header information.  Specify '=' to get default value\n");
+		fprintf (stderr, "\t-E input is an ESRI ArcInfo interchange ASCII grid.  Optionally,\n");
+		fprintf (stderr, "\t   append the nodata value that represents NaNs [obtain from ESRI grid]\n");
+		fprintf (stderr, "\t   Information for -R , -I, and -F are obtained from the ESRI file.\n");
 		fprintf (stderr, "\t-F will force pixel registration [Default is grid registration]\n");
 		GMT_explain_option ('H');
 		fprintf (stderr, "\t-N set value for nodes without input xyz triplet [Default is NaN]\n");
@@ -222,24 +238,25 @@ main (int argc, char **argv)
 
 	if (b_only && z_only) GMT_io.binary[GMT_IN] = FALSE;
 
-	if (!just_swap) {
+	if (!(just_swap || ESRI)) {
 		if (!project_info.region_supplied) {
 			fprintf (stderr, "%s: GMT SYNTAX ERROR:  Must specify -R option\n", GMT_program);
 			error++;
 		}
-		if (dx <= 0.0 || dy <= 0.0) {
+		GMT_RI_prepare (&grd);	/* Ensure -R -I consistency and set nx, ny */
+		if (grd.x_inc <= 0.0 || grd.y_inc <= 0.0) {
 			fprintf (stderr, "%s: GMT SYNTAX ERROR -I option.  Must specify positive increment(s)\n", GMT_program);
 			error++;
 		}
-		if (!grdfile) {
-			fprintf (stderr, "%s: GMT SYNTAX ERROR option -G:  Must specify output file\n", GMT_program);
-			error++;
-		}
+	}
+	if (!just_swap && !grdfile) {
+		fprintf (stderr, "%s: GMT SYNTAX ERROR option -G:  Must specify output file\n", GMT_program);
+		error++;
 	}
 	if (just_swap && !z_only) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR option -S:  Must also specify -Z\n", GMT_program);
 		error++;
-	}	
+	}
 
 	if ((GMT_io.binary[GMT_IN] || r.binary) && gmtdefs.io_header[GMT_IN]) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data cannot have header -H\n", GMT_program);
@@ -250,7 +267,7 @@ main (int argc, char **argv)
 		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data (-bi) must have at least 3 columns\n", GMT_program);
 		error++;
 	}
-				
+
 	if (error) exit (EXIT_FAILURE);
 
 	GMT_put_history (argc, argv);	/* Update .gmtcommands4 */
@@ -270,52 +287,48 @@ main (int argc, char **argv)
 		fprintf (stderr, "%s: Warning: Single precision grid format is unlikely to hold absolute time coordinates without serious truncation.\n", GMT_program);
 		fprintf (stderr, "%s: Warning: You should select a grid format that allows for double precision elements.\n", GMT_program);
 	}
-		
-	GMT_grd_init (&grd, argc, argv, FALSE);
-	
-	/* if (!project_info.region) d_swap (s, e); */  /* Got w/s/e/n, make into w/e/s/n */
 
-	/* Decode grd information given, if any */
-	
+	/* Set units to "Langitude", "Latitude" or "Time" when coordinates are given as such
+	   Override them with whatever is given with the -D option */
+
+	switch (GMT_io.out_col_type[0]) {
+		case GMT_IS_LON:
+			strcpy (grd.x_units, "Longitude [degrees_east]"); break;
+		case GMT_IS_ABSTIME:
+		case GMT_IS_RELTIME:
+			strcpy (grd.x_units, "Time [seconds]"); break;
+	}
+	switch (GMT_io.out_col_type[1]) {
+		case GMT_IS_LAT:
+			strcpy (grd.y_units, "Latitude [degrees_north]"); break;
+		case GMT_IS_ABSTIME:
+		case GMT_IS_RELTIME:
+			strcpy (grd.y_units, "Time [seconds]"); break;
+	}
 	if (got_input) GMT_decode_grd_h_info (input, &grd);
-	
-	if (pixel) {
-		grd.node_offset = 1;
-		one_or_zero = 0;
-		off = 0.0;
-	}
-	else {
-		grd.node_offset = 0;
-		one_or_zero = 1;
-		off = 0.5;
-	}
 
 	if (just_swap)
 		r.swab = TRUE;
-	else {
-		grd.nx = irint ((e-w)/dx) + one_or_zero;
-		grd.ny = irint ((n-s)/dy) + one_or_zero;
-		grd.x_min = w;	grd.x_max = e;
-		grd.y_min = s;	grd.y_max = n;
-		grd.x_inc = dx;	grd.y_inc = dy;
-	
+	else if (!ESRI) {
+		grd.xy_off = 0.5 * grd.node_offset;
+
 		GMT_grd_RI_verify (&grd, 1);
-		
+
 		if (gmtdefs.verbose) fprintf (stderr, "%s: nx = %d  ny = %d\n", GMT_program, grd.nx, grd.ny);
-	
+
 		nm = grd.nx * grd.ny;
-	
+
 		a = (float *) GMT_memory (VNULL, (size_t)nm, sizeof (float), GMT_program);
 		flag = (int *) GMT_memory (VNULL, (size_t)nm, sizeof (int), GMT_program);
 
-		idx = 1.0 / dx;	idy = 1.0 / dy;
 		n_expected_fields = (z_only) ? 1 : 3;
 		z = (z_only) ? 0 : 2;
-		ij = -1;	/* Will be incremented to 0 or set first time around */
 	}
 
 	GMT_set_z_io (&r, &grd);
 
+	GMT_set_xy_domain (wesn, &grd);	/* May include some padding if gridline-registered */
+
 	if (n_files > 0)
 		nofile = FALSE;
 	else
@@ -324,7 +337,7 @@ main (int argc, char **argv)
 
 	for (fno = 1; !done && fno < n_args; fno++) {	/* Loop over input files, if any */
 		if (!nofile && argv[fno][0] == '-') continue;
-		
+
 		if (nofile) {	/* Just read standard input */
 			fp = GMT_stdin;
 			done = TRUE;
@@ -339,9 +352,9 @@ main (int argc, char **argv)
 		}
 
 		if (!nofile && gmtdefs.verbose) fprintf (stderr, "%s: Working on file %s\n", GMT_program, argv[fno]);
-		
+
 		if (gmtdefs.io_header[GMT_IN]) for (i = 0; i < gmtdefs.n_header_recs; i++) GMT_fgets (line, BUFSIZ, fp);
-	
+
 		if (just_swap) {	/* Just swap data and exit */
 
 			if (gmtdefs.verbose) fprintf (stderr, "%s: Swapping data bytes only\n", GMT_program);
@@ -365,7 +378,7 @@ main (int argc, char **argv)
 
 			exit (EXIT_SUCCESS);	/* We are done here */
 		}
-		
+
 		if (z_only) {	/* Read separately because of all the possible formats */
 			if (nodata_set && GMT_is_dnan (no_data_d)) nodata_set = FALSE;	/* No point testing */
 			if (r.skip) fseek (fp, (long)r.skip, SEEK_CUR);
@@ -386,11 +399,70 @@ main (int argc, char **argv)
 				fprintf (stderr, "%s: Found %d records, but %d was expected (aborting)!\n", GMT_program, ij, r.n_expected);
 				exit (EXIT_FAILURE);
 			}
-			
+
 			GMT_check_z_io (&r, a);	/* This fills in missing periodic row or column */
 		}
+		else if (ESRI) {	/* Read an ESRI Arc Interchange grid format in ASCII */
+			int n_left, j;
+			float value;
+			
+			if (fscanf (fp, "%*s %d", &grd.nx) != 1) {
+				fprintf (stderr, "%s: Error decoding ncols record\n", GMT_program);
+				exit (EXIT_FAILURE);
+			}
+			if (fscanf (fp, "%*s %d", &grd.ny) != 1) {
+				fprintf (stderr, "%s: Error decoding ncols record\n", GMT_program);
+				exit (EXIT_FAILURE);
+			}
+			if (fscanf (fp, "%s %lf", line, &grd.x_min) != 2) {
+				fprintf (stderr, "%s: Error decoding xll record\n", GMT_program);
+				exit (EXIT_FAILURE);
+			}
+			if (fscanf (fp, "%*s %lf", &grd.y_min) != 1) {
+				fprintf (stderr, "%s: Error decoding yll record\n", GMT_program);
+				exit (EXIT_FAILURE);
+			}
+			if (fscanf (fp, "%*s %lf", &grd.x_inc) != 1) {
+				fprintf (stderr, "%s: Error decoding cellsize record\n", GMT_program);
+				exit (EXIT_FAILURE);
+			}
+			if (!e_set && fscanf (fp, "%*s %f", &ESRI_nan) != 1) {	/* Only read if not specified in -E */
+				fprintf (stderr, "%s: Error decoding nodata_value record\n", GMT_program);
+				exit (EXIT_FAILURE);
+			}
+			grd.y_inc = grd.x_inc;
+			grd.node_offset = 0;
+			if (!strcmp (line, "xxlcenter")) {	/* Pixel grid */
+				grd.node_offset = 1;
+				grd.x_min -= 0.5 * grd.x_inc;
+				grd.y_min -= 0.5 * grd.y_inc;
+			}
+			grd.xy_off = 0.5 * grd.node_offset;
+			grd.x_max = grd.x_min + (grd.nx - 1) * grd.x_inc;
+			grd.y_max = grd.y_min + (grd.ny - 1) * grd.y_inc;
+			
+			GMT_grd_RI_verify (&grd, 1);
+
+			if (gmtdefs.verbose) fprintf (stderr, "%s: nx = %d  ny = %d\n", GMT_program, grd.nx, grd.ny);
+			nm = n_left = grd.nx * grd.ny;
+
+			a = (float *) GMT_memory (VNULL, (size_t)nm, sizeof (float), GMT_program);
+			i = 0;
+			j = grd.ny - 1;
+			while (fscanf (fp, "%f", &value) == 1 && n_left) {
+				ij = j * grd.nx + i;
+				a[ij] = (value == ESRI_nan) ? GMT_f_NaN : value;
+				i++;
+				if (i == grd.nx) i = 0, j--;
+				n_left--;
+			}
+			if (n_left) {
+				fprintf (stderr, "%s: Expected %d points, found only %d\n", GMT_program, nm, nm - n_left);
+				exit (EXIT_FAILURE);
+			}
+		}
 		else {	/* Get x, y, z */
-		
+
 			while ((n_fields = GMT_input (fp, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
 
 				if (GMT_io.status & GMT_IO_MISMATCH) {
@@ -398,15 +470,15 @@ main (int argc, char **argv)
 					exit (EXIT_FAILURE);
 				}
 				n_read++;
-		
-				if (in[1] < s || in[1] > n) continue;	/* Outside y-range */
-				if (in[0] < w || in[0] > e) continue;	/* Outside y-range */
+
+				if (GMT_y_is_outside (in[1],  wesn[2], wesn[3])) continue;	/* Outside y-range */
+				if (GMT_x_is_outside (&in[0], wesn[0], wesn[1])) continue;	/* Outside x-range */
 
 				/* Ok, we are inside the region - process data */
-			
-				ii = GMT_x_to_i (in[0], w, idx, off, grd.nx);
+
+				ii = GMT_x_to_i (in[0], grd.x_min, grd.x_inc, grd.xy_off, grd.nx);
 				if (ii == grd.nx) ii--, n_confused++;
-				jj = GMT_y_to_j (in[1], s, idy, off, grd.ny);
+				jj = GMT_y_to_j (in[1], grd.y_min, grd.y_inc, grd.xy_off, grd.ny);
 				if (jj == -1) jj++, n_confused++;
 				ij = jj * grd.nx + ii;
 				if (high_low) {	/* Always come here if looking for extreme values */
@@ -428,7 +500,38 @@ main (int argc, char **argv)
 		if (fp != GMT_stdin) GMT_fclose (fp);
 	}
 
-	if (!z_only) {	/* xyz data could have resulted in duplicates */
+	if (!(z_only || ESRI)) {	/* xyz data could have resulted in duplicates */
+		if (GMT_io.in_col_type[0] == GMT_IS_LON && (fabs (fabs (grd.x_max - grd.x_min) - 360.0) < GMT_CONV_LIMIT) && !grd.node_offset) {	/* Make sure longitudes got replicated */
+			int j, ij_west, ij_east;
+
+			for (j = 0; j < grd.ny; j++) {	/* For each row, look at west and east bin */
+				ij_west = j * grd.nx;
+				ij_east = ij_west + grd.nx - 1;
+				
+				if (flag[ij_west] && !flag[ij_east]) {		/* Nothing in east bin, just copy from west */
+					a[ij_east] = a[ij_west];
+					flag[ij_east] = flag[ij_west];
+				}
+				else if (flag[ij_east] && !flag[ij_west]) {	/* Nothing in west bin, just copy from east */
+					a[ij_west] = a[ij_east];
+					flag[ij_west] = flag[ij_east];
+				}
+				else {	/* Both have some stuff, consolidate combined value into the west bin, then replicate to the east */
+					if (high_low) {	/* Always come here if looking for extreme values */
+						if ((high_low == -1 && (a[ij_east] < a[ij_west])) || (high_low == +1 && (a[ij_east] > a[ij_west]))) a[ij_west] = a[ij_east];
+					}
+					else { 	/* Add up incase we must average */
+						a[ij_west] += a[ij_east];
+						flag[ij_west] += flag[ij_east];
+					}
+					/* Replicate: */
+					a[ij_east] = a[ij_west];
+					flag[ij_east] = flag[ij_west];
+				}
+
+			}
+		}
+
 		for (ij = 0; ij < nm; ij++) {	/* Check if all nodes got one value only */
 			if (flag[ij] == 1) {	/* This catches nodes with one value or the -Al|u single values */
 				if (count) a[ij] = 1.0;
@@ -443,34 +546,20 @@ main (int argc, char **argv)
 					a[ij] = (float)flag[ij];
 				else if (average)
 					a[ij] /= (float)flag[ij];
+				/* implicit else means return the sum of the values */
 				n_filled++;
 				n_stuffed++;
 			}
 		}
-		if (GMT_io.in_col_type[0] == GMT_IS_LON && (fabs (fabs (grd.x_max - grd.x_min) - 360.0) < GMT_CONV_LIMIT) && !grd.node_offset) {	/* Make sure longitudes got replicated */
-			int j, ij_left, ij_right;
-
-			for (j = 0; j < grd.ny; j++) {
-				ij_left = j * grd.nx;
-				ij_right = ij_left + grd.nx - 1;
-				if (flag[ij_left] && !flag[ij_right]) {
-					a[ij_right] = a[ij_left];
-					n_empty--;
-				}
-				else if (flag[ij_right] && !flag[ij_left]) {
-					a[ij_left] = a[ij_right];
-					n_empty--;
-				}
-			}
-		}
-
+		GMT_free ((void *)flag);
+		
 		if (gmtdefs.verbose) {
 			sprintf (line, "%s\n", gmtdefs.d_format);
 			fprintf (stderr, "%s:  n_read: %d  n_used: %d  n_filled: %d  n_empty: %d set to ", GMT_program,
 				n_read, n_used, n_filled, n_empty);
 			(GMT_is_dnan (no_data_d)) ? fprintf (stderr, "NaN\n") : fprintf (stderr, line, no_data_d);
 			if (n_bad) fprintf (stderr, "%s: %d records unreadable\n", GMT_program, n_bad);
-			if (n_stuffed) fprintf (stderr, "%s: Warning - %d nodes had multiple entries that were averaged\n", GMT_program, n_stuffed);
+			if (n_stuffed && !count) fprintf (stderr, "%s: Warning - %d nodes had multiple entries that were processed\n", GMT_program, n_stuffed);
 			if (n_confused) fprintf (stderr, "%s: Warning - %d values gave bad indices: Pixel vs gridline confusion?\n", GMT_program, n_confused);
 		}
 	}
@@ -479,9 +568,10 @@ main (int argc, char **argv)
 		fprintf (stderr, "%s: Error writing file %s\n", GMT_program, grdfile);
 		exit (EXIT_FAILURE);
 	}
-	
+
 	GMT_free ((void *)a);
-	GMT_free ((void *)flag);
-	
+
 	GMT_end (argc, argv);
+
+	exit (EXIT_SUCCESS);
 }
